Package: localScore
Type: Package
Title: Package for Sequence Analysis by Local Score
Version: 2.0.5
Date: 2026-04-20
Authors@R: c(person("Sebastian", "Simon", role = "aut"),
            person("Chris", "Verschelden", role = "aut"),
            person("Charly", "Marty", role = "aut"),
            person("David", "Robelin", role = c("aut","cre"),
                   email = "david.robelin@inrae.fr"),
            person("Sabine", "Mercier", role = "aut",
                   email = "sabine.mercier@univ-tlse2.fr"),
            person("Sebastien", "Dejean", role = "aut",
                   email="sebastien.dejean@math.univ-toulouse.fr"),
            person(given="The authors of Eigen the library for the included version of Eigen", role = "cph"))
Copyright: See the file COPYRIGHTS for various embedded Eigen library
        copyright details
Maintainer: David Robelin <david.robelin@inrae.fr>
Description: Functionalities for calculating the local score and calculating statistical relevance (p-value) to find a local Score in a sequence of given distribution (D. Robelin, S. Déjean, S. Mercier (2025) <doi:10.24072/pcjournal.650> ; S. Mercier and J.-J. Daudin (2001) <https://hal.science/hal-00714174/>) ; S. Karlin and S. Altschul (1990) <https://pmc.ncbi.nlm.nih.gov/articles/PMC53667/>  ; S. Mercier, D. Cellier and F. Charlot (2003) <https://hal.science/hal-00937529v1/> ; A. Lagnoux, S. Mercier and P. Valois (2017) <doi:10.1093/bioinformatics/btw699> ).
License: GPL (>= 2) | file LICENSE
Imports: Rcpp (>= 0.12.16), utils
LinkingTo: Rcpp
RoxygenNote: 7.3.3
LazyData: true
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
Depends: R (>= 3.2.0)
NeedsCompilation: yes
Packaged: 2026-04-21 16:29:44 UTC; drobelin
Author: Sebastian Simon [aut],
  Chris Verschelden [aut],
  Charly Marty [aut],
  David Robelin [aut, cre],
  Sabine Mercier [aut],
  Sebastien Dejean [aut],
  The authors of Eigen the library for the included version of Eigen
    [cph]
Repository: CRAN
Date/Publication: 2026-04-21 22:22:16 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 21:23:27 UTC; unix
Archs: localScore.so.dSYM

Package: localScore
Type: Package
Title: Package for Sequence Analysis by Local Score
Version: 2.0.5
Date: 2026-04-20
Authors@R: c(person("Sebastian", "Simon", role = "aut"),
            person("Chris", "Verschelden", role = "aut"),
            person("Charly", "Marty", role = "aut"),
            person("David", "Robelin", role = c("aut","cre"),
                   email = "david.robelin@inrae.fr"),
            person("Sabine", "Mercier", role = "aut",
                   email = "sabine.mercier@univ-tlse2.fr"),
            person("Sebastien", "Dejean", role = "aut",
                   email="sebastien.dejean@math.univ-toulouse.fr"),
            person(given="The authors of Eigen the library for the included version of Eigen", role = "cph"))
Copyright: See the file COPYRIGHTS for various embedded Eigen library
        copyright details
Maintainer: David Robelin <david.robelin@inrae.fr>
Description: Functionalities for calculating the local score and calculating statistical relevance (p-value) to find a local Score in a sequence of given distribution (D. Robelin, S. Déjean, S. Mercier (2025) <doi:10.24072/pcjournal.650> ; S. Mercier and J.-J. Daudin (2001) <https://hal.science/hal-00714174/>) ; S. Karlin and S. Altschul (1990) <https://pmc.ncbi.nlm.nih.gov/articles/PMC53667/>  ; S. Mercier, D. Cellier and F. Charlot (2003) <https://hal.science/hal-00937529v1/> ; A. Lagnoux, S. Mercier and P. Valois (2017) <doi:10.1093/bioinformatics/btw699> ).
License: GPL (>= 2) | file LICENSE
Imports: Rcpp (>= 0.12.16), utils
LinkingTo: Rcpp
RoxygenNote: 7.3.3
LazyData: true
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
Depends: R (>= 3.2.0)
NeedsCompilation: yes
Packaged: 2026-04-21 16:29:44 UTC; drobelin
Author: Sebastian Simon [aut],
  Chris Verschelden [aut],
  Charly Marty [aut],
  David Robelin [aut, cre],
  Sabine Mercier [aut],
  Sebastien Dejean [aut],
  The authors of Eigen the library for the included version of Eigen
    [cph]
Repository: CRAN
Date/Publication: 2026-04-21 22:22:16 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-21 22:36:44 UTC; unix
Archs: localScore.so.dSYM

# Generated by roxygen2: do not edit by hand

export(CharSequence2ScoreSequence)
export(CharSequences2ScoreSequences)
export(RealScores2IntegerScores)
export(automatic_analysis)
export(daudin)
export(exact_mc)
export(karlin)
export(karlinMonteCarlo)
export(karlinMonteCarlo_double)
export(karlin_parameters)
export(lindley)
export(loadMatrixFromFile)
export(loadScoreFromFile)
export(localScoreC)
export(localScoreC_double)
export(localScoreC_int)
export(maxPartialSumd)
export(mcc)
export(monteCarlo)
export(monteCarlo_double)
export(proba_theoretical_first_excursion_iid)
export(proba_theoretical_ith_excursion_iid)
export(proba_theoretical_ith_excursion_markov)
export(recordTimes)
export(scoreSequences2probabilityVector)
export(sequences2transmatrix)
export(stationary_distribution)
export(transmatrix2sequence)
importFrom(Rcpp,evalCpp)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(stats,density)
importFrom(stats,ecdf)
importFrom(stats,knots)
importFrom(stats,lm)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(localScore)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  warning = FALSE,
  collapse = TRUE,
  comment = "#>"
)

# Source : https://yihui.org/rmarkdown-cookbook/time-chunk
# all_times <- list()  # store the time for each chunk
# knitr::knit_hooks$set(time_it = local({
#   now <- NULL
#   function(before, options) {
#     if (before) {
#       # record the current time before each chunk
#       now <<- Sys.time()
#     } else {
#       # calculate the time difference after a chunk
#       res <- difftime(Sys.time(), now, units = "secs")
#       all_times[[options$label]] <<- res
#       # return a character string to show the time
#       paste("Time for the code chunk", options$label, " to run:", round(res,
#         2), "seconds")
#     }
#   }
# }))
# knitr::opts_chunk$set(time_it = FALSE)


## ----setup, echo=FALSE--------------------------------------------------------
library(localScore)

## -----------------------------------------------------------------------------
score.values <- -3:2                                       # Possible score values
score.probability <- c(0.4, 0.0, 0.1, 0.0, 0.2, 0.3)       # Associated score probabilities
score.expectation <- sum(score.values * score.probability) # Score expectation
print(score.expectation)
n <- 1000                                            # sequence size  
score.sequence <- sample(score.values, n, replace = TRUE, prob = score.probability)

## -----------------------------------------------------------------------------
score.lindley <- lindley(score.sequence)
oldpar <- par(no.readonly = TRUE)
par(mfrow = c(2,1))
plot(score.sequence, typ = 'l', main = "Sequence score")
plot(score.lindley, typ = 's', main = "Associated Lindley process")
par(oldpar)

## -----------------------------------------------------------------------------
segments.sequence <- localScoreC(score.sequence)
localScore.sequence <- segments.sequence$localScore # Local score and position of the segment
print(localScore.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores # suboptimal local scores and positions
print(head(subLocalScore.sequence))

## -----------------------------------------------------------------------------
# Exact p-value (computational limitation, see help(daudin))
daudin(localScore.sequence["value"], 
       sequence_length = n,
       score_probabilities = score.probability,
       sequence_min = min(score.values),
       sequence_max = max(score.values))
# Karlin and Dembo approximation (n big)
karlin(localScore.sequence["value"], 
       sequence_length = n,
       score_probabilities = score.probability,
       sequence_min = min(score.values),
       sequence_max = max(score.values))
# Improved Karlin and Dembo approximation (computational limitation, see help(mcc))
mcc(localScore.sequence["value"], 
       sequence_length = n,
       score_probabilities = score.probability,
       sequence_min = min(score.values),
       sequence_max = max(score.values))

## -----------------------------------------------------------------------------
transitionMatrix <- matrix(c(0.2, 0.3, 0.5,
                             0.3, 0.4, 0.3,
                             0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
score.values <- c(-3, -1, 2)
row.names(transitionMatrix) <- score.values
score.stationary.distribution <- stationary_distribution(transitionMatrix)
score.expectation <- sum(score.values * score.stationary.distribution)
print(score.expectation)
# Generating example markov sequence of score:
n <- 10000
score.sequence <- transmatrix2sequence(matrix = transitionMatrix,
                                       length = n,
                                       score = score.values)
head(score.sequence)

## -----------------------------------------------------------------------------
segments.sequence <- localScoreC(score.sequence)
localScore.sequence <- segments.sequence$localScore # Local score and position of the segment
print(localScore.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores # suboptimal local scores and positions
print(head(subLocalScore.sequence))

## -----------------------------------------------------------------------------
# Exact p-value (computational limitation, see help(exact_mc))
exact_mc(local_score = localScore.sequence["value"],
         m = transitionMatrix,
         sequence_length = n,
         prob0 = score.stationary.distribution)

## -----------------------------------------------------------------------------
# Some sort of drifting markov sequence generator
sequence.generator <- function(n, P1, P2, score_values) {
  nstate <- dim(P1)[1]
  sequence.sim <- rep(NA, n)
  sequence.sim[1] <- sample(1:nstate, 1, prob = stationary_distribution(P1))
  for (i in 2:n) {
    P <- (n - i) / (n - 1) * P1 + (i - 1) / (n - 1) * P2
    sequence.sim[i] <- sample(1:nstate, 1, prob = P[sequence.sim[i - 1],])
  }
  return(score_values[sequence.sim])
}

P1 <- matrix(c(0.2, 0.3, 0.5,
               0.3, 0.4, 0.3,
               0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
P2 <- matrix(c(0.2, 0.1, 0.7,
               0.6, 0.4, 0.0,
               0.8, 0.2, 0.2), byrow = TRUE, ncol = 3)
score.values <- c(-4, -1, 2)
n <- 500
score.sequence <- sequence.generator(n, P1, P2, score.values)
head(score.sequence)
mean(score.sequence) # Expected to be strictly negative

## -----------------------------------------------------------------------------
score.lindley <- lindley(score.sequence)
x <- 1:n
# lw1 <- loess(score.sequence ~ x)
oldpar <- par(no.readonly = TRUE)
par(mfrow = c(2,1))
plot(score.sequence, typ = 'l', col = "grey", main = "Sequence score")
# lines(x, lw1$fitted[x], col = "red")}}}
plot(score.lindley, typ = 's', main = "Associated Lindley process")
par(oldpar)

## -----------------------------------------------------------------------------
segments.sequence <- localScoreC(score.sequence)
localScore.sequence <- segments.sequence$localScore # Local score and position of the segment
print(localScore.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores # suboptimal local scores and positions
print(head(subLocalScore.sequence))

## -----------------------------------------------------------------------------
p.value <- monteCarlo(local_score = localScore.sequence["value"],
           FUN = function(n, P1, P2, score_values) {
             return(sequence.generator(n, P1, P2, score_values))
             },
           n = n, P1 = P1, P2 = P2, score_values = score.values,  #generative function parameters
           plot = TRUE,
           numSim = 1000 # low number here to compute the vignette in a "short" time
          )
print(p.value)

## -----------------------------------------------------------------------------
transitionMatrix <- matrix(c(0.2, 0.3, 0.5,
                             0.3, 0.4, 0.3,
                             0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
score.values <- c(-3, -1, 2)
row.names(transitionMatrix) <- score.values
score.stationary.distribution <- stationary_distribution(transitionMatrix)
score.expectation <- sum(score.values * score.stationary.distribution)
print(score.expectation)
# Generating example markov sequence of score:
n <- 10000
score.sequence <- transmatrix2sequence(matrix = transitionMatrix,
                                       length = n,
                                       score = score.values)

## -----------------------------------------------------------------------------
segments.sequence <- localScoreC(score.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores

## -----------------------------------------------------------------------------
iVisualExcursion <- 1 #first strictly positive excursion
iSegment <- subLocalScore.sequence[iVisualExcursion,]
print(iSegment)
# determining i in the sense of Karlin and Dembo (1990) (i >= iVisualExcursion)
i <- sum(segments.sequence$RecordTime <= iSegment$begin)
print(i)
proba_theoretical_ith_excursion_markov(iSegment$value, score.values, transitionMatrix, score.values, i = i)

## -----------------------------------------------------------------------------
mySeq <- sample(-2:2, 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
mySeq
scoreSequenceExpectation <- sum(c(-2:2)*c(0.5, 0.3, 0.05, 0.1, 0.05))
scoreSequenceExpectation
localScoreC(mySeq)


## -----------------------------------------------------------------------------
mySeq <- sample(c(-3,2,0,1,5), 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
head(mySeq)
localScoreC(mySeq)

## -----------------------------------------------------------------------------
seq1 <- c(1,-2,3,1,-1,2)
seq2 <- c(1,-2,3,1,-1,2,-1,1)
localScoreC(seq1)
localScoreC(seq2)

## -----------------------------------------------------------------------------
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score_reels, proba.sple = proba_score_reels,
                               length.sple = 10)
localScoreC(seq.essai) 

## -----------------------------------------------------------------------------
# Loading a fasta protein
data(Seq219)
Seq219
# or using your own fasta sequence
#MySeqAA_P49755 <- as.character(read.table(file="P49755.fasta",skip=1)[,1])
#MySeqAA_P49755
# Loading a scoring function
data(HydroScore)
?HydroScore
# or using your own scoring function
# HydroScoreKyte<- loadScoreFromFile("Kyte1982.txt")
# Transforming the amino acid sequence into a score sequence with the score function
# in HydroScore file
SeqScore_P49755 <- CharSequence2ScoreSequence(Seq219, HydroScore)
head(SeqScore_P49755)
length(SeqScore_P49755)
# Computing the local score
localScoreC(SeqScore_P49755)

## ----fig.width=7.2, fig.height=4----------------------------------------------
monteCarlo(local_score = 10, FUN = function(x) {
  return(sample(x = x, size = length(x), replace = TRUE))
}, x = SeqScore_P49755)

## -----------------------------------------------------------------------------
# Example
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
monteCarlo(5.5,
  FUN = sample_from_model, plot = TRUE, score.sple = score_reels, proba.sple = proba_score_reels,
  length.sple = 100, numSim = 1000
)

## ----fig.width=7.2, fig.height=4----------------------------------------------
fu <- function(n, size, prob, shift) {
  rbinom(size = size, n = n, prob = prob) + shift
}
# Note: (maybe) too small simulated_sequence_length value, to reduce vignette build time
# Look at the linear regression graph to adjust this parameter
karlinMonteCarlo(12,
  FUN = fu, n = 10000, size = 8, prob = 0.2, shift = -2,
  sequence_length = 1000000, simulated_sequence_length = 5000
) 

## ----fig.width=7.2, fig.height=4----------------------------------------------
score_reels <- c(-1.5, -0.5, 0, 0.5, 1.5)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
fu <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
# Note: (maybe) too small simulated_sequence_length value, to reduce vignette build time
# Look at the linear regression graph to adjust this parameter
karlinMonteCarlo(85.5,
  FUN = fu, score.sple = score_reels, proba.sple = proba_score_reels,
                 length.sple = 10000, numSim = 1000, sequence_length = 100000, 
  simulated_sequence_length = 5000
)

## -----------------------------------------------------------------------------
daudin(
  local_score = 15, sequence_length = 500, score_probabilities =
    c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1), sequence_min = -3, sequence_max = 2
)

## -----------------------------------------------------------------------------
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score_reels, proba.sple = proba_score_reels,
                               length.sple = 100)
localScoreC(seq.essai) 
C <- 10 # homothetic coefficient
localScoreC(as.integer(C*seq.essai))

## -----------------------------------------------------------------------------
RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)
M.s.r <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ExtendedIntegerScore
M.s.prob <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ProbExtendedIntegerScore
M.SL <- localScoreC(as.integer(C * seq.essai))$localScore[1]
M.SL
pval.E <- daudin(
  local_score = M.SL, sequence_length = length(seq.essai), score_probabilities = M.s.prob,
  sequence_min = -10, sequence_max = 10
)
pval.E

## -----------------------------------------------------------------------------
SL.real <- localScoreC(seq.essai)$localScore[1]
SL.real
pval.MC <- monteCarlo(
  local_score = SL.real, FUN = sample_from_model,
  score.sple = score_reels, proba.sple = proba_score_reels,
  length.sple = length(seq.essai), plot = TRUE, numSim = 5000
)
pval.MC

## -----------------------------------------------------------------------------
score.v <- -2:1
score.p <- c(0.3, 0.2, 0.2, 0.3)
sum(score.v*score.p)
karlin(
  local_score = 14, sequence_length = 100000, sequence_min = -2, sequence_max = 1,
  score_probabilities = c(0.3, 0.2, 0.2, 0.3)
)
karlin(
  local_score = 14, sequence_length = 1000, sequence_min = -2, sequence_max = 1,
  score_probabilities = c(0.3, 0.2, 0.2, 0.3)
)

## -----------------------------------------------------------------------------
# With missing score values
karlin(
  local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 1,
  score_probabilities = c(0.3, 0.2, 0.0, 0.2, 0.3)
)

## -----------------------------------------------------------------------------
mcc(
  local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 2,
  score_probabilities = c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1)
)

daudin(
  local_score = 14, sequence_length = 1000, score_probabilities =
    c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1), sequence_min = -3, sequence_max = 2
)
karlin(
  local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 2,
  score_probabilities = c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1)
)

## -----------------------------------------------------------------------------
automatic_analysis(sequences = list("x1" = c(1,-2,2,3,-2, 3, -3, -3, -2)), model = "iid")

## -----------------------------------------------------------------------------
score <- c(-2, -1, 0, 1, 2)
proba_score <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sum(score*proba_score)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score, proba.sple = proba_score, length.sple = 5000)
MyAnalysis <- automatic_analysis(
  sequences = list("x1" = seq.essai),
  distribution = proba_score, score_extremes = c(-2, 2), model = "iid"
)$x1
MyAnalysis$"p-value"
MyAnalysis$"method applied"
MyAnalysis$localScore$localScore

## -----------------------------------------------------------------------------
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score_reels, proba.sple = proba_score_reels, length.sple = 1000)

# Homothetie
C <- 10
RealScores2IntegerScores(score_reels,proba_score_reels, coef=C)
M.s.r <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ExtendedIntegerScore
M.s.prob <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ProbExtendedIntegerScore
# The analysis
MyAnalysis <- automatic_analysis(
  sequences = list("x1" = as.integer(C * seq.essai)), model = "iid",
  distribution = M.s.prob, score_extremes = range(M.s.r)
)
MyAnalysis$x1$"p-value"
MyAnalysis$x1$"method applied"

# Without the homothety, the function gives a wrong result
# MyAnalysis2 <- automatic_analysis(sequences = list("x1" = seq.essai), model = "iid")
# MyAnalysis2$x1$"p-value"
# MyAnalysis2$x1$"method applied"



## -----------------------------------------------------------------------------
scoreValues <- c(-2, -1, 2)
mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
initialProb <- stationary_distribution(mTransition)
exact_mc(local_score = 50, m = mTransition, sequence_length = 100, 
        score_values = scoreValues, prob0 = initialProb)

## -----------------------------------------------------------------------------
scoreValues <- c(-2, -1, 2)
mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
rownames(mTransition) <- scoreValues
initialProb <- stationary_distribution(mTransition)
exact_mc(local_score = 50, m = mTransition, sequence_length = 100)

## -----------------------------------------------------------------------------
MyTransMat <-
  matrix(c(
    0.3, 0.1, 0.1, 0.1, 0.4, 0.2, 0.2, 0.1, 0.2, 0.3, 0.3, 0.4, 0.1, 0.1, 0.1, 0.3, 0.3, 0.1, 0.0, 0.3,
    0.1, 0.1, 0.2, 0.3, 0.3
  ), ncol = 5, byrow = TRUE)

MySeq.CM <- transmatrix2sequence(matrix = MyTransMat, length = 150, score = -2:2)
MySeq.CM
AA.CM <- automatic_analysis(sequences = list("x1" = MySeq.CM), model = "markov")
AA.CM

## -----------------------------------------------------------------------------
Ls.CM <- AA.CM$x1$localScore[[1]][1]
monteCarlo(
  local_score = Ls.CM,
  FUN = transmatrix2sequence, matrix = MyTransMat,
  length = 150, score = -2:2,
  plot = FALSE, numSim = 5000
)

## ----fig.width=7.2------------------------------------------------------------
set.seed(1)
mySeq <- sample(c(-3,2,0,1,5), 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
lindley(mySeq)
oldpar <- par(no.readonly = TRUE)
par(mfrow = c(2,1))
plot(lindley(mySeq), type = "s")
plot(mySeq,typ = 'l')
par(oldpar)

## -----------------------------------------------------------------------------
set.seed(1)
mySeq <- sample(c(-3,2,0,1,5), 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
mySeq
recordTimes(mySeq)

## -----------------------------------------------------------------------------
seq1 <- sample(7:8, size = 10, replace = TRUE)
seq2 <- sample(2:3, size = 15, replace = TRUE)
l <- list(seq1, seq2)
r <- scoreSequences2probabilityVector(l)
r
length(r)

## -----------------------------------------------------------------------------
data(Seq219)
data(HydroScore)
SeqScore <- CharSequence2ScoreSequence(Seq219, HydroScore)
n <- length(SeqScore)
n

## -----------------------------------------------------------------------------
LS <- localScoreC(SeqScore)$localScore[1]
LS

## -----------------------------------------------------------------------------
prob <- scoreSequences2probabilityVector(list(SeqScore))
prob

## -----------------------------------------------------------------------------
time.daudin <- system.time(
  res.daudin <- daudin(
    local_score = LS, sequence_length = n,
       score_probabilities = prob,
       sequence_min = min(SeqScore),
    sequence_max = max(SeqScore)
  )
)
res.daudin

## -----------------------------------------------------------------------------
time.karlin <- system.time(
  res.karlin <- karlin(
    local_score = LS, sequence_length = n,
       score_probabilities = prob,
       sequence_min = min(SeqScore),
    sequence_max = max(SeqScore)
  )
)
res.karlin

## -----------------------------------------------------------------------------
time.mcc <- system.time(
  res.mcc <- mcc(
    local_score = LS, sequence_length = n,
       score_probabilities = prob,
       sequence_min = min(SeqScore),
    sequence_max = max(SeqScore)
  )
)
res.mcc

## -----------------------------------------------------------------------------
time.MonteCarlo1 <- system.time(
  res.MonteCarlo1 <- monteCarlo(
    local_score = LS,
    FUN = function(x) {
      return(sample(
        x = x, size = length(x),
        replace = TRUE
      ))
    },
    x = SeqScore, numSim = 200
  )
)
res.MonteCarlo1

## -----------------------------------------------------------------------------
time.MonteCarlo2 <- system.time(
  res.MonteCarlo2 <- monteCarlo(
    local_score = LS,
    FUN = function(x) {
      return(sample(
        x = x, size = length(x),
        replace = TRUE
      ))
    },
    x = SeqScore, numSim = 5000
  )
)
res.MonteCarlo2

## -----------------------------------------------------------------------------
res.pval <- c(
  Daudin = res.daudin, Karlin = res.karlin, MCC = res.mcc,
  MonteCarlo1 = res.MonteCarlo1, MonteCarlo1 = res.MonteCarlo2
)
names(res.pval) <- c("Exact", "Approximation", "Improved appx", "MonteCarlo1", "MonteCarlo2")
res.pval

## -----------------------------------------------------------------------------
rbind(time.daudin, time.karlin, time.mcc,time.MonteCarlo1, time.MonteCarlo2)

## -----------------------------------------------------------------------------
data(Seq31)
SeqScore.Short <- CharSequence2ScoreSequence(Seq31, HydroScore)
n.short <- length(SeqScore.Short)
n.short

## -----------------------------------------------------------------------------
SeqScore.S <- SeqScore.Short
LS.S <- localScoreC(SeqScore.S)$localScore[1]
prob.S <- scoreSequences2probabilityVector(list(SeqScore.S))

LS.S
prob.S

## -----------------------------------------------------------------------------
time.daudin <- system.time(
  res.daudin. <- daudin(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.S,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  )
)

time.karlin <- system.time(
  res.karlin <- try(karlin(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.S,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  ))
)

time.mcc <- system.time(
  res.mcc <- try(mcc(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.S,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  ))
)

# karlinMonteCarlo is meaningless for short sequence as its object is to avoid
# simulating very long sequences.
# time.karlinMonteCarlo <- system.time(
# res.karlinMonteCarlo <-
#     karlinMonteCarlo(
#       local_score = LS.S, plot = FALSE,
#                    sequence_length = n.short,
#                    simulated_sequence_length = 1000,
#                    FUN = sample, x = min(SeqScore.S):max(SeqScore.S),
#                    size = 1000, prob = prob.S, replace = TRUE,
#       numSim = 50000
#     )
# )

time.MonteCarlo <- system.time(
  res.MonteCarlo <- monteCarlo(
    local_score = LS.S, plot = FALSE,
    FUN = function(x) {
      return(sample(
        x = x, size = length(x),
        replace = TRUE
      ))
    },
    x = SeqScore.S, numSim = 5000
  )
)

## -----------------------------------------------------------------------------
res.pval <- c(Daudin = res.daudin, MonteCarlo = res.MonteCarlo)
names(res.pval) <- c("Daudin", "MonteCarlo")
res.pval
rbind(time.daudin, time.MonteCarlo)

## -----------------------------------------------------------------------------
set.seed(1)
prob.bis <- dnorm(-5:5, mean = -0.5, sd = 1)
prob.bis <- prob.bis / sum(prob.bis)
names(prob.bis) <- -5:5
# Score Expectation
sum((-5:5)*prob.bis)

time.mcc <- system.time(
  res.mcc <- mcc(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.bis,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  )
)

time.daudin <- system.time(
  res.daudin <- daudin(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.bis,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  )
)

simu <- function(n, p) {
  return(sample(x = -5:5, size = n, replace = TRUE, prob = p))
}
time.MonteCarlo <- system.time(
res.MonteCarlo <-
    monteCarlo(
      local_score = LS.S, plot = FALSE,
      FUN = simu, n.short, prob.bis, numSim = 5000
    )
)

## -----------------------------------------------------------------------------
res.pval <- c(MCC=res.mcc,Daudin = res.daudin, MonteCarlo = res.MonteCarlo)
names(res.pval) <- c("MCC", "Daudin", "MonteCarlo")
res.pval
rbind(time.mcc,time.daudin, time.MonteCarlo)

## -----------------------------------------------------------------------------
data(Seq1093)
SeqScore.Long <- CharSequence2ScoreSequence(Seq1093, HydroScore)
n.Long <- length(SeqScore.Long)
n.Long
SeqScore.Long <- CharSequence2ScoreSequence(Seq1093, HydroScore)
LS.L <- localScoreC(SeqScore.Long)$localScore[1]
LS.L
prob.L <- scoreSequences2probabilityVector(list(SeqScore.Long))
prob.L
sum(prob.L*as.numeric(names(prob.L))) 

## ----echo=FALSE---------------------------------------------------------------
time.daudin.L <- system.time(
  res.daudin.L <- daudin(
    local_score = LS.L, sequence_length = n.Long,
       score_probabilities = prob.L,
       sequence_min = min(SeqScore.Long),
    sequence_max = max(SeqScore.Long)
  )
)

time.karlin.L <- system.time(
  res.karlin.L <- karlin(
    local_score = LS.L, sequence_length = n.Long,
       score_probabilities = prob.L,
       sequence_min = min(SeqScore.Long),
    sequence_max = max(SeqScore.Long)
  )
)

time.mcc.L <- system.time(
  res.mcc.L <- mcc(
    local_score = LS.L, sequence_length = n.Long,
       score_probabilities = prob.L,
       sequence_min = min(SeqScore.Long),
    sequence_max = max(SeqScore.Long)
  )
)

time.MonteCarlo.L <- system.time(
  res.MonteCarlo.L <-
    monteCarlo(
      local_score = LS.L,
                   FUN = sample, x = min(SeqScore.Long):max(SeqScore.Long),
                   size = n.Long, prob = prob.L, replace = TRUE, plot = FALSE,
      numSim = 5000
    )
)

# karlinMonteCarlo is meaningless for short sequence as its object is to avoid
# simulating very long sequences.
# time.karlinMonteCarlo.L <- system.time(
#   res.karlinMonteCarlo.L <-
#     karlinMonteCarlo(
#       local_score = LS.L,
#       sequence_length = n.Long,
#       simulated_sequence_length = 1000,
#       FUN = sample, x = min(SeqScore.Long):max(SeqScore.Long),
#       size = 1000, prob = prob.L, replace = TRUE,
#       numSim = 10000, plot = FALSE
#     )
# )

## -----------------------------------------------------------------------------
res.pval.L <- c(res.daudin.L, res.mcc.L, res.karlin.L, NA,res.MonteCarlo.L)
names(res.pval.L) <- c("Daudin", "MCC", "Karlin", "KarlinMonteCarlo", "MonteCarlo")
res.pval.L

## -----------------------------------------------------------------------------
rbind(
  time.daudin.L, time.karlin.L, time.mcc.L, NA,
  time.MonteCarlo.L
)

## -----------------------------------------------------------------------------
MySeqsList <- list(Seq31, Seq219, Seq1093)
names(MySeqsList) <- c("Q09FU3.fasta", "P49755.fasta", "Q60519.fasta")
MySeqsScore <- lapply(MySeqsList, FUN = CharSequence2ScoreSequence, HydroScore)
AA <- automatic_analysis(MySeqsScore, model = "iid")
AA$Q09FU3.fasta
AA$Q09FU3.fasta$`method applied`
AA$Q60519.fasta$`method applied`

## -----------------------------------------------------------------------------
cbind(prob, prob.S, prob.L, "3 sequences" = scoreSequences2probabilityVector(MySeqsScore))

## -----------------------------------------------------------------------------
daudin.bis <- daudin(local_score = LS.S, sequence_length = n.short, score_probabilities = scoreSequences2probabilityVector(MySeqsScore), sequence_max = 5, sequence_min = -5)
daudin.bis
AA$Q09FU3.fasta$`p-value`
# automatic_analysis(sequences=list('MySeq.Short'=MySeq.Short), model='iid', distribution=proba.S)


## -----------------------------------------------------------------------------
data(HydroScore)
data(SeqListSCOPe)
MySeqScoreList <- lapply(SeqListSCOPe, FUN = CharSequence2ScoreSequence, HydroScore)
head(MySeqScoreList)
AA <- automatic_analysis(sequences = MySeqScoreList, model = "iid")
AA[[1]]
# the p-value of the first 10 sequences 
sapply(AA, function(x) {
  x$`p-value`
})[1:10]
# the 20th smallest p-values
sort(sapply(AA, function(x) {
  x$`p-value`
}))[1:20]
which(sapply(AA, function(x) {
  x$`p-value`
}) < 0.05)
table(sapply(AA, function(x) {
  x$`method`
}))
# The maximum sequence length equals 404 so it here normal that the exact method is used for all the 606 sequences of the data base 
scoreSequences2probabilityVector(MySeqScoreList)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  warning = FALSE,
  collapse = TRUE,
  comment = "#>"
)

# Source : https://yihui.org/rmarkdown-cookbook/time-chunk
# all_times <- list()  # store the time for each chunk
# knitr::knit_hooks$set(time_it = local({
#   now <- NULL
#   function(before, options) {
#     if (before) {
#       # record the current time before each chunk
#       now <<- Sys.time()
#     } else {
#       # calculate the time difference after a chunk
#       res <- difftime(Sys.time(), now, units = "secs")
#       all_times[[options$label]] <<- res
#       # return a character string to show the time
#       paste("Time for the code chunk", options$label, " to run:", round(res,
#         2), "seconds")
#     }
#   }
# }))
# knitr::opts_chunk$set(time_it = FALSE)


## ----setup, echo=FALSE--------------------------------------------------------
library(localScore)

## -----------------------------------------------------------------------------
seq <- c(-1,2,-3,-4,-1,3,-1,-2,3,-4,2)
lindley(seq)
plot(1:length(seq), lindley(seq), type = 'l')
localScoreC(seq)

## -----------------------------------------------------------------------------
recordTimes(seq)

## -----------------------------------------------------------------------------
proba_theoretical_ith_excursion_markov(a = 5,
                                       theta = c("A", "B"),
                                       lambda = matrix(c(0.5, 0.5, 0.8, 0.2),
                                                       ncol = 2, byrow = TRUE),
                                       score_function = c(-1,1),
                                       i = 2,
                                       epsilon = 1e-16,
                                       prob0 = c(0.5, 0.5))

## -----------------------------------------------------------------------------
proba_theoretical_ith_excursion_markov(a = 5,
                                       theta = c("K", 12, 1.54),
                                       lambda = matrix(c(0.9, 0, 0.1, 0.9, 0.1, 0, 0, 0.8, 0.2),
                                                       ncol = 3, byrow = TRUE),
                                       score_function = c(-3,-1,2),
                                       i = 4)

## -----------------------------------------------------------------------------
transition_matrix <- matrix(runif(400, min = 0, max = 1), nrow = 20)
transition_matrix <- t(apply(transition_matrix, 1, function(x) x/sum(x)))
theta <- letters[1:20]
score_f <- c(-3,-1,2,-3,-1,2,-3,-1,2,-1,
              -3,-1,2,-3,-1,2,-3,-1,2,-1)
sum(stationary_distribution(transition_matrix)*score_f) #score expectation (stationary)
system.time(pv1 <- proba_theoretical_ith_excursion_markov(a=5, theta, transition_matrix, score_f, i = 4))
pv1$proba_q_i_geq_a

## -----------------------------------------------------------------------------
library(localScore)
data("Seq1093")
Seq1093
nchar(Seq1093)
data(HydroScore)
LongSeqScore <- CharSequence2ScoreSequence(Seq1093, HydroScore)
table(LongSeqScore)
localScoreC(LongSeqScore)

head(sort(localScoreC(LongSeqScore)$suboptimalSegmentScores[,1], decreasing = TRUE))
plot(1:1093,lindley(LongSeqScore), type = 'l')
plot(1:400,lindley(LongSeqScore)[1:400], type = 'l')

## -----------------------------------------------------------------------------
LS <- localScoreC(LongSeqScore)$localScore["value"]
prob1 <- scoreSequences2probabilityVector(list(LongSeqScore))
prob1
daudin(local_score = LS, sequence_length = length(LongSeqScore),
       score_probabilities = prob1,
       sequence_min = min(LongSeqScore),
       sequence_max = max(LongSeqScore))

## -----------------------------------------------------------------------------
tmpMarkovParameters <- sequences2transmatrix(LongSeqScore)
lambda <- tmpMarkovParameters$transition_matrix
apply(lambda, 1, sum) #to check stochasticity
prob0 <- stationary_distribution(lambda)
print(prob0)
score_values <- tmpMarkovParameters$score_value
print(score_values)

## -----------------------------------------------------------------------------
exact_mc(LS, lambda, sequence_length = length(LongSeqScore), score_values = score_values)

## -----------------------------------------------------------------------------
subOptSegment <- localScoreC(LongSeqScore)$suboptimalSegmentScores
o <- order(subOptSegment[,1], decreasing = TRUE)
subOptSegment <- subOptSegment[o,] # reordering segments by decreasing score values
print(subOptSegment)

## -----------------------------------------------------------------------------
lindley(LongSeqScore)
recordTimes(LongSeqScore)
LongSeqScore[1] > 0

## -----------------------------------------------------------------------------
subOptSegment["1",] #First excursion
subOptSegment[as.character(dim(subOptSegment)[1]),] #Last excursion

## -----------------------------------------------------------------------------
theta <- letters[1:length(score_values)] # arbitrary
score_function <- score_values           # defined earlier 
a <- 40 
i <- 1
if (FALSE) { # To avoid long computation on CRAN servers... (Comment if you
                                                          # want to try)
  system.time(pv2 <- proba_theoretical_ith_excursion_markov(a, theta, lambda, 
                                          score_function,i)$proba_q_i_geq_a)
  } else {
  pv2 <- 0.1741784
  }
#> utilisateur système  écoulé
#> 38.061       0.033   38.137
pv2

a <- 6 
i <- 77
system.time(pv3<-proba_theoretical_ith_excursion_markov(a, theta, lambda, 
                                       score_function, i)$proba_q_i_geq_a)
pv3

## -----------------------------------------------------------------------------
a <- 6 
i <- 20
system.time(pv4 <- proba_theoretical_ith_excursion_markov(a, theta, 
                                       lambda, 
                                       score_function,i)$proba_q_i_geq_a)
pv4

i <-  10
system.time(pv5 <- proba_theoretical_ith_excursion_markov(a, theta, 
                                       lambda, 
                                       score_function,i)$proba_q_i_geq_a)
pv5

## -----------------------------------------------------------------------------
LongSeqScore.inv <- rev(LongSeqScore)
localScoreC(LongSeqScore.inv)
head(sort(localScoreC(LongSeqScore.inv)$suboptimalSegmentScores[,1], decreasing = TRUE))
plot(1:1093,lindley(LongSeqScore.inv), type = 'l')

## -----------------------------------------------------------------------------
markov_parameters <- sequences2transmatrix(LongSeqScore.inv)
lambda.inv <- markov_parameters$transition_matrix
if (FALSE) {
  system.time(pv5 <- proba_theoretical_ith_excursion_markov(a = 38, theta, 
                                                            lambda.inv, 
                                                            score_function, i = 96
  )$proba_q_i_geq_a)
} else {
  pv5 <- 0.005460216
}
#> utilisateur système écoulé
#> 35.478      0.046   35.573
pv5
proba_theoretical_ith_excursion_markov(a = 6, theta, lambda.inv,
                                       score_function,i = 1
                                       )$proba_q_i_geq_a

## -----------------------------------------------------------------------------
a <- 65 
i <- 30
if (FALSE) { # To avoid long computation on CRAN servers... (Comment if you
                                                          # want to try)
  system.time(pv6 <- proba_theoretical_ith_excursion_markov(a, theta, 
                                                            lambda, score_function, 
                                                            i)$proba_q_i_geq_a)
} else {
  pv6 <- 0.0004221151
}
#> utilisateur système écoulé
#> 104.861     0.101   105.062
pv6 

## -----------------------------------------------------------------------------
subOptSegment.inv <- localScoreC(LongSeqScore.inv)$suboptimalSegmentScores
which.max(subOptSegment.inv[,1])
print(subOptSegment.inv[16,])

a <- 65 
i <- 16
if (FALSE) {# To avoid long computation on CRAN servers... (Comment if you
                                                          # want to try)
  system.time(pv7 <- proba_theoretical_ith_excursion_markov(a, theta, lambda.inv, 
                                                            score_function,i)$proba_q_i_geq_a)
} else {
  pv7 <- 0.0004379929
}
#> utilisateur système écoulé
#> 103.018     0.088   103.229
pv7 

#' Short protein sequence
#'
#' A short protein sequence of 31 amino acids corresponding to Q09FU3.fasta query in UniProt Data base.
#'
#' @format A character string with 31 characters "MLTITSYFGFLLAALTITSVLFIGLNKIRLI"
#' @usage data(Seq31)
#' @source \url{https://www.uniprot.org/}
#' @examples
#' data(Seq31)
#' Seq31
#' nchar(Seq31)
#' data(HydroScore) 
#' SeqScore <- CharSequence2ScoreSequence(Seq31,HydroScore)
#' SeqScore
#' localScoreC(SeqScore)$localScore
#' LS <- localScoreC(SeqScore)$localScore[1]
#' prob1 <- scoreSequences2probabilityVector(list(SeqScore))
#' daudin(local_score = LS, sequence_length = nchar(Seq31),
#'                score_probabilities = prob1,
#'                sequence_min = min(SeqScore),
#'                sequence_max = max(SeqScore))
#' score <- -5:5
#' prob2 <- c(0.15,0.15,0.1,0.1,0.0,0.05,0.15,0.05,0.2,0.0,0.05)
#' sum(prob2*score)
#' karlin(local_score = LS, sequence_length = nchar(Seq31),
#'        score_probabilities = prob2,
#'        sequence_min = min(SeqScore),
#'        sequence_max = max(SeqScore))
"Seq31"
#' 
#' Protein sequence
#'
#' A protein sequence.
#'
#' @format A character string with 219 characters corresponding to P49755.fasta query in UniProt Data base.
#' @usage data(Seq219)
#' @source \url{https://www.uniprot.org/}
#' @examples
#' data(Seq219)
#' Seq219
#' nchar(Seq219)
#' data(HydroScore)
#' seqScore=CharSequence2ScoreSequence(Seq219,HydroScore)
#' seqScore[1:30]
#' localScoreC(seqScore)$localScore
#' prob1 <- scoreSequences2probabilityVector(list(seqScore))
#' daudin(local_score = 52, sequence_length = nchar(Seq219),
#'                score_probabilities = prob1,
#'                sequence_min = min(seqScore),
#'                sequence_max = max(seqScore))
#' score <- -5:5
#' prob2 <- c(0.15,0.15,0.1,0.1,0.0,0.05,0.15,0.05,0.2,0.0,0.05)
#' daudin(local_score = 52, sequence_length = nchar(Seq219),
#'        score_probabilities = prob2,
#'        sequence_min = min(seqScore),
#'        sequence_max = max(seqScore))
"Seq219"
#' 
#' Long protein sequence
#'
#' A long protein sequence.
#'
#' @format A character string with 1093 characters corresponding to Q60519.fasta in UniProt Data base.
#' @usage data(Seq1093)
#' @source \url{https://www.uniprot.org/}
#' @examples
#' data(Seq1093)
#' Seq1093
#' nchar(Seq1093)
#' data(HydroScore)
#' seqScore <- CharSequence2ScoreSequence(Seq1093,HydroScore)
#' seqScore[1:50]
#' localScoreC(seqScore)$localScore
#' LS <- localScoreC(seqScore)$localScore[1]
#' prob1 <- scoreSequences2probabilityVector(list(seqScore))
#' daudin(local_score = LS, sequence_length = nchar(Seq1093),
#'        score_probabilities = prob1,
#'        sequence_min = min(seqScore),
#'        sequence_max = max(seqScore))
#' karlin(local_score = LS, sequence_length = nchar(Seq1093),
#'        score_probabilities = prob1,
#'        sequence_min = min(seqScore),
#'        sequence_max = max(seqScore))
"Seq1093"
#' 
#' Dictionary
#'
#' Provides integer scores related to an hydrophobicity level of each amino acid. This score function is inspired by the Kyte and Doolittle (1982) scale.
#'
#' @format A score function for the 20 amino acid
#' @usage data(HydroScore)
#' @source Kyte & Doolittle (1982) J. Mol. Biol. 157, 105-132
#' @examples
#' data(HydroScore)
#' HydroScore
#' data(Seq219)
#' Seq219
#' seqScore <- CharSequence2ScoreSequence(Seq219,HydroScore)
#' seqScore[1:30]
#' localScoreC(seqScore)$localScore
"HydroScore"
#'
#' Several sequences
#'
#' A vector of character strings
#'
#' @format A list of 285 character strings with their entry codes as names
#' @usage data(SeqListSCOPe)
#' @source Structural Classification Of Proteins database (SCOP). More precisely this data contain the 285 protein sequences of the data called "CF_scop2dom_20140205aa" with length from 31 to 404.
#' @examples
#' data(SeqListSCOPe)
#' head(SeqListSCOPe)
#' SeqListSCOPe[1]
#' nchar(SeqListSCOPe[1])
#' summary(sapply(SeqListSCOPe, nchar))
#' data(HydroScore)
#' MySeqScoreList <- lapply(SeqListSCOPe, FUN=CharSequence2ScoreSequence, HydroScore)
#' head(MySeqScoreList)
#' AA <- automatic_analysis(sequences=MySeqScoreList, model='iid')
#' AA[[1]]
#' # the p-value of the first 10 sequences 
#' sapply(AA, function(x){x$`p-value`})[1:10]
#' # the 20th smallest p-values
#' sort(sapply(AA, function(x){x$`p-value`}))[1:20]
#' which(sapply(AA, function(x){x$`p-value`})<0.05)
#' table(sapply(AA, function(x){x$`method`}))
#' # The maximum sequence length equals 404 so it here normal that the exact method is used for
#' # all the 606 sequences of the data base 
#' # Score distribution learned on the data set
#' scoreSequences2probabilityVector(MySeqScoreList)
"SeqListSCOPe"
#'
#' Congenital oesophageal atresia data
#'
#' The data consists of individual dates of birth over n=35 cases of the birth defects oesophageal and tracheo-oesophagean fistula
#' observed in a hospital in Birmingham, U.K., over 2191 days from 1950 through 1955, with Day one set as 1 January 1950
#'
#' @format A matrix of 2191 lines and 2 columns. Each line is a day on the first column, and associated to a case (0/1) on the second column.
#' @usage data(Aeso)
#' @source Dolk H. Secular pattern of congenital oesophageal atresia--George Knox, 1959. J Epidemiol Community Health. 1997;51(2):114-115. <doi:10.1136/jech.51.2.114>
#' @examples
#' data(Aeso)
#' head(Aeso)
#' p <- sum(Aeso[,2]) / dim(Aeso)[1]
#' print(p)
"Aeso"
#'
#' Stevens-Johnson syndrome data
#'
#' The Stevens-Johnson syndrome is an acute and serious dermatological disease due to a drug allergy. 
#' The syndrome appearance is life-threatening emergency. They are very rare, around 2 cases per million people per year. 
#'
#' @format A data.frame of 824 lines, each describing a syndrome appearance described by 15 covariates:
#' Case ID,
#' Initial FDA Received Date,
#' days since last fda,
#' Event Date,
#' Latest FDA Received Date,
#' Suspect Product Names,
#' Suspect Product Active Ingredients,
#' Reason for Use,
#' Reactions,
#' Serious,
#' Outcomes,
#' Sex,
#' Patient Age,
#' Sender,
#' Concomitant Product Names
#' The third column correspond to the number of days between two adverse events.
#' @usage data(SJSyndrome)
#' @source FDA open data.
#' @examples
#' data(SJSyndrome)
#' summary(SJSyndrome)
"SJSyndrome"
#'
#' Deprecated
#' 
#' Some datasets are deprecated and will be removed in next version of the package.
#' Please use instead:
#' \itemize{
#'   \item data(aeso.data) -> data(Aeso)
#'   \item data(SJSyndrome.data) -> data(SJSyndrome)
#'   \item data(MySeqList) -> data(SeqListSCOPe)
#'   \item data(dico) -> data(HydroScore)
#'   \item data(LongSeq) -> data(Seq1093)
#'   \item data(MidSeq) -> data(Seq219)
#'   \item data(ShortSeq) -> data(Seq31)
#' }
#' @usage data(Aeso) ; data(SJSyndrome) ;
#'        data(SeqListSCOPe) ; data(HydroScore) ;
#'        data(Seq1093) ; data(Seq219) ; data(Seq31)
#' @format Refers to the "See also" links below to obtain a description of each dataset.
#' @aliases SJSyndrome.data MySeqList dico LongSeq MidSeq ShortSeq
#' @seealso \code{\link{Aeso}} \code{\link{SJSyndrome}} \code{\link{SeqListSCOPe}} 
#'          \code{\link{HydroScore}} \code{\link{Seq1093}} \code{\link{Seq219}} \code{\link{Seq31}}
#' @keywords internal
"aeso.data"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @description Calculates the exact p-value in the identically and independently distributed of a given local score, a sequence length that 'must not be too large' and for a given score distribution
#' @title Daudin [p-value] [iid]
#' @return A double representing the probability of a local score as high as the one given as argument. 
#' @param local_score the observed local score
#' @param sequence_length length of the sequence
#' @param score_probabilities the probabilities for each score from lowest to greatest (Optionnaly with scores as names)
#' @param sequence_min minimum score (optional if \code{score_values} OR names(score_probabilities) is defined)
#' @param sequence_max maximum score (optional if \code{score_values} OR names(score_probabilities) is defined)
#' @param score_values vector of integer score values, associated to score_probabilities  (optional if
#' \code{sequence_min} and \code{sequence_max} OR names(score_probabilities) are defined)
#' @details 
#' Either \code{sequence_min} and \code{sequence_max} are specified as input, OR all possible score values in 
#' \code{score_values} vector ; one of this choice is required. 
#' 
#' Small in this context depends heavily on your machine. 
#' On a 3,7GHZ machine this means for daudin(1000, 5000, c(0.2, 0.2, 0.2, 0.1, 0.2, 0.1), -2, 3)
#' an execution time of ~2 seconds. This is due to the calculation method using matrix exponentiation
#' which takes times. The size of the matrix of the exponentiation is equal to a+1 with a the
#' local score value. The matrix must be put at the power n, with n the sequence length.
#' Moreover, it is known that the local score value is expected to be in mean of order log(n).
#' @examples 
#' p1 <- daudin(local_score = 4, sequence_length = 50, 
#'        score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
#'        sequence_min = -3, sequence_max = 2)
#' p2 <- daudin(local_score = 4, sequence_length = 50, 
#'        score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
#'        score_values = as.integer(-3:2))
#' p1 == p2 # TRUE
#' 
#' prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
#' score_values <- which(prob != 0) - 6 # keep only non null probability scores
#' prob0 <- prob[prob != 0]             # and associated probability
#' p <- daudin(150, 10000, prob, sequence_min = -5, sequence_max =  5)
#' p0 <- daudin(150, 10000, prob0, score_values = score_values)
#' names(prob0) <- score_values
#' p1 <- daudin(150, 10000, prob0)
#' p == p0 # TRUE
#' p == p1 # TRUE
#' @export
daudin <- function(local_score, sequence_length, score_probabilities, sequence_min = NULL, sequence_max = NULL, score_values = NULL) {
    .Call('_localScore_daudin', PACKAGE = 'localScore', local_score, sequence_length, score_probabilities, sequence_min, sequence_max, score_values)
}

#' @description \code{karlin} Calculates an approximated p-value of a given local score value and a long sequence length in the identically and independently distributed model for the sequence. See also \code{\link{mcc}} function for another approximated method in the i.i.d. model that improved the one given by \code{\link{karlin}} or \code{\link{daudin}} for exact calculation. \cr
#' \code{karlin_parameters} is a annex function returning the parameters \eqn{\lambda}, \eqn{K^+} and \eqn{K^*} defined in Karlin and Dembo (1992).
#' @details This method works the better the longer the sequence is. Important note : the calculus of the parameter of the distribution uses
#' the resolution of a polynome which is a function of the score distribution, of order max(score)-min(score). There exists only empirical methods to solve a polynome of order greater that 5
#' with no warranty of reliable solution.
#' The found roots are checked internally to the function and an error message is throw in case of inconsistent. In such case, you could try to change your score scheme (in case of discretization)
#' or use the function \code{\link{karlinMonteCarlo}} .
#' This function implements the formulae given in Karlin and Dembo (1992), page 115-6. As the score is discrete here (lattice score function),
#' there is no limit distribution of the local score with the size of the sequence, but an inferior and a superior bound
#' are given. The output of this function is conservative as it gives the upper bound for the p-value.
#' Notice the lower bound can easily be found as it is the same call of function with parameter value local_score+1.
#' @title Karlin [p-value] [iid]
#' @return A double representing the probability of a local score as high as the one given as argument
#' @inheritParams daudin
#' @seealso \code{\link{mcc}}, \code{\link{daudin}}, \code{\link{karlinMonteCarlo}}, \code{\link{monteCarlo}}
#' @examples 
#' karlin(150, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -5, 5)
#' p1 <- karlin(local_score = 15, sequence_length = 5000, 
#'        score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
#'        sequence_min = -3, sequence_max = 2)
#' p2 <- karlin(local_score = 15, sequence_length = 5000, 
#'        score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
#'        score_values = -3:2)
#' p1 == p2 # TRUE
#' 
#' prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
#' score_values <- which(prob != 0) - 6 # keep only non null probability scores
#' prob0 <- prob[prob != 0]             # and associated probability
#' p <- karlin(150, 10000, prob, sequence_min = -5, sequence_max =  5)
#' p0 <- karlin(150, 10000, prob0, score_values = score_values) 
#' names(prob0) <- score_values
#' p1 <- karlin(150, 10000, prob0)
#' p == p0 # TRUE
#' p == p1 # TRUE
#' @export
karlin <- function(local_score, sequence_length, score_probabilities, sequence_min = NULL, sequence_max = NULL, score_values = NULL) {
    .Call('_localScore_karlin', PACKAGE = 'localScore', local_score, sequence_length, score_probabilities, sequence_min, sequence_max, score_values)
}

#' @rdname karlin
#' @export
karlin_parameters <- function(score_probabilities, sequence_min = NULL, sequence_max = NULL, score_values = NULL) {
    .Call('_localScore_karlin_parameters', PACKAGE = 'localScore', score_probabilities, sequence_min, sequence_max, score_values)
}

#' @description Calculates an approximated p-value for a given local score value and a medium to long sequence length in the identically and independently distributed model
#' @details This methods is actually an improved method of Karlin and produces more precise results. It should be privileged whenever possible. \cr
#' As with karlin, the method works the better the longer the sequence. Important note : the calculus of the parameter of the distribution uses
#' the resolution of a polynome which is a function of the score distribution, of order max(score)-min(score). There exists only empirical methods to solve a polynome of order greater that 5
#' with no warranty of reliable solution.
#' The found roots are checked internally to the function and an error message is throw in case of inconsistency. In such case, you could try to change your score scheme (in case of discretization)
#' or use the function \code{\link{karlinMonteCarlo}} .
#' @title MCC [p-value] [iid]
#' @return A double representing the probability of a local score as high as the one given as argument
#' @inheritParams daudin
#' @seealso \code{\link{karlin}}, \code{\link{daudin}}, \code{\link{karlinMonteCarlo}}, \code{\link{monteCarlo}}
#' @examples 
#' mcc(40, 100, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -6, 4)
#' mcc(40, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -6, 4)
#' mcc(150, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -5, 5)
#' p1 <- mcc(local_score = 15, sequence_length = 5000, 
#'        score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
#'        sequence_min = -3, sequence_max = 2)
#' p2 <- mcc(local_score = 15, sequence_length = 5000, 
#'        score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
#'        score_values = -3:2)
#' p1 == p2 # TRUE
#' 
#' prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
#' score_values <- which(prob != 0) - 6 # keep only non null probability scores
#' prob0 <- prob[prob != 0]             # and associated probability
#' p <- mcc(150, 10000, prob, sequence_min = -5, sequence_max =  5)
#' p0 <- mcc(150, 10000, prob0, score_values = score_values)
#' names(prob0) <- score_values
#' p1 <- mcc(150, 10000, prob0)
#' p == p0 # TRUE
#' p == p1 # TRUE
#' @export
mcc <- function(local_score, sequence_length, score_probabilities, sequence_min = NULL, sequence_max = NULL, score_values = NULL) {
    .Call('_localScore_mcc', PACKAGE = 'localScore', local_score, sequence_length, score_probabilities, sequence_min, sequence_max, score_values)
}

#' @description Calculates the distribution of the maximum of the partial sum process for a given value in the identically and independently distributed model
#' @details Implement the formula (4) of the article Mercier, S., Cellier, D., & Charlot, D. (2003). An improved approximation for assessing the statistical significance of molecular sequence features. Journal of Applied Probability, 40(2), 427-441. doi:10.1239/jap/1053003554 \cr
#' Important note : the calculus of the parameter of the distribution uses
#' the resolution of a polynome which is a function of the score distribution, of order max(score)-min(score). There exists only empirical methods to solve a polynome of order greater that 5
#' with no warranty of reliable solution.
#' The found roots are checked internally to the function and an error message is throw in case of inconsistency. 
#' @title Maximum of the partial sum [probability] [iid]
#' @return A double representing the probability of the maximum of the partial sum process equal to k
#' @param k value at which calculates the probability
#' @inheritParams daudin
#' @examples 
#' maxPartialSumd(10, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -6, 4)
#' prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
#' score_values <- which(prob != 0) - 7 # keep only non null probability scores
#' prob0 <- prob[prob != 0]             # and associated probability
#' p <- maxPartialSumd(10, prob, sequence_min = -6, sequence_max =  4)
#' p0 <- maxPartialSumd(10,prob0, score_values = score_values)
#' p == p0 # TRUE
#' @export
maxPartialSumd <- function(k, score_probabilities, sequence_min = NULL, sequence_max = NULL, score_values = NULL) {
    .Call('_localScore_maxPartialSumd', PACKAGE = 'localScore', k, score_probabilities, sequence_min, sequence_max, score_values)
}

#' @description Calculates stationary distribution of markov transition matrix by use of eigenvectors of length 1
#' @title Stationary distribution [Markov chains]
#' @return A vector with the probabilities
#' @param m Transition Matrix [matrix object]
#' @examples 
#' B = t(matrix (c(0.2, 0.8, 0.4, 0.6), nrow = 2))
#' stationary_distribution(B)
#' @export
stationary_distribution <- function(m) {
    .Call('_localScore_stationary_distribution', PACKAGE = 'localScore', m)
}

#' @description Calculates the exact p-value for short numerical Markov chains. Memory usage and time computation can be too large for a high local score value and high score range (see details).
#' @title Exact method for p-value [Markov chains]
#' @return A double representing the probability of a local score as high as the one given as argument
#' @param local_score Integer local score for which the p-value should be calculated
#' @param m Transition matrix [matrix object]. Optionality, rownames can be corresponding score values. m should be a transition matrix of an ergodic Markov chain.
#' @param sequence_length Length of the sequence
#' @param score_values A integer vector of sequence score values (optional). If not set, the rownames of m are used if they are numeric and set.
#' @param prob0 Vector of probability distribution of the first score of the sequence (optional). If not set, the stationnary distribution of m is used.
#' @details This method computation needs to allocate a square matrix of size local_score^(range(score_values)). This matrix is then exponentiated to sequence_length.
#' @seealso \code{\link{monteCarlo}}
#' @examples 
#' mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
#' scoreValues <- -1:1
#' initialProb <- stationary_distribution(mTransition)
#' exact_mc(local_score = 12, m = mTransition, sequence_length = 100, 
#'         score_values = scoreValues, prob0 = initialProb)
#' exact_mc(local_score = 150, m = mTransition, sequence_length = 1000, 
#'          score_values = scoreValues, prob0 = initialProb)
#' rownames(mTransition) <- scoreValues
#' exact_mc(local_score = 12, m = mTransition, sequence_length = 100, prob0 = initialProb)
#' # Minimal specification
#' exact_mc(local_score = 12, m = mTransition, sequence_length = 100)
#' # Score valeus with "holes"
#' scoreValues <- c(-2, -1, 2)
#' mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
#' initialProb <- stationary_distribution(mTransition)
#' exact_mc(local_score = 50, m = mTransition, sequence_length = 100, 
#'         score_values = scoreValues, prob0 = initialProb)
#' @export
exact_mc <- function(local_score, m, sequence_length, score_values = NULL, prob0 = NULL) {
    .Call('_localScore_exact_mc', PACKAGE = 'localScore', local_score, m, sequence_length, score_values, prob0)
}

#' @title Local score
#' @description Calculates the local score for a sequence of scores, the sub-optimal segments, and the associated record times. The local score is the maximal sum of values contained in a segment among all possible segments of the sequence. In other word, it generalizes a sliding window approach, considering of all possible windows size.
#' @param v a sequence of numerical values as vector (integer or double).
#' @param suppressWarnings (optional) if warnings should not be displayed
#' @return A list containing:
#' \item{localScore}{the local score value and the begin and end index of the segment realizing this optimal score;}
#' \item{suboptimalSegmentScores}{An array containing sub-optimal local scores, that is all the local maxima of the Lindley rocess (non negative excursion) and their begin and end index;}
#' \item{RecordTime}{The record times of the Lindley process as defined in Karlin and Dembo (1990).}
#' @details The \code{localScoreC} function is implemented in a templated C function. Be aware that the type of the output (\code{integer} or \code{double}) depends on the type of the input. The function \code{localScoreC_double} \code{localScoreC_int} explicitly use the corresponding type (with an eventual conversion in case of integer). Warning: in R, \code{typeof(c(1,3,4,10)) == "double"}. You can set a type of a vector with \code{mode()} or \code{as.integer()} functions for example. \cr
#' \code{localScoreC_int} is just a call to \code{as.integer()} before calling \code{localScoreC}. \code{localScore_double} is just a call to \code{localScoreC}, and as such is deprecated.
#' @seealso \code{\link{lindley}}
#' @examples 
#' localScoreC(c(1.2,-2.1,3.5,1.7,-1.1,2.3))
#' # one segment realizing the local score value
#' seq.OneSegment <- c(1,-2,3,1,-1,2)
#' localScoreC(seq.OneSegment) 
#' seq.TwoSegments <- c(1,-2,3,1,2,-2,-2,-1,1,-2,3,1,2,-1,-2,-2,-1,1)
#' # two segments realizing the local score value
#' localScoreC(seq.TwoSegments) 
#' # only the first realization
#' localScoreC(seq.TwoSegments)$localScore 
#' # all the realization of the local together with the suboptimal ones
#' localScoreC(seq.TwoSegments)$suboptimalSegmentScores 
#' # for small sequences, you can also use lindley() function to check if 
#' # several segments achieve the local Score
#' lindley(seq.TwoSegments) 
#' plot(1:length(seq.TwoSegments),lindley(seq.TwoSegments),type='b')
#' seq.TwoSegments.InSameExcursion <- c(1,-2,3,2,-1,0,1,-2,-2,-4,1)
#' localScoreC(seq.TwoSegments.InSameExcursion)
#' # lindley() shows two realizations in the same excursion (no 0 value between the two LS values)
#' lindley(seq.TwoSegments.InSameExcursion) 
#' # same beginning index but two possible ending indexes
#' # only one excursion realizes the local score even in there is two possible lengths of segment
#' localScoreC(seq.TwoSegments.InSameExcursion)$suboptimalSegmentScores 
#' plot(1:length(seq.TwoSegments.InSameExcursion),lindley(seq.TwoSegments.InSameExcursion),type='b')
#' # Technical note about type correspondance
#' seq.OneSegment <- c(1,-2,3,1,-1,2)
#' seq.OneSegmentI <- as.integer(seq.OneSegment)
#' typeof(seq.OneSegment)  # "double" (beware)
#' typeof(seq.OneSegmentI) # "integer"
#' LS1 <- localScoreC(seq.OneSegment, suppressWarnings = TRUE)
#' LS1I <- localScoreC(seq.OneSegmentI, suppressWarnings = TRUE)
#' typeof(LS1$localScore)  # "double"
#' typeof(LS1I$localScore) # "integer"
#' typeof(LS1$suboptimalSegmentScores$value)  # "double"
#' typeof(LS1I$suboptimalSegmentScores$value) # "integer"
#' # Force to use integer values (trunk values if necessary)
#' seq2 <- seq.OneSegment + 0.5
#' localScoreC(seq2, suppressWarnings = TRUE)
#' localScoreC_int(seq.OneSegment, suppressWarnings = TRUE)
#' @export
localScoreC <- function(v, suppressWarnings = FALSE) {
    .Call('_localScore_localScoreC', PACKAGE = 'localScore', v, suppressWarnings)
}

context("Test of stationary_distribution function for Markov models")

test_that("Stationary distribution", {
  
  # Following is a list of transition matrix and associated stationnary distribution with "no particularities"
  p_all = list(
    list(p = matrix(c(0.6, 0.1, 0.3,
                  0.1, 0.7, 0.2,
                  0.2, 0.2, 0.6), byrow=TRUE, ncol=3),
         mu = c(0.2758621, 0.3448276, 0.3793103)),
    list(p = matrix(c(0.0, 0.7, 0.3,
                  0.1, 0.7, 0.2,
                  0.2, 0.2, 0.6), byrow=TRUE, ncol=3),
         mu= c(0.1230769, 0.5230769, 0.3538462)),
    list(p = matrix(c(0.6, 0.1, 0.3,
                      0.1, 0.7, 0.2,
                      0.5, 0.5, 0.0), byrow=TRUE, ncol=3),
         mu = c(0.3571429, 0.4464286, 0.1964286)),
    list(p = matrix(c(0.6, 0.4, 0.0,
                      0.1, 0.7, 0.2,
                      0.0, 0.4, 0.6), byrow=TRUE, ncol=3),
         mu = c(0.1428571, 0.5714286, 0.2857143)),
    list(p = matrix(c(0.6, 0.4, 0.0,
                      0.3, 0.0, 0.7,
                      0.0, 0.5, 0.5), byrow=TRUE, ncol=3),
         mu = c(0.2380952, 0.3174603, 0.4444444)),
    list(p = matrix(c(0.0, 0.4, 0.6,
                      0.3, 0.0, 0.7,
                      0.5, 0.5, 0.0), byrow=TRUE, ncol=3),
         mu = c(0.2914798, 0.3139013, 0.3946188)),
    # next matrix : state '1' is not reacheable in finite time
    list(p = matrix(c(0.0, 0.0, 1.0,
                      0.0, 0.7, 0.3,
                      0.0, 0.5, 0.5), byrow=TRUE, ncol=3),
         mu = c(0.000, 0.625, 0.375))
  )
  
  #Next function calculate stationnary distribution in R code (without any checking and assuming a lot of unchecked hypothesis). 
  #It is better to not use it in test, but use the results hard-coded.
  cal_vp_R = function(ptrans) {
    dist_stat = eigen(t(ptrans), symmetric = FALSE)$vectors[,1]
    return(dist_stat/sum(dist_stat))
  }
  
  # Developper note : the '!!' is to print the value of 'i' in the output (useful to debug failed tests)
  for (i in 1:length(p_all)) {
    # Check that stationary distribution sums to 1.
    expect_equal(sum(stationary_distribution(p_all[[!!i]]$p)),1)
    # Check that stationary distribution is all positive valued (null included)
    expect_true(all(stationary_distribution(p_all[[!!i]]$p)>=0))
    # Check correct value of stationary distribution
    expect_equal(stationary_distribution(p_all[[!!i]]$p),p_all[[!!i]]$mu, tolerance = 1e-6, scale = 1)
  }

    # Check error on non stochastic matrix
  p1 = matrix(c(5.2, 1.0, 0.0,
                1.0, 0.0, 0.0,
                0.0, 0.0, 1.0), byrow=TRUE, ncol=3)
  expect_error(stationary_distribution(p1),"[ERROR] Transition probability matrix is not stochastic (row sum not equal 1.). Sum of line 1 equal 6.200000", fixed = TRUE)
  
  # Check error/warning on non irreductible Markov matrix (XXX TODO, or not)
  # p2 : cyclic
  p2 = matrix(c(0.0, 1.0, 0.0,
                0.0, 0.0, 1.0,
                1.0, 0.0, 0.0), byrow=TRUE, ncol=3)
  expect_equal(stationary_distribution(p2),c(0.3333333, 0.3333333, 0.3333333), tolerance = 1e-6, scale = 1)
  # p3 : absorbing state '2'
  p3 = matrix(c(0.2, 0.5, 0.3,
                0.0, 1.0, 0.0,
                0.1, 0.8, 0.1), byrow=TRUE, ncol=3)
  expect_equal(stationary_distribution(p3),c(0.0, 1.0, 0.0), tolerance = 1e-6, scale = 1)
})

context("Tests of exact_mc function (Markov model case)")

##### ERROR HANDLING ############
# Error link to #10215 ticket on forge DGA
test_that("Wrong matrix size error", {
  f <- c(-2,-1,0,1,2)
  min.f <- min(f)
  max.f <- max(f)
  nb_Alpha <- length(f)
  seq <- c(-2, -1, 1 , 1 , 1 , 1 )
  n <- length(seq)
  P <- matrix(ncol = 4, nrow = 4)
  P[1,] <- c(0.3, 0.3, 0.2, 0.2); P[2,] <- c(0.3, 0.3, 0.2, 0.2) ;
  P[3,] <- c(0.2, 0.3, 0.3, 0.2); P[4,] <- c(0.3, 0.2, 0.2, 0.3)
  LS <- 4
  expect_error(exact_mc(LS, P, sequence_length = n, score_values = f),
              "[ERROR exact_mc : Invalid Input] m should be a square matrix of size the length of score_values", fixed = TRUE)
})

############ 
test_that("Non ergodic Markov Chain", {
  #Liée à la demande #10408 sur la forge DGA.
  # Attention le code ci-desous est pour la tracabilité et n'est pas un vrai test (la valeur attendue ici n'a pas été vérifiée)
  min.f <- -2
  max.f <- 2
  f <- min.f:max.f
  nb_Alpha <- max.f - min.f + 1
  seq <- c(-2, -1, 1 , 1 , 1 , 1 )
  n <- length(seq)
  P <- matrix(ncol = nb_Alpha, nrow = nb_Alpha)
  P[1,] <- c(0.3, 0.3, 0.0, 0.2, 0.2)
  P[2,] <- c(0.3, 0.3, 0.0, 0.2, 0.2)
  P[3,] <- c(0.0, 0.0, 1.0, 0.0, 0.0)
  P[4,] <- c(0.2, 0.3, 0.0, 0.3, 0.2)
  P[5,] <- c(0.3, 0.2, 0.0, 0.2, 0.3)
  LS <- 4
  expect_error(exact_mc(LS, P, sequence_length = n, score_values = f), "Markov matrix is not irreductible (many eigenvalues == 1).", fixed = TRUE )
  ## Attention : valeur attendue ci-dessous non vérifiée théoriquement.
  expect_equal(exact_mc(LS, P, sequence_length = n, score_values = f, prob0 = c(1.0,0.0,0.0,0.0,0.0)), 0.30376)
})

test_that("Bad probability vector input", {
  mTransition <- t(matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), nrow = 3))
  scoreValues <- -1:1
  expect_error( exact_mc(local_score = 12, m = mTransition, sequence_length = 100, prob0 = -1:1),"[ERROR exact_mc : Invalid Input] prob0 vector should sum to 1",fixed = TRUE)
  expect_error( exact_mc(local_score = 12, m = mTransition, sequence_length = 100, prob0 = c(0,2,-1)),"[ERROR exact_mc : Invalid Input] prob0 vector should contains values between 0 and 1",fixed = TRUE)
})

test_that("prob0 specifed or not", {
  P0 <- matrix(ncol = 4, nrow = 4)
  P0[1,] <- c(0.3, 0.3, 0.2, 0.2)
  P0[2,] <- c(0.3, 0.3, 0.2, 0.2)
  P0[3,] <- c(0.2, 0.3, 0.3, 0.2)
  P0[4,] <- c(0.3, 0.2, 0.2, 0.3)
  seq <- c(-2, -1, 1 , 1 , 1 , 1 )
  n <- length(seq)
  LS <- 4
  mu <- stationary_distribution(P0)
  expect_equal(exact_mc(LS, P0, n, -2:1, mu),0.01976)
  expect_equal(exact_mc(LS, P0, n, -2:1), 0.01976)
})


test_that("Named transition matrix or not",{
  P0 <- matrix(ncol = 4, nrow = 4)
  P0[1,] <- c(0.3, 0.3, 0.2, 0.2)
  P0[2,] <- c(0.3, 0.3, 0.2, 0.2)
  P0[3,] <- c(0.2, 0.3, 0.3, 0.2)
  P0[4,] <- c(0.3, 0.2, 0.2, 0.3)
  seq <- c(-2, -1, 1 , 1 , 1 , 1 )
  n <- length(seq)
  LS <- 4
  mu <- stationary_distribution(P0)
  expect_error(exact_mc(LS, P0, n, prob0 = mu), "[ERROR exact_mc : Invalid Input] Either m is a matrix  with score values as rownames or specify score values via score_values parameter", fixed = TRUE)
  colnames(P0) <- -2:1
  expect_error(exact_mc(LS, P0, n, prob0 = mu), "[ERROR exact_mc : Invalid Input] Either m is a matrix  with score values as rownames or specify score values via score_values parameter", fixed = TRUE)
  rownames(P0) <- -2:1
  expect_equal(exact_mc(LS, P0, n, prob0 = mu), 0.01976)  
})

test_that("Minimal call",{
  P0 <- matrix(ncol = 4, nrow = 4)
  P0[1,] <- c(0.3, 0.3, 0.2, 0.2)
  P0[2,] <- c(0.3, 0.3, 0.2, 0.2)
  P0[3,] <- c(0.2, 0.3, 0.3, 0.2)
  P0[4,] <- c(0.3, 0.2, 0.2, 0.3)
  seq <- c(-2, -1, 1 , 1 , 1 , 1 )
  n <- length(seq)
  LS <- 4
  rownames(P0) <- -2:1
  expect_equal(exact_mc(LS, P0, n), 0.01976)  
})

test_that("Comparison exact_mc with daudin() in cas i.i.d", {
  probs <- c(0.3, 0.1, 0.2, 0.2, 0.1, 0.1)
  P0 <- matrix(ncol = 6, nrow = 6)
  P0[1,] <- probs
  P0[2,] <- probs
  P0[3,] <- probs
  P0[4,] <- probs
  P0[5,] <- probs
  P0[6,] <- probs
  scoremin <- -3
  scoremax <- 2
  rownames(P0) <- scoremin:scoremax
  n <- 100
  LS <- 5
  pEMC <- exact_mc(LS, P0, n)
  pDaudin <- daudin(LS, n , probs, scoremin, scoremax)
  expect_equal(pEMC, pDaudin)
})


POUBELLE <- function() {
  exact_mc3(LS, P0, n)
  exact_mc(P, LS, sequence_length = n, sequence_min = -2, sequence_max = 2)
  exact_mc(P0, LS, sequence_length = n, sequence_min = -2, sequence_max = 1)
  #double exact_mc2(int localScore, NumericMatrix m, NumericVector score_values, long sequence_length, NumericVector prob0){
  exact_mc2(LS, P0, -2:1, n, rep(0.25,4))
  exact_mc3(LS, P0, n,-2:1, rep(0.25,4))
  
  mu <- stationary_distribution(P0)
  exact_mc3(LS, P0, n, -2:1, mu)
  exact_mc3(LS, P0, n, -2:1)
  
  P0.1 <- P0 ; rownames(P0.1) <- -2:1
  exact_mc3(LS, P0.1, n)
  exact_mc3(LS, P0, n) #should give error
  tmp <- sapply(0:10,exact_mc3, P0.1, n)
  sum(-diff(tmp)) # should be 1 if 10 is enough
}

context("Test of R function karlin")

test_that("Example from the R documentation of karlin(.)", {
  p <- karlin(150, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -5, 5)
  kp <- karlin_parameters(c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -5, 5)
#  expect_equal(p, 0.0265141)
  expect_equal(p, 0.03464975)
  expect_equal(kp, c(K_star = 0.01823197, K_plus = 0.01930885, lambda = 0.05738689))
})

test_that("Entry probability does not sum to 1.0", {
  prob <- c(0.6,0.1,0.1,0.1) # sum to 0.9 (error)
  expect_error(karlin(150, 10000, prob, -2, 1), "[Invalid Input] score_probabilities must sum to 1.0.", fixed = TRUE)
  expect_error(karlin_parameters(prob, -2,1), "[Invalid Input] score_probabilities must sum to 1.0.", fixed = TRUE)
})

############## Link to ticket #11183 on forge DGA #######################
test_that("Example from the Karlin and Altschull 1990 Appendix", {
  # We should find lambda = ln(q/p) and K+ = (q-p)^2/q and Kstar = exp(-lambda*delta)*K+
  # We should also find that using Kstar instead of K+ is equivalent to call the function with parameter local_Score+1
  p <- 0.3 #P(X=1)
  q <- 0.4 #P(X=-1)
  r <- 1-p-q #P(X=0)
  seqlength <- 1000
  a <- 30
  pv <- karlin(a, seqlength, c(q,r,p), -1, 1)
  kparameters <- karlin_parameters(c(q,r,p), -1, 1)
  
  lambdaTh <- log(q/p)
  KplusTh <- ((q-p)^2)/q
  pvTh <- 1.0-exp(-KplusTh*seqlength*exp(-lambdaTh*a))
  expect_equal(unname(kparameters['lambda']), lambdaTh)
  expect_equal(pv, pvTh)
  expect_equal(unname(kparameters['K_plus']), KplusTh)
  
  delta <- 1 #span (for lattice case)
  KstarTh <- exp(-lambdaTh*delta)*KplusTh
  expect_equal(unname(kparameters['K_star']), KstarTh)
  expect_equal(
    karlin(a+1, seqlength, c(q,r,p), -1, 1) ,
    1.0-exp(-KstarTh*seqlength*exp(-lambdaTh*a))
  )
})


############## Link to ticket #10382 on forge DGA #######################
test_that("Leading zero probability element does not change the results", {
  prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
  prob0 <- c(0,prob)
  p <- karlin(150, 10000, prob, -5, 5)
  pk <- karlin_parameters(prob, -5, 5)
  p0 <- karlin(150, 10000, prob0, -5-1, 5)
  pk0 <- karlin_parameters(prob0, -5-1, 5)
  expect_equal(p, p0)
  expect_equal(pk, pk0)
  expect_equal(unname(pk0),c(0.01823197, 0.01930885, 0.05738689 ))
})

test_that("Leading multiple zero probability elements does not change the results", {
  prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
  prob0 <- c(0,0,0,0,prob)
  p <- karlin(150, 10000, prob, -5, 5)
  pk <- karlin_parameters(prob, -5, 5)
  p0 <- karlin(150, 10000, prob0, -5-4, 5)
  pk0 <- karlin_parameters(prob0, -5-4, 5)
  expect_equal(p, p0)
  expect_equal(pk, pk0)
  expect_equal(unname(pk0),c(0.01823197, 0.01930885, 0.05738689 ))
})

test_that("Trailing zero probability element does not change the results", {
  prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
  prob0 <- c(prob,0)
  p <- karlin(150, 10000, prob, -5, 5)
  p0 <- karlin(150, 10000, prob0, -5, 5+1)
  pk <- karlin_parameters(prob, -5, 5)
  pk0 <- karlin_parameters(prob0, -5, 5+1)
  expect_equal(p, p0)
  expect_equal(pk, pk0)
})

test_that("Trailing multiple zero probability elements does not change the results", {
  prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
  prob0 <- c(prob, 0,0,0,0)
  p <- karlin(150, 10000, prob, -5, 5)
  p0 <- karlin(150, 10000, prob0, -5, 5+4)
  pk <- karlin_parameters(prob, -5, 5)
  pk0 <- karlin_parameters(prob0, -5, 5+4)
  expect_equal(p, p0)
  expect_equal(pk, pk0)
})

test_that("Leading and trailing multiple zero probability elements does not change the results", {
  prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
  prob0 <- c(0,0,0,0,prob,0,0)
  p <- karlin(150, 10000, prob, -5, 5)
  p0 <- karlin(150, 10000, prob0, -5-4, 5+2)
  pk <- karlin_parameters(prob, -5, 5)
  pk0 <- karlin_parameters(prob0, -5-4, 5+2)
  expect_equal(pk, pk0)
  expect_equal(p, p0)
})
###################################################################################

test_that("Double each score does not change the results", {
  # En pratique, on multiplie par deux tous les scores ; théoriquement, on doit trouver la même p-value
  prob <- c(0.6, 0.1, 0.1, 0.2)
  scoremin <- -1
  scoremax <- scoremin + length(prob) -1 #2
  scoremin2 <- scoremin*2 #-2
  scoremax2 <- scoremax*2 #4
  prob0 <- c(0.6, 0.0, 0.1, 0.0, 0.1, 0.0, 0.2)
  p <- karlin(80+1, 10000, prob, scoremin, scoremax)
  p0 <- karlin(80*2+1, 10000, prob0, scoremin2, scoremax2)
  pk <- karlin_parameters(prob, scoremin, scoremax)
  pk0 <- karlin_parameters(prob0, scoremin2, scoremax2)
  expect_equal(p, p0)
  expect_equal(pk[c("K_star", "K_plus")], pk0[c("K_star", "K_plus")])
  expect_equal(0.5*pk[c("lambda")], pk0[c("lambda")])
})



# Below : this check is running fine except on M1Mac configuration (polynomial root finding issue which throw a controlled error)
# test_that("Example from the a pig dataset (unit score scheme)", {
#   score1.prob.ext = c(`-2` = 0.0681561425858712, `-1` = 0.759474076388722, `0` = 0.121671514073504, 
#                       `1` = 0.0310891041511429, `2` = 0.0125039059683196, `3` = 0.00563901641708531,
#                       `4` = 0.0010912674566738, `5` = 0.000242770954017739, `6` = 7.93212027978752e-05,
#                       `7` = 3.60550921808523e-05, `8` = 1.44220368723409e-05, `9` = 2.40367281205682e-06)
#   p <- karlin(248, 416030, score1.prob.ext, -2, 9)
#   expect_equal(p, 0.0)
# })

#' Functions parameters global description 
#' @param theta  alphabet used (vector of character)
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param a score strictly positive
#' @param prob0 Distribution of the first element of the Markov chain. Default to stationary distribution of lambda
#' @name MarkovParameters
NULL
#> NULL

#' @title compute all the descending trajectories of length c from (l-s)>=0 to l<0 with scores include in (Smin:-1), under condition that for every 
#' trajectory (t_1,...,t_c): ((l-s)+t_1+...+t_(c-1))>=0
#' some notations are the same as in the paper:"Exact distribution of excursion heights of the Lindley process in a Markovian model"
#' @param s -1*distance between starting point and arrival point of the trajectory (s<0)
#' @param c value in 1:(-s+1) represents the number of step of the trajectory (same notation as in the paper)
#' @param storage empty vector, used to store the result
#' @param l arrival value of the trajectory (l<0) (same notation as in the paper)
#' @param Smin lowest score possible to use in the trajectory
#' @param allowed allowed values for t_i
#' @return vector containing the trajectories
#' @examples
#' ######The following example should return an empty list
#' s=-4 #highest score=3
#' c=6
#' storage=rep(NA,c)
#' l=-1
#' Smin=-3
#' allowed = seq(-1, Smin, -1)
#' part(s,c,storage,l,Smin,allowed)
#' ######The following example should return this list: (-1,-1,-1)
#' s=-3 #highest score=2
#' c=3
#' storage=rep(NA,c)
#' l=-1
#' Smin=-2
#' allowed = seq(-1, Smin, -1)
#' part(s,c,storage,l,Smin,allowed)
#' ######The following example should return a list of 6 vectors
#' s=-5 #highest score =4
#' c=3
#' storage=rep(NA,c)
#' l=-1
#' Smin=-4
#' allowed = seq(-1, Smin, -1)
#' part(s,c,storage,l,Smin,allowed)
#' @noRd
part <- function(s, c, storage, l, Smin, allowed) {
  return_val = list()
  if ( c == 0 ) {
    if ( s == 0 ) {
      return_val[[length(return_val) + 1]] = storage
    }
  } else if ( c == 1 ) {
    if ( s %in% allowed ) {
      if ( s <= l) {
        storage[[length(storage)]] <- s
        return_val[[length(return_val) + 1]] = storage
      }
    }
  } else if ( (Smin * c) <= s & s <= (-c) ) {
    for (ele in allowed) {
      storage[(length(storage) - c) + 1] <- ele
      part_val = part((s-ele), (c-1), storage, l, Smin, allowed)
      for ( y in part_val) {
        return_val[[length(return_val) + 1]] = y
      }
    }
  }
  return(return_val)
}

#' @title compute all the descending trajectories from j to i with scores include in {Smin;-1}, 
#' under condition that for every trajectory (t_1,...,t_c): (j+t_1+...+t_(c-1))>=0
#' @param l arrival value (should be strictly negative)
#' @param j starting value (should be strictly positive)
#' @param Smin lowest possible score
#' @return list containing trajectories stored as vectors
#' @noRd
#' @examples
#' arrival_value=-4
#' starting_value=5
#' lowest_score=-10
#' traj(l=arrival_value,j=starting_value,Smin=lowest_score)
traj <- function(l, j, Smin) {
  trajectories = c()
  allowed = seq(-1, Smin, -1)
  for (c in 1:(j-l+1)) {
    trajectories <- c(trajectories, part((l-j), c, rep(NA, c), l, Smin, allowed))
  }
  return (trajectories)
}

#' 
#' #' @title Helper function used for the computation of all possible trajectories. Used in the computation of the matrix M
#' #' @param i value between -1 and the lowest score
#' #' @param j value between 1 and the highest score
#' #' @param m lowest score
#' #' @return all possibles trajectories
#' #' @examples
#' #' possible_tuples = lapply(min_val:(-1), function(ell) {
#' #' possible_hill_ending = lapply(1:max_val, function(j){
#' #'   traj(ell, j, min_val)
#' #' })
#' #' names(possible_hill_ending) = c(1:max_val)
#' #' return(possible_hill_ending)
#' #' })
#' traj <- function(i, j, m) {
#'   trajectories = lapply(traj_aux(i,j,m), function(i) c(i))
#'   return(trajectories)
#' }
#' 


#' @title Helper function to turn string back into list
#' @param str containing a letter of theta and a score separated by ":"
#' @return list containing a key (the letter) and a score
#' @examples
#' alpha <- "K:1"
#' str_to_alphabeta(alpha)
#' @noRd
str_to_alphabeta <- function(str) {
  tmp <- unlist(strsplit(str, split = ":", fixed = TRUE))
  return(list(key = tmp[1], score = as.integer(tmp[2])))
}

#' @title cell_value, compute the value of a cell for the matrix P
#' @param alpha letters used as indices of the row
#' @param beta  letters used as indices of the column
#' @param theta  alphabet used (vector of character)
#' @param a threshold score
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @return reel number
#' @examples
#' cell_value(alpha=list(key="L",score=4),
#'            beta=list(key="K",score=3),
#'            theta=c('K','L'),
#'            a=5,
#'            lambda=matrix(c(0.1, 0.9, 0.2, 0.8), ncol = 2, byrow = TRUE),
#'            score_function=c(4,-1))
#' @noRd
cell_value <- function(alpha, beta, theta, a, lambda, score_function) {
  if ( alpha$score == beta$score & ( alpha$score == -1 | alpha$score == as.double(a) ) ) { #XXX Ajouter condition alpha$key == beta$key ?
    return(1)
  } 
  
  index_alpha <- match(alpha$key, theta)
  index_beta  <- match(beta$key , theta)
  
  lambda_a_b <- lambda[index_alpha, index_beta]
  score <- score_function[index_alpha]
  
  if ( !(alpha$score >= 0 & alpha$score <= a-1) ) {
    return(0) 
  }
  
  if ((beta$score == -1) & (alpha$score + score < 0)) {
    return(lambda_a_b)
  } else if ((beta$score == a) & (a <= alpha$score + score) ) {
    return(lambda_a_b)
  } else if ((beta$score == alpha$score + score) & (alpha$score + score >= 0) & (alpha$score + score <= a - 1)) {
    return(lambda_a_b)
  } else {
    return(0)
  }
}


#' @title create_matrix_P, compute the transition probability matrix P of the stopped process Y, same notation as in the 
#' paper:"Exact distribution of excursion heights of the Lindley process in a Markovian model"
#' @param a threshold score
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix, same size and order as theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param combination vector of strings, each string containing a letter of theta and a score separated by ":"
#' @return a transition probability matrix, P
#' @examples
#' create_matrix_P(a=5,
#'                 theta=c("K","L"),
#'                 lambda=matrix(c(0.1, 0.9, 0.2, 0.8), ncol = 2, byrow = TRUE),
#'                 score_function=c(4,-1),
#'                 combination=c("K:-1","L:-1","K:0","L:0","K:1","L:1","K:2","L:2","K:3","L:3","K:4","L:4","K:5","L:5"))
#' @noRd
create_matrix_P <- function(a, theta, lambda, score_function, combination) {
  P <- matrix(nrow = length(combination), ncol = length(combination))
  dimnames(P) <- list(combination, combination)
  i <- 1
  j <- 1
  for (alpha in combination) {
    for (beta in combination) {
      decomp_alpha <- str_to_alphabeta(alpha)
      decomp_beta  <- str_to_alphabeta(beta)
      P[i, j] = cell_value(decomp_alpha, decomp_beta, theta, a, lambda, score_function)
      j <- j + 1
    }
    j <- 1
    i <- i + 1
  }
  P <- P[-(length(combination) - length(theta) + 1):-(length(combination)),]
  P <- P[-1:-length(theta),]
  return(P)
}


#' @title Probability P_alpha(Q(1)>=a) that the height of the first excursion >=a given the sequence begins with the letter alpha given a markovian model on the sequence.
#' @param a threshold score
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix, square, same size and order as theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param alpha first letter of the sequence
#' @return s: reel number between 0 and 1 representing the theoretical probability of reaching a score of a on the first excursion
#' #' @details : The first letter 'alpha' is not included in the score sequence. The conditioning is BEFORE the sequence.
#' Also beware that a sequence beginning with a negative score gives a "flat" excursion, with score 0 are considered.
#' It is also used in \code{\link{proba_theoretical_first_excursion()}}.
#' @examples
#' proba_theoretical_first_excursion_alpha(a=5,
#'                  theta=c("K","L","M"),
#'                  lambda=matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE),
#'                  score_function=c(-2,-1,2),
#'                  alpha="L")
#' @noRd
proba_theoretical_first_excursion_markov_alpha <- function(a, theta, lambda, score_function, alpha) {
  list.theta <- as.list(theta)
  list.a <- as.list(c(-1:a))
  combination <- as.vector(outer(list.theta, list.a, paste, sep = ":"))
  
  P <- create_matrix_P(a, theta, lambda, score_function, combination)
  
  Q_C <- P[,-(length(combination) - length(theta) + 1):-(length(combination))]
  Q_C <- Q_C[,-1:-length(theta)]
  Q_B <- P[,1:length(theta)]                                            
  Q_A <- P[,(length(combination) - length(theta) + 1):length(combination)]
  
  z <- (solve(diag(ncol(Q_C)) - Q_C)) %*% (Q_A %*% rep(1, ncol(Q_A)))
  s <- 0 
  
  index_alpha <- match(alpha, theta)
  
  for (delta in theta) {
    index_delta <- match(delta, theta)
    score <- score_function[index_delta]
    lambda_alpha_delta <- lambda[index_alpha, index_delta]
    
    if (0 <= score & score <= (a-1)) {
      for (mu in theta) {
        index_mu <- match(mu, theta)
        lambda_delta_mu <- lambda[index_delta, index_mu]  
        
        key <- paste(mu, score, sep = ":")
        index_z <- match(key, rownames(z))     
        s <- s + lambda_alpha_delta * lambda_delta_mu * z[index_z]
      }
    }
    if (score >= a) {
      s <- s + lambda_alpha_delta
    }
  }
  return( list( s = s, z = z, p=P) )
}

#' @title generalize_to_all_min, takes a list of trajectories to -1 and return all the trajectories with arrival value from -1 to Smin
#' @description the idea of this function is that the trajectories from s to -2 are the same as those from s to -1 if we remove 1 to the last element of each trajectory,
#' with the condition that the last element of each trajectory have to be >=Smin
#' Useful because it's faster than computing all trajectories with part() function (linear complexity vs exponential)
#' @param list_of_trajectories the list of trajectories from any starting point to -1 with scores included in (Smin:-1), same format as the return of traj() function
#' @param Smin the minimal score, trajectories will be compute with scores included in (Smin:-1)
#' @return a list of lists of vector, containing all the trajectories from the starting point of list_of_trajectories to all the arrival value included in (Smin:-1)
#' generalize_to_all_min[[i]][[j]] is the j-th trajectory from starting point to (-i)
#' @examples
#' ###The following example should return a list of 9 lists of trajectories
#' generalize_to_all_min(traj(-1,4,-10),-10)
#' ###The following example should return a list (of size one) of list (of size one) 
#' with this vector: (-1,-1,-1,-1,-1) generalize_to_all_min(traj(-1,4,-1),-1)
#' @noRd
generalize_to_all_min <- function(list_of_trajectories, Smin)  {
  generalized_list = list()
  i = 0
  for (j in 1:(-Smin)){
    k=1
    generalized_list[[j]] = list()
    generalized_list[[j]][k][[1]] = rep(NA, length(list_of_trajectories[i+1][[1]]))
    for (i in 1:length(list_of_trajectories)){
      if(list_of_trajectories[i][[1]][length(list_of_trajectories[i][[1]])]-(j-1)>=(Smin)){
        generalized_list[[j]][k][[1]] = c(list_of_trajectories[i][[1]][1:length(list_of_trajectories[i][[1]])-1], list_of_trajectories[i][[1]][length(list_of_trajectories[i][[1]])]-(j-1))
        k = k + 1
      }
    }
  }
  return (generalized_list)
}

#' @title Probability \eqn{P(Q(1)\geq a)} that the height of the first excursion is greater or equal to \code{a} given a Markov model on the letters sequence
#' @description Mathematical definition of an excursion of the Lindley process is based on the record times of the partial
#' sum sequence associated to the score sequence (see Karlin and Altschul 1990, Karlin and Dembo 1992) and
#' define the successive times where the partial sums are strictly decreasing. There must be distinguished
#' from the visual excursions of the Lindley sequence. Detailed definitions are given in the documentation.
#' @param a score strictly positive
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param prob0 Distribution of the first letter of the sequence (must be in the same order as theta). If NULL, stationary distribution of lambda is calculated and applied.
#' @return theoretical probability of reaching a score of a on the first excursion given the first letter of the sequence follow the distribution given by parameter 'prob0"
#' @details Beware that a sequence beginning with a negative score gives a "flat" excursion, with score 0, are considered.
#' @seealso \code{\link{proba_theoretical_ith_excursion_markov()}}
#' @examples
#' theta = c("a","b","c","d")
#' lambda = matrix(c(0.1,0.2,0.4,0.3,
#'                   0.3,0.1,0.5,0.1,
#'                   0.2,0.6,0.1,0.1,
#'                   0.4,0.1,0.1,0.4),
#'                 ncol=4, byrow = TRUE, dimnames = list(theta,theta))
#' all(apply(lambda,1,sum)==1) #TRUE for markov transition matrix
#' stationary_dist = stationary_distribution(lambda)
#' score_function = c(a=-3,b=-1,c=1,d=2)
#' sum(score_function*stationary_dist) #Score expectation (should be <0)
#' prob0 = c(.25,.25,.25,.25) #
#' proba_theoretical_first_excursion_markov(3, theta, lambda, score_function, prob0)
#' proba_theoretical_ith_excursion_markov(3, theta, lambda, score_function, i=1, prob0) #for comparison
#' @noRd
proba_theoretical_first_excursion_markov <- function(a, theta, lambda, score_function, prob0=NULL) {
  # RM théoriquement plus utilisée
  if (is.null(prob0)){
    prob0 = stationary_distribution(lambda)
  }

  if (a <= 0) {
    return(1.0)
  }
  
  prob = 0.0
  for (i in 1:length(theta)) {
    if (prob0[i] != 0.0) {
      p <- proba_theoretical_first_excursion_markov_alpha(a, theta, lambda, score_function, theta[i])
      prob  <- prob + prob0[i]*p$s
    }
  }
  return(prob)
}


#' @title Transition matrix of the start of excursions, used for function computing probability of the i-th excursion, same notation as in the 
#' paper:"Exact distribution of excursion heights of the Lindley process in a Markovian model"
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix, square, same size and order as theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param N_iter maximal number of iteration (recommended : 1000)
#' @param epsilon threshold of the difference between matrices of two following iteration, this can reduce the number of iterations
#' @return transition probability matrix, square, same size and order as theta
#' @examples
#' matrix_M(c("K","L","M"), matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), c(-2,-1,2), 1000)
#' @noRd
matrix_M <- function(theta, lambda, score_function, N_iter, epsilon = 1e-16) {
  min_val <- min(score_function)
  max_val <- max(score_function)
  positive_scores <- score_function[score_function > 0]
  possible_tuples <- lapply(1:max_val, function(max) if (max %in% positive_scores) {generalize_to_all_min(traj(-1, max, min_val), min_val)})
  lambda_matrices <- lapply(min_val:max_val, function(j) {
    intermediary_lambda_matrix <- matrix(0, length(theta), length(theta))
    if ( j %in% score_function) {
      indexes = which(score_function %in% j)
      for ( ind in indexes ) {
        intermediary_lambda_matrix[, ind] <- lambda[, ind]
      }
    }
    return( intermediary_lambda_matrix )
  })
  names(lambda_matrices) <- c(min_val:max_val)
  M_iter_aux <- lambda_matrices[as.character(min_val:-1)]
  names(M_iter_aux) <- as.character(min_val:(-1))
  M_iter <- lapply(min_val:(-1), function(i) {
    return(matrix(0, length(theta), length(theta)))
  })
  names(M_iter) <- c(min_val:(-1))
  i = 1
  norme = rep(1, abs(min_val))
  eps = rep(epsilon, abs(min_val))
  while((i < N_iter) &! ((i > 10) & (min(norme < eps)))) {
    for ( ell in min_val:(-1) ) {
      ell_char = as.character(ell)
      zero_char = as.character(0)
      M_iter[[ell_char]] = lambda_matrices[[ell_char]] + lambda_matrices[[zero_char]] %*% M_iter_aux[[ell_char]]
      for ( j in unique(positive_scores) ) {
        j_char = as.character(j)
        for ( tuple in possible_tuples[[j]][[(-ell)]] ) {
          matr_prod = lambda_matrices[[j_char]]
          for ( step in tuple ) {
            matr_prod <- matr_prod %*% M_iter_aux[[as.character(step)]]
          }
          M_iter[[ell_char]] <- M_iter[[ell_char]] + matr_prod
        }
      }
    }
    for ( ell in min_val:(-1) ) {
      ell_char = as.character(ell)
      norme[[-ell]] = max(abs(M_iter_aux[[ell_char]] - M_iter[[ell_char]]))
      M_iter_aux[[ell_char]] = M_iter[[ell_char]]
    }
    i = i + 1
  }
  if (i == N_iter & !min(norme < eps)){
    warning("The computation of the transition probability matrix M has stopped without convergence. It reached the maximal number of iterations and the norm is not < epsilon.")
  }
  M <- matrix(0, length(theta), length(theta))
  for (ell_char in names(M_iter)) {
    M <- M + M_iter[[ell_char]]
  }
  return ( M )
}


#' @title theoretical probability of reaching the threshold score a on the i-th excursion (sequential order) of a markov's score sequence
#' @description
#' This function implements the results of the paper: "Exact distribution of excursion heights of the Lindley process in a Markovian model", this is an
#' exact method. 
#' Scores of score function have to be integers, the expectancy of the score have to be negative, the score function have to contains at least
#' one positive integer, i have to be >0
#' @details
#' Be careful: the computational time is exponential in function of the maximum score of the score function.
#' The computational time also increase with the rise of the threshold score a and with the lowering of the minimum of the score function and have a cubic complexity in function of the size of theta.
#' Lowering epsilon can also decrease the execution time of the function but it can have an impact on the accuracy of the probabilities.
#' @param a threshold score
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of the markov chain, square, same size and order as theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param i number of wanted excursion (sequential order)
#' @param prob0 probability distribution of the first letter of the sequence (stationary distribution of lambda if the parameter is NULL),
#' used only for computation of proba_q_i_geq_a
#' @param epsilon threshold for the computation of matrix M: to calculate the probabilities, this function need to compute the matrix M (transition probability matrix of the beginning of excursions, as in the paper)
#' with a recurrence, this recurrence stop at a certain number of iterations or when the maximal absolute value of the difference between an iteration and the next one is < epsilon.
#' @return a list containing:\tabular{ll}{
#'     \code{proba_q_i_geq_a} \tab reel number between 0 and 1 representing the theoretical probability that the i-th excursion is greater or equal a \cr
#'     \tab \cr
#'     \code{P_alpha} \tab vector containing the probabilities (of reaching the threshold score on the i-th excursion) depending on the first letter of the sequence (alpha) \cr
#' }
#' @examples
#' ## In this example: the probability to reach a score of a in the third excursion is: 0.04237269
#' ## The conditional probabilities to reach a score of a in the third excursion if the first
#' ## letter of the sequence is K, L or M are respectively: 0.04239004, 0.04247805 and 0.04222251
#' proba_theoretical_ith_excursion_markov(a = 5,
#'                theta = c("K","L","M"),
#'                lambda = matrix(c(0.5, 0.3, 0.2,
#'                                  0.4, 0.2, 0.4,
#'                                  0.4, 0.4, 0.2), 
#'                                ncol = 3, byrow = TRUE),
#'                score_function = c(-2,-1,2),
#'                i = 3,
#'                prob0 = c(0.4444444, 0.2962963, 0.2592593))
#' ### This example implements the numerical application of the paper,
#' ###    the global probability is 0.2095639
#' proba_theoretical_ith_excursion_markov(a = 6,
#'             theta = c("a","b","c","d","e"),
#'             lambda = matrix(c(0.1, 0.7, 0.05, 0.05, 0.1,
#'                               0.3, 0.3, 0.2, 0.15, 0.05,
#'                               0.1, 0.4, 0.15, 0.2, 0.15,
#'                               0.5, 0.05, 0.25, 0.1, 0.1,
#'                               0.25, 0.05, 0.5, 0.15, 0.05),
#'                               ncol = 5,nrow=5, byrow = TRUE),
#'             score_function = c(-3,-2,-1,6,7),
#'             i = 3)
#' @export
proba_theoretical_ith_excursion_markov <- function(a, theta, lambda, score_function, i, prob0 = NULL, epsilon = 1e-16) {
  theta = as.character(theta)
  if(is.null(prob0)){
    prob0 = stationary_distribution(lambda)
  }
  for (x in score_function){
    if(x%%1){
      stop("[Invalid Input] scores of score_function should be integers.")
    }
  }
  if(sum(score_function * stationary_distribution(lambda)) >= 0){
    stop("[Invalid Input] score expectation should be negative.")
  }
  if(all(score_function <= 0)){
    stop("[Invalid Input] score_function should have at least one positive integer.")
  }
  if(length(unique(theta)) != length(theta)){
    stop("[Invalid Input] theta should contains only unique values")
  }
  if (i < 1){
    stop("[Invalid Input] i should be a strictly positive integer.")
  }
  if(!length(theta) == dim(lambda)[1]){
    stop("[Invalid Input] lambda should be the same size as theta.")
  }
  if (a == 0){
    P_alpha = rep(1,length(theta))
    names(P_alpha) = theta
    return(list(proba_q_i_geq_a = 1, P_alpha = P_alpha))
  }
  P_alpha = c()
  for (alpha in theta) {
    P_alpha[alpha] = proba_theoretical_first_excursion_markov_alpha(a, theta, lambda, score_function, alpha)$s
  }
  proba_q_i_geq_a = 0
  for (alpha in theta) {
    index_alpha = which(theta == alpha)
    proba_q_i_geq_a = proba_q_i_geq_a + (prob0[index_alpha] * P_alpha[alpha])
  }
  proba_q_i_geq_a = as.double(proba_q_i_geq_a)
  if ( i == 1 ) { 
    return ( list(proba_q_i_geq_a = proba_q_i_geq_a, P_alpha = P_alpha))
  }
  M = matrix_M(theta, lambda, score_function, N_iter = 1000, epsilon = epsilon)
  if (i > 1) {
    previous_exc = P_alpha
    M_i = M
    while (i > 2){
      M_i = M %*% M_i
      i = i - 1
    }
    P_alpha = M_i %*% P_alpha
    P_alpha = as.vector(P_alpha)
    names(P_alpha) = theta
  }
  proba_q_i_geq_a = 0
  for( alpha in theta ) {
    index_alpha = which(theta == alpha)
    proba_q_i_geq_a = proba_q_i_geq_a + (prob0[index_alpha] * P_alpha[[alpha]])
  }
  return (list(proba_q_i_geq_a = proba_q_i_geq_a, P_alpha = P_alpha))
}

#' @title Probability \eqn{P(Q(1)\geq a)} that the height of the first excursion is greater or equal to \code{a} given a i.i.d. model on the letters sequence
#' @description Mathematical definition of an excursion of the Lindley process is based on the record times of the partial
#' sum sequence associated to the score sequence (see Karlin and Altschul 1990, Karlin and Dembo 1992) and
#' define the successive times where the partial sums are strictly decreasing. There must be distinguished
#' from the visual excursions of the Lindley sequence. The number \code{i} is the number of excursion in sequential order. Detailed definitions are given in the vignette.
#' @param a score strictly positive
#' @param theta vector containing the alphabet used
#' @param theta_distribution distribution vector of theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @return theoretical probability of reaching a score of a on the first excursion supposing an  i.i.d model on the letters sequence
#' @details Beware that a sequence beginning with a negative score gives a "flat" excursion, with score 0 are considered.
#' @examples
#' proba_theoretical_first_excursion_iid(3, c("a","b","c","d"), 
#'                                       c(a=0.1,b=0.2,c=0.4,d=0.3), c(a=-3,b=-1,c=1,d=2))
#' @export
proba_theoretical_first_excursion_iid <- function(a, theta, theta_distribution, score_function){
  #Trick : use markov results with transition matrix with equal lines
  if (a <= 0) {
    return(1.0)
  }
  #Build transition matrix
  l <- length(theta)
  lambda = matrix(ncol = l,nrow = l, byrow = TRUE, dimnames = list(theta, theta))
  for (i in 1:l) {
    lambda[i,] <- theta_distribution
  }
  p <- proba_theoretical_first_excursion_markov_alpha(a, theta, lambda, score_function, theta[1])$s
  return(p)
}

#'@title Probability \eqn{P(Q(i)\geq a)} that the height of the ith excursion (sequential order) is greater or equal to \code{a} given a i.i.d. model on the letters sequence
#'@inherit proba_theoretical_first_excursion_iid  description params 
#'@param i Number of excursion in sequential order
#'@return theoretical probability of reaching a score of a on the ith excursion supposing an  i.i.d model on the letters sequence
#'@details
#'  In the i.i.d., the distribution of the ith excursion is the same as the first excursion. This function is just for convenience, and the result is the same as \code{proba_theoretical_first_excursion_iid}. Beware that a sequence beginning with a negative score gives a "flat" excursion, with score 0 are considered.
#'@examples
#'p1 <- proba_theoretical_ith_excursion_iid(3, c("a","b","c","d"), 
#'                                       c(a=0.1,b=0.2,c=0.4,d=0.3), c(a=-3,b=-1,c=1,d=2), i = 10)
#'p2 <- proba_theoretical_first_excursion_iid(3, c("a","b","c","d"), 
#'                                       c(a=0.1,b=0.2,c=0.4,d=0.3), c(a=-3,b=-1,c=1,d=2))
#'p1 == p2  #TRUE
#'@seealso \code{\link{proba_theoretical_first_excursion_iid}}
#'@export
proba_theoretical_ith_excursion_iid <- function(a, theta, theta_distribution, score_function, i = 1){
  return(proba_theoretical_first_excursion_iid(a, theta, theta_distribution, score_function))
}

context("Exact distribution of first and ith excursions - theoretical methods - Markov and iid model")
test_that("Results of intermediate function: traj, test when l!=Smin", {
  expected_results = list(
    c(-3, -6),
    c(-4, -5),
    c(-1, -2, -6),
    c(-1, -3, -5),
    c(-2, -1, -6),
    c(-2, -2, -5),
    c(-3, -1, -5),
    c(-1, -1, -1, -6),
    c(-1, -1, -2, -5),
    c(-1, -2, -1, -5),
    c(-2, -1, -1, -5),
    c(-1, -1, -1, -1, -5)
  )
  result <- traj( l = -5, j = 4, Smin = -6)
  expect_equal(result, expected_results)
})

test_that("Results of intermediate function: traj, test when l==Smin", {
  expected_results = list(
    c(-3, -3, -3),
    c(-1, -2, -3, -3),
    c(-1, -3, -2, -3),
    c(-2, -1, -3, -3),
    c(-2, -2, -2, -3),
    c(-2, -3, -1, -3),
    c(-3, -1, -2, -3),
    c(-3, -2, -1, -3),
    c(-1, -1, -1, -3, -3),
    c(-1, -1, -2, -2, -3),
    c(-1, -1, -3, -1, -3),
    c(-1, -2, -1, -2, -3),
    c(-1, -2, -2, -1, -3),
    c(-1, -3, -1, -1, -3),
    c(-2, -1, -1, -2, -3),
    c(-2, -1, -2, -1, -3),
    c(-2, -2, -1, -1, -3),
    c(-3, -1, -1, -1, -3),
    c(-1, -1, -1, -1, -2, -3),
    c(-1, -1, -1, -2, -1, -3),
    c(-1, -1, -2, -1, -1, -3),
    c(-1, -2, -1, -1, -1, -3),
    c(-2, -1, -1, -1, -1, -3),
    c(-1, -1, -1, -1, -1, -1, -3)
  )
  result <- traj(l = -3, j = 6, Smin = -3)
  expect_equal(result, expected_results)
})

test_that("Results of intermediate function: traj, test when there is no trajectory", {
  expected_results = list()
  result <- traj(l = -3, j = 6, Smin = -1)
  expect_equal(result, expected_results)
})

test_that("Results proba theoretical third excursion normal case (Numerical application of the paper:\"Exact distribution of excursion heights of the Lindley
process in a Markovian model\")", {
  #Validated by simulation
  expected_results = list()
  expected_results$proba_q_i_geq_a = c(0.2095639, 0.2095639, 0.2095639, 0.2095639, 0.2095639, 0.2095639, 0.1524696, 0.1196005, 0.1154948, 0.1112926)
  expected_results$P_alpha = list(c(a = 0.2113536, b = 0.2088997, c = 0.2105256, d = 0.2065410, e = 0.2098564),
                                  c(a = 0.1560269, b = 0.1513963, c = 0.1540934, d = 0.1475148, e = 0.1510870),
                                  c(a = 0.1212326, b = 0.1190616, c = 0.1203996, d = 0.1171290, e = 0.1193353),
                                  c(a = 0.1170527, b = 0.1149792, c = 0.1162590, d = 0.1131304, e = 0.1152516),
                                  c(a = 0.1128484, b = 0.1107814, c = 0.1120513, d = 0.1089475, e = 0.1110196)
  )
  
  #a=1 to a=5 should return the same results as a=6, we don't verify them to reduce computational time
  for (a in 6:10){
    result <- proba_theoretical_ith_excursion_markov(a = a,
                                                     theta = c("a","b","c","d","e"),
                                                     lambda = matrix(c(0.1,0.7,0.05,0.05,0.1, 0.3,0.3,0.2,0.15,0.05, 0.1,0.4,0.15,0.2,0.15, 0.5,0.05,0.25,0.1,0.1, 0.25,0.05,0.5,0.15,0.05), ncol = 5,nrow=5, byrow = TRUE),
                                                     score_function = c(-3,-2,-1,6,7),
                                                     i = 3)
    expect_equal(result$proba_q_i_geq_a, expected_results$proba_q_i_geq_a[a], tolerance = 1e-6)
    expect_equal(result$P_alpha, expected_results$P_alpha[a-5][[1]], tolerance = 1e-6)
  }
})

test_that("Results proba theoretical ith excursion, normal case (with an integer in theta)", {
  #Validated by simulation
  expected_result = list()
  expected_result$proba_q_i_geq_a = 0.3083594
  expected_result$P_alpha = c(K = 0.2158516, "1" = 0.3453626)
  result <- proba_theoretical_ith_excursion_markov(a = 5,
                                                   theta = c("K", 1),
                                                   lambda = matrix(c(0.5, 0.5, 0.2, 0.8), ncol = 2, byrow = TRUE),
                                                   score_function = c(-4,1),
                                                   i = 1)
  expect_equal(result$proba_q_i_geq_a, expected_result$proba_q_i_geq_a, tolerance = 1e-6)
  expect_equal(result$P_alpha, expected_result$P_alpha, tolerance = 1e-6)
})

test_that("Results proba theoretical first excursion normal case (2)", {
  result = proba_theoretical_first_excursion_markov(a = 5, 
                                                    theta = c("K","L","M"), 
                                                    lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), 
                                                    score_function = c(-2,-1,2),
                                                    prob0 = c(0.4444444, 0.2962963, 0.2592593) )
  expect_equal(result, 0.04134028)
  #  expect_equal(as.vector(result$z), expected_results$z)
})

test_that("Results proba theoretical second excursion, specific case (transition matrix have only low or high values) ", {
  #Validated by simulation
  expected_result = list()
  expected_result$proba_q_i_geq_a = 0.01818579
  expected_result$P_alpha =  c(K = 0.01998373, L = 0.01800000, M = 0.00396745)
  result <- proba_theoretical_ith_excursion_markov(a = 4,
                                                   theta = c("K", "L", "M"),
                                                   lambda = matrix(c(0.9, 0, 0.1, 0.9, 0.1, 0, 0, 0.8, 0.2), ncol = 3, byrow = TRUE),
                                                   score_function = c(-3,-1,2),
                                                   i = 2)
  expect_equal(result$proba_q_i_geq_a, expected_result$proba_q_i_geq_a, tolerance = 1e-6)
  expect_equal(result$P_alpha, expected_result$P_alpha, tolerance = 1e-6)
})

test_that("Results proba theoretical second excursion, specific case (a = 0) ", {
  #Validated by simulation
  expected_result = list()
  expected_result$proba_q_i_geq_a = 1
  expected_result$P_alpha =  c(K = 1, L = 1, M = 1)
  result <- proba_theoretical_ith_excursion_markov(a = 0,,
                                                   theta = c("K", "L", "M"),
                                                   lambda = matrix(c(0.3, 0.2, 0.5, 0.4, 0.4, 0.2, 0.3, 0.3, 0.4), ncol = 3, byrow = TRUE),
                                                   score_function = c(-3,-1,2),
                                                   i = 2)
  expect_equal(result$proba_q_i_geq_a, expected_result$proba_q_i_geq_a, tolerance = 1e-6)
  expect_equal(result$P_alpha, expected_result$P_alpha, tolerance = 1e-6)
})

test_that("Results proba theoretical second excursion, specific case (0 in the score function) ", {
  #Validated by simulation
  expected_result = list()
  expected_result$proba_q_i_geq_a = 0.2816628
  expected_result$P_alpha = c(K = 0.3691211, L = 0.2132907, M = 0.3155178, N = 0.2312518, O = 0.3066551)
  result <- proba_theoretical_ith_excursion_markov(a = 4,,
                                                   theta = c("K", "L", "M","N","O"),
                                                   lambda = matrix(c(0.1, 0.7, 0.05, 0.05, 0.1,
                                                                     0.3, 0, 0.2, 0.15, 0.35,
                                                                     0.1, 0.4, 0.15, 0.2, 0.15,
                                                                     0.3, 0.05, 0.25, 0.3, 0.1,
                                                                     0.25, 0.45, 0.1, 0.15, 0.05), ncol = 5, nrow = 5, byrow = TRUE),
                                                   score_function = c(-5, -3, 0, 1, 4),
                                                   i = 2)
  expect_equal(result$proba_q_i_geq_a, expected_result$proba_q_i_geq_a, tolerance = 1e-6)
  expect_equal(result$P_alpha, expected_result$P_alpha, tolerance = 1e-6)
})

test_that("Results proba theoretical second excursion, specific case (diagonal of the transition matrix is filled by 0) ", {
  #Validated by simulation
  expected_result = list()
  expected_result$proba_q_i_geq_a = 0.1254411
  expected_result$P_alpha = c(K = 0.1237689, L = 0.1302016, M = 0.1227621)
  result <- proba_theoretical_ith_excursion_markov(a = 5,
                                                   theta = c("K", "L", "M"),
                                                   lambda = matrix(c(0, 0.3, 0.7, 0.4, 0, 0.6, 0.4, 0.6, 0), ncol = 3, byrow = TRUE),
                                                   score_function = c(-2,-1,2),
                                                   i = 2)
  expect_equal(result$proba_q_i_geq_a, expected_result$proba_q_i_geq_a, tolerance = 1e-6)
  expect_equal(result$P_alpha, expected_result$P_alpha, tolerance = 1e-6)
})

test_that("Result that differ from matrix_M", {
  expected_result = matrix(c(1,0,
                           1,0),
                           nrow = 2,
                           byrow = TRUE)

  result <- matrix_M(theta = c("K", "L"),
                            lambda = matrix(c(0.5, 0.5, 0.2, 0.8), ncol = 2, byrow = TRUE),
                            score_function = 2 * c(-4,1),
                            N_iter = 1000,
                            epsilon = 1e-16)
  expect_equal(result, expected_result)
})

test_that("Result where two positive scores are equal", {
  expected_result = matrix(c(0.2192127, 0.72629245, 0.05449489, 0, 0,
                           0.4653842, 0.32935851, 0.20525729, 0, 0,
                           0.3845969, 0.45565097, 0.15975218, 0, 0,
                           0.6615238, 0.08277737, 0.25569883, 0, 0,
                           0.4153842, 0.07935851, 0.50525729, 0, 0),
                         nrow = 5,
                         byrow = TRUE)
  
  result <- matrix_M(theta = c("a","b","c","d","e"),
                     lambda = matrix(c(0.1,0.7,0.05,0.05,0.1, 0.3,0.3,0.2,0.15,0.05, 0.1,0.4,0.15,0.2,0.15, 0.5,0.05,0.25,0.1,0.1, 0.25,0.05,0.5,0.15,0.05), ncol = 5,nrow=5, byrow = TRUE),
                     score_function = c(-10, -2, -1, 6, 6),
                     N_iter = 1000,
                     epsilon = 1e-16)
  expect_equal(result, expected_result, tolerance = 1e-06)
})

test_that("Results proba theoretical fourth excursion, error case (all scores are negative) ", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K", "L", "M"),
                                                      lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE),
                                                      score_function = c(-2, -1, -2),
                                                      i = 4),"[Invalid Input] score_function should have at least one positive integer.",fixed = TRUE)
})

test_that("Results proba theoretical fourth excursion, error case (theta have a non-unique value) ", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K", "L", "L"),
                                                      lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE),
                                                      score_function = c(-2, -1, 2),
                                                      i = 4),"[Invalid Input] theta should contains only unique values",fixed = TRUE)
})

test_that("Results proba theoretical fourth excursion, error case (theta have a different number of values) ", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K", "L"),
                                                      lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE),
                                                      score_function = c(-2, -1, 2),
                                                      i = 4),"[Invalid Input] lambda should be the same size as theta.",fixed = TRUE)
})

test_that("Results proba theoretical second excursion, error case (score expectation is positive)",{
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K", "L", "M"),
                                                      lambda = matrix(c(0.5, 0.1, 0.4, 0.4, 0.05, 0.55, 0.3, 0.4, 0.3), ncol = 3, byrow = TRUE),
                                                      score_function = c(2, -1, 2),
                                                      i = 2),"[Invalid Input] score expectation should be negative.",fixed = TRUE)
})

test_that("Results proba theoretical fourth excursion, error case (score are not integers) ", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K","L","M"),
                                                      lambda = matrix(c(0.1, 0.5, 0.4, 0.4, 0.05, 0.55, 0.3, 0.4, 0.3), ncol = 3, byrow = TRUE),
                                                      score_function = c(2.4,-1.8,-2),
                                                      i = 3),"[Invalid Input] scores of score_function should be integers.",fixed = TRUE)
})

test_that("Results proba theoretical fifth excursion, error case (i = 0) ", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K", "L", "M"),
                                                      lambda = matrix(c(0.1, 0.5, 0.4, 0.4, 0.05, 0.55, 0.3, 0.4, 0.3), ncol = 3, byrow = TRUE),
                                                      score_function = c(2,-1,-2),
                                                      i = 0),"[Invalid Input] i should be a strictly positive integer.",fixed = TRUE)
})

test_that("Results proba theoretical fourth excursion, error case (lambda is not a stochastic matrix) ", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5,
                                                      theta = c("K","L","M"),
                                                      lambda = matrix(c(0.1, 0.5, 0.5, 0.4, 0.01, 0.05, 0.3, 0.4, 0.3), ncol = 3, byrow = TRUE),
                                                      score_function = c(-4, -1, 2),
                                                      i = 1),"[ERROR] Transition probability matrix is not stochastic (row sum not equal 1.). Sum of line 1 equal 1.100000",fixed = TRUE)
})
##############

test_that("Results proba theoretical first execution special case only positive scores", {
  result = proba_theoretical_first_excursion_markov(a = 5, 
                                                    theta = c("K","L","M"), 
                                                    lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), 
                                                    score_function = c(2,2,2),
                                                    prob0 = c(0.4444444, 0.2962963, 0.2592593) )
  expected_results = list()
  expected_results$s = 1
  expected_results$z = c(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)
  
  expect_equal(result, expected_results$s)
  #  expect_equal(as.vector(result$z), expected_results$z)
})

test_that("Results proba theoretical first execution special case only negative scores", {
  result = proba_theoretical_first_excursion_markov(a = 5, 
                                                    theta = c("K","L","M"), 
                                                    lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), 
                                                    score_function = c(-2,-2,-2),
                                                    prob0 = c(0.4444444, 0.2962963, 0.2592593) )
  expected_results = list()
  expected_results$s = 0
  expected_results$z = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
  
  expect_equal(result, expected_results$s)
  #  expect_equal(as.vector(result$z), expected_results$z)
})

test_that("Results proba theoretical ith execution normal case", {
  # Note : simulating with 1.000.000 iterations, we found 0.033892. 
  # With this simple score scheme, could it be calculated exactly ?
  result = proba_theoretical_ith_excursion_markov(a = 5, 
                                                  theta = c("K","L","M"), 
                                                  lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), 
                                                  score_function = c(-1,0,1),
                                                  prob0 = c(0.4444444, 0.2962963, 0.2592593), i = 3 )
  expected_results = list()
  expected_results$proba_q_i_geq_a = 0.03392221 # 0.034025 
  #expected_results$proba_q_i_geq_a =  0.0172477 #XXX je ne sais pas laquelle des valeurs est bonne entre l'initial (0.034025) et celle actuelle (0.0172)
  
  expect_equal(result$proba_q_i_geq_a, expected_results$proba_q_i_geq_a, tolerance = 0.000001)
})

test_that("Results proba theoretical ith execution special case only positive scores", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5, 
                                                      theta = c("K","L","M"), 
                                                      lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), 
                                                      score_function = c(2,2,2),
                                                      prob0 = c(0.4444444, 0.2962963, 0.2592593), i = 3 ),
               "[Invalid Input] score expectation should be negative.", fixed = TRUE
  )
})

test_that("Results proba theoretical ith execution special case only negative scores", {
  expect_error(proba_theoretical_ith_excursion_markov(a = 5, 
                                                      theta = c("K","L","M"), 
                                                      lambda = matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), 
                                                      score_function = c(-2,-2,-2),
                                                      prob0 = c(0.4444444, 0.2962963, 0.2592593), i = 3 ),
               "[Invalid Input] score_function should have at least one positive integer.", fixed = TRUE
  )
})

test_that("Results matrice M", {
  result = matrix_M(c("K","L","M"),
                   matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE),
                    c(-2,-1,2),
                    N_iter = 100)
  expected_results = matrix(c(0.6525332, 0.3474668,    0,
                              0.7050664, 0.2949336,    0,
                              0.5525332, 0.4474668,    0), ncol = 3, byrow = TRUE)
  #(theta, a, lambda, score_function, N_iter)
  expect_equal(expected_results, result, tolerance = 1e-4)
})

#' @useDynLib localScore
#' @importFrom Rcpp evalCpp
#' @importFrom graphics abline barplot par plot
#' @importFrom stats lm ecdf knots density 
#' @importFrom utils read.csv setTxtProgressBar txtProgressBar
#' @importFrom utils read.csv setTxtProgressBar txtProgressBar
#' 
#' @rdname localScoreC 
#' @export
localScoreC_double <- function(v, suppressWarnings = FALSE) {
  mode(v) <- "numeric"
  return(localScoreC(v, suppressWarnings))
}
#' 
#' @rdname localScoreC
#' @export
localScoreC_int <- function(v, suppressWarnings = FALSE) {
  v <- as.integer(v)
  return(localScoreC(v, suppressWarnings))
}
#' 
#' @title Monte Carlo method [p-value]
#' @description Calculates an empirical p-value based on Monte Carlo simulations.
#' Perfect for small sequences (both markov chains and identically and independently distributed) with length ~ 10^3. 
#' @param local_score local score observed in a segment.
#' @param FUN function to simulate similar sequences with.
#' @param ... parameters for FUN
#' @param plot boolean value if to display plots for cumulated function and density
#' @param numSim number of sequences to generate during simulation
#' @param keepSimu Boolean, default to FALSE. If TRUE, the simulated local scores are returned as the localScores element of the output list.
#' @return If \code{keepSimu} is FALSE, returns a numeric value corresponding to the probability to obtain a local score with value greater or equal to the parameter \code{local_score}. \cr
#' If \code{keepSimu} is TRUE, returns a list containing:\tabular{ll}{
#'     \code{p_value} \tab Floating value corresponding to the probability to obtain a local score with a value greater or equal to the parameter \code{local_score} \cr
#'     \code{localScores} \tab Vector of size \code{numSim} containing the simulated local scores
#' }
#' @details
#' Be careful that the parameters names of the function FUN should differ from those of monteCarlo function.\cr
#' The density plot produced by \code{plot == TRUE} depends on the type of the simulated local scores: 
#' if they are integer, a barplot of relative frequency is used, else \code{plot(density(...))} is used. \cr
#' This function calls \code{\link{localScoreC}} which type of the output depends on the type of the input. 
#' To be efficient, be aware to use a simulating function \code{FUN} that return a vector of adequate type ("integer" or "numeric"). Warning: in R, \code{typeof(c(1,3,4,10)) == "double"}. You can set a type of a vector with \code{mode()} or \code{as.integer()} functions for example. \cr 
#' \code{monteCarlo_double()} is deprecated. At this point, it is just a call to \code{monteCarlo()} function.
#' @seealso \code{\link{karlinMonteCarlo}} \code{\link{localScoreC}}
#' @examples
#' \donttest{
#' monteCarlo(120, FUN = rbinom, n = 100, size = 5, prob=0.2)
#' }
#' \donttest{
#' mySeq <- sample(-7:3, replace = TRUE, size = 1000)
#' monteCarlo(local_score = 18, FUN = function(x) {return(sample(x = x, 
#'            size = length(x), replace = TRUE))}, x = mySeq)
#' }
#' \donttest{
#' #Examples of non integer score function
#' mySeq2 <- sample(-7:6 - 0.5, replace = TRUE, size = 1000)
#' monteCarlo(local_score = 50.5, FUN = function(x) {return(sample(x = x, 
#'            size = length(x), replace = TRUE))}, x = mySeq2)
#' }
#' \donttest{
#' #Examinating simulated local scores
#' mySeq2 <- sample(-7:6, replace = TRUE, size = 1000)
#' simu <- monteCarlo(local_score = 50.5, FUN = function(x) {return(sample(x = x, 
#'            size = length(x), replace = TRUE))}, x = mySeq2, keepSimu = TRUE)
#' hist(simu$localScores)
#' }
#' \donttest{
#' # Markovian example
#' MyTransMat <-
#'     matrix(c(0.3,0.1,0.1,0.1,0.4, 0.2,0.2,0.1,0.2,0.3, 0.3,0.4,0.1,0.1,0.1, 0.3,0.3,0.1,0.0,0.3,
#'              0.1,0.1,0.2,0.3,0.3), ncol = 5, byrow=TRUE)
#' monteCarlo(local_score = 50,
#'           FUN = transmatrix2sequence, matrix = MyTransMat,
#'           length=150, score = c(-2,-1,0,2,3), plot=FALSE, numSim = 5000)
#' }
#' @export
monteCarlo <- function(local_score, FUN, ..., plot = TRUE, numSim = 1000, keepSimu = FALSE){
  epsilon <- 1e-8 #Awfull trick used to correct CdF in case of discrete score values
  if (local_score < 0)
    stop("[Invalid Input] local scores are never strictly negative.")
  if (local_score == 0) #Trivial case
    return(1.0)
  if (numSim <= 0)
    stop("[Invalid Input] Please supply a positive number of simulations.")
  mcParametersNames <- c("local_score", "FUN", "numSim", "plot", "keepSimu")
  FUNparametersName <- names(formals(FUN))
  if (any(FUNparametersName %in% mcParametersNames))
    stop("[Invalid Input] Parameters of FUN should differ from parameters of monteCarlo function")

  #simulate sequences to get some local scores to work with
  maxScores <- sim_local_scores(numSim = numSim, FUN = FUN, ...)
  if (!all(!is.na(maxScores)))
    warning(paste0("Some simulated local scores are NA. Number of NA: ",sum(is.na(maxScores))))
  
  #Empirical cumulated distribution of found local Scores
  scoreCdf <- ecdf(maxScores)
  if (all(maxScores %% 1 == 0)) { # is.integer(maxScores) can lead to misleading 
    scoreD <- table(maxScores)/length(maxScores)
  } else if (is.numeric(maxScores)) {
    scoreD <- density(maxScores)
  } else {
    stop("The local score associated to simulating function should be either an integer or real number.")
  }
  #if the user wants to display the distribution of local scores, print it to the plot screen
  if (plot) {
    #set plot window
    oldpar <- par(no.readonly = TRUE) 
    on.exit(par(oldpar))
    par(mfrow = c(1,2))
    plot(scoreD, main = "Distribution of local scores\n for given sequence", ylab = "Probability", xlab = "Local score")
    abline(v = local_score, col = "red", lty = 3) 
    plot(scoreCdf, main = "Cumulative Distribution Function", ylab = "cumulative distribution(local_scores)))", xlab = "Local score")
    abline(v = local_score, col = "red", lty = 3)  #greater or equal, thus set marker at localScore -1
    abline(h = 1 - scoreCdf(local_score * (1 - epsilon)), col = "red", lty = 3)
  }
  result <- 1 - scoreCdf(local_score * (1 - epsilon))
  names(result) <- c("p_value")
  if (keepSimu) {
    return(list("p_value" = result, localScores = maxScores))
  } else {
    return(result) 
  }
}

#' @title Simulate a sequence of local score [p-value]
#' @description Simulate a sequence of local score based on a simulation function of individual sequence of score.
#' @param FUN function to simulate similar sequences with.
#' @param ... parameters for FUN
#' @param numSim number of sequences to generate during simulation
#' @return vector containing a local score sample of size numSim
#' @details
#' If all simulated local score are integer (in mathematical sense), the returned value type is set to \code{integer}
#' @noRd
sim_local_scores <- function(FUN, ..., numSim = 1000) {
  #maxScores <- vector(mode = "numeric", length = numSim)
  maxScores <- rep(NA, length = numSim)
  if (interactive())
    progressBar <- txtProgressBar(min = 0, max = numSim, initial = 0, char = "=",
                                  width = NA, 
                                  paste0("progress in local scores simulation (", numSim, ")"),
                                  "progressbar", style = 3, file = "")
  for (i in 1:numSim) {
      simSequence <- FUN(...)
      #print(paste0("length simSequence: ",length(simSequence)))
      maxScores[i] <- localScoreC(simSequence, suppressWarnings = TRUE)$localScore["value"]
      if (interactive())
        setTxtProgressBar(progressBar, i)
  }
  if (all(maxScores %% 1 == 0))  # is.integer(maxScores) is misleading
    mode(maxScores) <- "integer"
  return(maxScores)
}

#' @describeIn monteCarlo Monte-Carlo function for double [deprecated]
#' @export
monteCarlo_double <- function(local_score, FUN, ..., plot = TRUE, numSim = 1000, keepSimu = FALSE) {
  return(monteCarlo(local_score, FUN, ..., plot = plot, numSim = numSim, keepSimu = keepSimu))
}

#' @title Lindley process
#' @description Creates a sequence of a Lindley process, also called CUSUM process, on a given sequence.  For a sequence (X_k)k, the Lindley process is defined as follows: W_0:=0 and W_(k+1)=max(0,W_k+X_(k+1)). It defines positive excursions above 0.
#' @param sequence numeric sequence of a Lindley process, eg service time per customer
#' @return a vector with the Lindley process steps
#' @examples
#' MySeq <- c(1,2,3,-4,1,-3,-1,2,3,-4,1)
#' lindley(MySeq)
#' plot(1:length(MySeq),lindley(MySeq),type='b')
#' @export
lindley <- function(sequence){
  seq <- rep(NA, length(sequence))
  sum <- 0
  i <- 1
  for (score in sequence) {
    sum <- max(0, score + sum)
    seq[i] <- sum
    i <- i + 1
  }
  return(seq)
}

#' @title Calculate the record times of a sequence
#' @description For a given sequence of real numbers, this function returns the
#'   index of the values where Lindley/CUSUM process are <0
#' @param sequence numeric sequence of a Lindley process, eg service time per
#'   customer
#' @return a vector with the record times. If no record time are found, return
#'   empty vector \code{integer(0)}
#' @details Note that the first record times which is always 0 is not included
#' in the returned vector
#' @examples
#' ####This example should return this vector: c(1,3,4,5,10)
#' seq1 <- c(-1,2,-2.00001,-4,-1,3,-1,-2,3,-4,2)
#' recordTimes(seq1)
#' ####This example should return integer(0) because there is no record times in this sequence
#' seq2 <- c(4,1,0,-1,-2,5,-5,0,4)
#' recordTimes(seq2)
#' @export
recordTimes <- function(sequence) {
  record_times <- rep(NA, length(sequence))
  record_times <- as.vector(record_times, mode = "integer")
  sum <- 0
  i <- 1
  j <- 1
  for (score in sequence) {
    if ( sum + score < 0) {
      record_times[j] <- i
      j <- j + 1
    }
    sum = max(0, sum + score)
    i <- i + 1
  }
  record_times <- record_times[!is.na(record_times)]
  return(record_times)
}

#' @title Monte Carlo - Karlin [p-value]
#' @description Estimates p-value of the local score based on a Monte Carlo
#'   estimation of Gumble parameters from simulations of smaller sequences with
#'   same distribution. Appropriate for great sequences with length > 10^3, for
#'   i.i.d and markovian sequence models.
#' @param local_score local score observed in a sequence.
#' @param sequence_length length of the sequence
#' @param simulated_sequence_length length of simulated sequences produced by
#' @param FUN function to simulate similar sequences with.
#' @param ... parameters for FUN
#' @param numSim number of sequences to create for estimation
#'   FUN
#' @param plot boolean value if to display plots for cumulated function, density
#'   and linearization of cumulative density function
#' @param keepSimu Boolean, default to FALSE. If TRUE, the simulated local
#'   scores are returned as the localScores element of the output list.
#' @return If \code{keepSimu} is FALSE, returns a list containing:\tabular{ll}{
#'   \code{p_value} \tab Probability to obtain a local score with a value
#'   greater or equal to the parameter \code{local_score} \cr \tab \cr \code{K*}
#'   \tab Parameter \eqn{K^*} defined in Karlin and Dembo (1990) \cr \tab
#'   \cr \code{lambda} \tab Parameter \eqn{\lambda} defined in Karlin and
#'   Dembo (1990) \cr }
#' If \code{keepSimu} is TRUE, returns a list containing:\tabular{ll}{
#'     \code{p_value} \tab Probability to obtain a local score with a value greater or equal to the parameter \code{local_score} \cr
#'     \tab \cr
#'     \code{K*} \tab Parameter \eqn{K^*} defined in Karlin and Dembo (1990) \cr
#'     \tab \cr
#'     \code{lambda} \tab Parameter \eqn{\lambda} defined in Karlin and Dembo (1990) \cr
#'     \code{localScores} \tab Vector of size \code{numSim} containing the simulated local scores for sequence size of \code{simulated_sequence_length}
#' }
#' @details The length of the simulated sequences is an argument specific to the
#'   function provided for simulation. Thus, it has to be provided also in the
#'   parameter \code{simulated_sequence_length} in the arguments of the "Monte
#'   Carlo - Karlin" function. It is a crucial detail as it influences precision
#'   and computation time of the result. Note that to get an appropriate
#'   estimation, the given average score must be non-positive. Be careful that
#'   the parameters names of the function \code{FUN} should differ from those of
#'   \code{karlinMonteCarlo} function. \cr Methods - Parameters \eqn{K^\star}
#'   and \eqn{\lambda} of Karlin and Dembo (1990) are estimated by a linear
#'   regression on the log(-log(cumulative distribution function of the local
#'   scores)) on shorter sequences (size \code{simulated_sequence_length}). The
#'   formula used are : \eqn{\hat{\lambda} = -\hat{b}} and \eqn{\hat{K^\star} =
#'   exp(\hat{a})/simulated\_sequence\_length} where \eqn{\hat{a}} is the
#'   intercept of the regression and \eqn{\hat{b}} is the slope of the
#'   regression. Then p-value is given by \eqn{p = exp(-K^\star * exp(-\lambda*x
#'   ))} where \eqn{x = local\_score - \log(sequence\_length)/\lambda}. \cr The
#'   density plot produced by \code{plot == TRUE} depends on the type of the
#'   simulated local scores: if they are integer, a barplot of relative
#'   frequency is used, else \code{plot(density(...))} is used. \cr This
#'   function calls \code{\link{localScoreC}} which type of the output depends
#'   on the type of the input. To be efficient, be aware to use a simulating
#'   function \code{FUN} that return a vector of adequate type ("integer" or
#'   "numeric"). Warning: in R, \code{typeof(c(1,3,4,10)) == "double"}. You can
#'   set a type of a vector with \code{mode()} or \code{as.integer()} functions
#'   for example. \cr \code{karlinMonteCarlo_double()} is deprecated. At this
#'   point, it is just a call to \code{karlinMonteCarlo()} function.
#' @seealso \code{\link{monteCarlo}} \code{\link{localScoreC}}
#' @examples
#' \donttest{
#' mySeq <- sample(-7:6, replace = TRUE, size = 100000)
#' #MonteCarlo taking random sample from the input sequence itself
#' karlinMonteCarlo(local_score = 160, sequence_length = 100000,
#'                simulated_sequence_length = 1000,
#'                FUN = function(x, sim_length) {
#'                         return(sample(x = x,
#'                                size = sim_length,
#'                                replace = TRUE))
#'                      },
#'                x = mySeq,
#'                sim_length = 1000,
#'                numSim = 1000)
#' }
#' \donttest{
#' #Markovian example (longer computation)
#' MyTransMat_reels <-  matrix(c(0.3, 0.1, 0.1, 0.1, 0.4,
#'                               0.2, 0.2, 0.1, 0.2, 0.3,
#'                               0.3, 0.4, 0.1, 0.1, 0.1,
#'                               0.3, 0.3, 0.1, 0.2, 0.1,
#'                               0.2, 0.1, 0.2, 0.4, 0.1),
#'                               ncol = 5, byrow=TRUE)
#' karlinMonteCarlo(local_score = 18.5, sequence_length = 200000,
#'                  simulated_sequence_length = 1500,
#'                  FUN = transmatrix2sequence,
#'                  matrix = MyTransMat_reels,
#'                  score =c(-1.5,-0.5,0,0.5,1), length = 1500,
#'                  plot=TRUE, numSim = 1500)
#' }
#' @export
karlinMonteCarlo <- function(local_score, sequence_length, simulated_sequence_length, FUN,  ..., numSim = 1000, plot = TRUE, keepSimu = FALSE) {
  if (local_score < 0)
    stop("[Invalid Input] local Scores are not negative.")
  if (numSim <= 0)
    stop("[Invalid Input] Please supply a positive number of simulations.")
  if (missing(sequence_length))
    stop("[Missing Argument] sequence_length is a required parameter")
  if (missing(simulated_sequence_length))
    stop("[Missing Argument] simulated_sequence_length is a required parameter")
  kmcParametersNames <- c("local_score", "sequence_length", "simulated_sequence_length", "FUN", "numSim", "plot")
  FUNparametersName <- names(formals(FUN))
  if (any(FUNparametersName %in% kmcParametersNames))
    stop("[Invalid Input] Parameters of FUN should differ from parameters of karlinMonteCarlo function")
  #simulate sequences to get some local scores to work with
  #maxScores <- sim_local_scores(numSim = numSim, FUN = FUN, ...) # bug voir #11238 sur forge DGA
  maxScores <- sim_local_scores(numSim = numSim, FUN = FUN, ...)

  #Empirical distribution of found local Scores on (short) simulated sequences
#  scoreCdf <- ecdf(maxScores)
  scoreShortD <- table(maxScores)/length(maxScores)

  # if (all(maxScores %% 1 == 0)) { # is.integer(maxScores) can lead to misleading 
  # } else if (is.numeric(maxScores)) {
  #   scoreShortD <- density(maxScores)
  # } else {
  #   stop("The local score associated to simulating function should be either an integer or real number.")
  # }
  
  #Cumulative distribution function of local score probabilities
  log_cumsum <- log(-log(cumsum(scoreShortD))) 
  #clean up eventual inf values
  log_cumsum <- log_cumsum[is.finite(log_cumsum)]
  
  #linear regression (Warning : at least two different simulated values of MaxScores needed)
  if (length(log_cumsum) < 2) {
    p_value <- NA
    k_star <- NA
    lambda <- NA
  } else {
    reg <- lm(log_cumsum~as.numeric(names(log_cumsum)))
    lambda <- -reg$coefficients[2]      #Inclination
    names(lambda) <- NULL
    k_star <- exp(reg$coefficients[1])/simulated_sequence_length #SIMULATED SEQUENCE LENGTH
    names(k_star) <- NULL
    
    x <- local_score - log(sequence_length)/lambda - 1
    #Karlin: for n great, P( ln(n)/lambda+x>= M) = exp(-K_star*exp(-lambda*x))
    #thus we set ln(n)/lambda+x = local_score and obtain for x = local_score - ln(n)/lambda
    #x = local_score - log(sequence_length)/lambda#-1 ? XXX a priori oui, il faut le '-1' (cf le code C++)
    
    #now we calculate p-value with our approximate K star and lambda
    p_value <- exp(-k_star*exp(-lambda*x))
  }
  if (keepSimu) {
    result <- list("p_value" = 1 - p_value, "K*" = as.numeric(k_star), 
                   "lambda" = as.numeric(lambda), localScores = maxScores)
  } else {
    result <- list("p_value" = 1 - p_value, "K*" = as.numeric(k_star),
                   "lambda" = as.numeric(lambda))
  }
  if (plot) {
    if (is.numeric(maxScores) && !all(maxScores %% 1 == 0))  # real scores so density plot
      scoreShortD <- density(maxScores)
    
    #absice transformations
    # xnew <- as.numeric(names(scoreShortD)) + log(sequence_length/simulated_sequence_length)/lambda
    # names(scoreShortD) <- xnew
    #set plot window to contain 3 slots
    oldpar <- par(no.readonly = TRUE) 
    on.exit(par(oldpar))
    par(mfrow = (c(1,3)))
    # lab <- rep("", length(xnew))
    # lab[seq.int(from = 1, to = length(lab), by = trunc(length(lab)/2))] <- round(xnew[seq.int(from = 1, to = length(lab), by = trunc(length(lab)/2))],1)
    #lab <- round(xnew, 1) # Une décimale pour les labels
    plot(scoreShortD, main = paste0("Distribution of simulated local scores\n (n=",simulated_sequence_length,")"), ylab = "Empirical density", xlab = "Local score values", frame = TRUE) #, xaxt = "n")
    #axis(1, at = lab, las = 2)
    
    plot(cumsum(table(maxScores)/length(maxScores))~as.numeric(names(table(maxScores))), main = paste0("Cumulative Distribution Function\n (n=",simulated_sequence_length,")"), ylab = "cumulative distribution(local_scores)))", xlab = "Local Score Values")
    plot(log_cumsum~as.numeric(names(log_cumsum)), main = "Linear Regression", ylab = "ln(-ln(cumulative destribution(local_scores)))", xlab = "Local score values")
    abline(reg, col = c("green"))
  }
  return(result)
}

#' @rdname karlinMonteCarlo
#' @export
karlinMonteCarlo_double <- function(local_score, sequence_length, simulated_sequence_length, FUN,  ..., numSim = 1000, plot = TRUE, keepSimu = FALSE){
  return(karlinMonteCarlo(local_score, sequence_length, simulated_sequence_length, FUN,  ..., numSim = numSim, plot = plot, keepSimu = keepSimu))
}

#' @title Transition matrix from sequence(s)
#' @description Calculates the transition matrix by counting occurrences of tuples in given vector list
#' @param sequences Sequences to be analyzed, can be a list of vectors, or a vector
#' @return A list object containing
#' \item{transition_matrix}{Transition Matrix with row names and column names are the associated score/state}
#' \item{score_value}{a vector containing the score/state, ordered in the same way as the matrix columns and rows}
#' @details
#' In output, score_value is coerced as integer if possible. Else, it is a character vector containing the states
#' of the Markov chain.
#' @examples
# seq1 <- sample(-1:1, size = 20, replace = TRUE)
# seq2 <- sample(-6:1, size = 20, replace = TRUE)
# seq3 <- sample(3:6, size = 50, replace = TRUE)
# sequences2transmatrix(list(seq1, seq2, seq3))
#' myseq <- sample(LETTERS[1:2], size=20,replace=TRUE)
#' sequences2transmatrix(myseq)
#' @export
sequences2transmatrix <- function(sequences){
  if (is.list(sequences))
    #get score in 1 vector separate by NA values
    allseq <- unlist(lapply(sequences, FUN = function(x){c(x,NA)}))
  else if (is.vector(sequences))
    allseq <- c(sequences,NA)
  else
    stop("The parameter should be either a list of vector or a vector")
  
  # Count Nuv/Nu
  p <- prop.table(table(allseq[-length(allseq)],allseq[-1]),margin = 1)
  # Treat p name as numeric or not
  state <- suppressWarnings(as.numeric(rownames(p)))
  if (all(!is.na(state))) #case all names are numeric
    score_value <- state
  else
    score_value <- rownames(p)
  if ((length(rownames(p)) != length(colnames(p))) || !all(rownames(p) == colnames(p)))
    stop("The score beginning the sequence appears only one time.")
  if (!all(!is.na(p))) # there is at least 1 NA is the matrix
    stop(("The score ending the sequence appears only one time."))
  return(list("transition_matrix" = p, "score_value" = score_value))
}

#' @title Sampling function for Markov chains
#' @description Creates Markov chains based on a transition matrix. Can be used as parameter for the Monte Carlo function.
#' @details The transition matrix is considered representing the transition from one score to another such that the score in the first row is the lowest 
#' and the last row are the transitions from the highest score to another. The matrix must be stochastic (no rows filled up with only '0' values).
#' @param matrix transition matrix of Markov process
#' @param length length of sequence to be sampled
#' @param initialIndex (optional) index of matrix which should be initial value of sequence. If none supplied, a value from the stationary distribution is sampled as initial value.
#' @param score (optional) a vector representing the scores (in ascending order) of the matrix index. If supplied, the result will be a vector of these values.
#' @return a Markov chain sampled from the transition matrix 
#' @details
#' It is possible to have the same score for different states of the markov chain. 
#' If no score supplied, the function returns a markov chain with state in 1:ncol(matrix)
#' @examples
#' B <-  matrix (c(0.2, 0.8, 0.4, 0.6), nrow = 2, byrow = TRUE)
#' transmatrix2sequence(B, length = 10)
#' transmatrix2sequence(B, length = 10, score = c(-2,1))
#' transmatrix2sequence(B, length = 10, score = c("A","B"))
#' MyTransMat <-
#'    matrix(c(0.3,0.1,0.1,0.1,0.4,
#'             0.2,0.2,0.1,0.2,0.3,
#'             0.3,0.4,0.1,0.1,0.1,
#'             0.3,0.3,0.1,0.0,0.3,
#'             0.1,0.1,0.2,0.3,0.3),
#'             ncol = 5, byrow=TRUE)
#' MySeq.CM <- transmatrix2sequence(matrix = MyTransMat,length=90, score =c(-2,-1,0,2,3))
#' MySeq.CM
#' @export
transmatrix2sequence <- function(matrix, length, initialIndex, score) {
  if (!is.matrix(matrix))
    stop("[Invalid Input] transition matrix must be of type matrix.")
  if (nrow(matrix) != ncol(matrix))
    stop("[Invalid Input] transition matrix must be a square matrix.")
  if (!missing(initialIndex) && (initialIndex > nrow(matrix)))
    stop("[Invalid Input] Initial index must be within transition matrix size.")
  if (length <= 0)
    stop("[Invalid Input] Please supply a positive length for the sequence to be built.")
  if (!missing(score) && (length(score) != ncol(matrix)) )
    stop("[invalid input] the size of the transition matrix and the dimension of the score vector should match.")
  #check if initial index exists: if not, use stationary distribution to dermine the first element
  #if stationary fails, abort
  #1 Create Initial Element
  markovChain <- vector(mode = "integer", length = length)
  if (missing(initialIndex)) {
    init_distribution <-  stationary_distribution(matrix)
    first_elem <-  sample(1:length(init_distribution), size = 1, prob = init_distribution)
    markovChain[1] <- first_elem
  } else {
    markovChain[1] <- initialIndex
  }
  #2 Create Sequence with Transition Matrix
  for (i in 2:length) {
    markovChain[i] <- sample(1:ncol(matrix), size = 1, prob = matrix[markovChain[i - 1],])
  }
  #3 Map Score to Index if parameter 'score' is specified
  if (!missing(score)) {
    markov_sequence <- score[markovChain] 
    if (is.numeric(score) && all(score %% 1 == 0)) #integer values
      storage.mode(markov_sequence) <- "integer"
    return(markov_sequence)
  } else
    return(markovChain)
}

loadCharSequencesFromFile <- function(filepath){
  if (missing(filepath))
    filepath = file.choose()
  return(File2CharSequences(filepath))
}

#' @title Load score from file
#' @description Reads a csv file containing an alphabet, the associated scores, and eventually the associated probabilities.
#' @param filepath optional: location of file on disk. If not provided, a file picker dialogue box will be opened.
#' @param header does the file contain a header line (defaut to \code{TRUE})
#' @param ... optional: use arguments from read.csv
#' @details
#' The file should contains a header and 2 or 3 columns : first column the letters, second column the associated scores, optional third column associated probabilities. Letters should be unique and probabilities should sum to 1.
#' 
#' @return A data.frame. Rownames correspond to the first column, usually Letters. Associated numerical scores are in the second column. If probabilities are provided, 
#' they will be loaded too and presumed to be in the third column
#' @export
loadScoreFromFile <- function(filepath, header = TRUE, ...){
  #Pour info : do.call(rbind,dic) renvoie un dataframe classique à 1 ou 2 colonnes avec la lettre comme 'rowname'
  if (!missing(filepath) && !is.character(filepath))
    stop("[Invalid Input] Filepath must be a string.")
  if (!missing(filepath) && !file.exists(filepath))
    stop("[Invalid Input] File does not exist.")
  if (missing(filepath))
    filepath <- file.choose()
  dic <- read.csv(filepath, header = header, ...)
  #File content checks
  if (!is.numeric(dic[,2]))
    stop("This file is incompatible with the required format. Second column should contains numerical values")
  if (length(dic) > 2) {
    if (!sum(dic[,3]) == 1)
      stop("This file is incompatible with the required format. Third column should sum to 1 (probabilities)")
    if (!all(dic[,3] >= 0) && !all(dic[,3] <= 1))
      stop("This file is incompatible with the required format. Third column values should be between 0 and 1 (probabilities)")
  }
  if (anyDuplicated(dic[,1]) != 0)
    stop("This file is incompatible with the required format. First column should not contain duplicated values.")
  rownames(dic) <- dic[,1]
  dic[,1] <- NULL
  return(dic)
}

#' @title Loads matrix from csv-File
#' @description Reads a csv file without header and returns the matrix. For file formats please see section "File Formats" in vignette.
#' @param filepath optional: Location of file on disk. If not provided, a file picker dialog will be opened.
#' @return A Matrix Object
#' @export
loadMatrixFromFile <- function(filepath){
  if (!missing(filepath) && !is.character(filepath))
    stop("[Invalid Input] Filepath must be a string.")
  if (!missing(filepath) && !file.exists(filepath))
    stop("[Invalid Input] File does not exist.")
  if (missing(filepath))
    filepath <- file.choose()
  obj <- read.csv(filepath, header = FALSE)
  return(data.matrix(obj))
}

#' @title Convert a character sequence into a score sequence
#' @description Convert a character sequence into a score sequence. See CharSequences2ScoreSequences() function for several sequences
#' @param charseq a character sequence, given as a string
#' @param dictionary a data.frame with rownames containing letters, first column containing associated scores, optional second column containing associated probabilities
#' @return a vector of a score sequence
#' @seealso \code{\link{CharSequences2ScoreSequences}}
#' @examples
#' data(Seq31)
#' Seq31
#' data(HydroScore)
#' CharSequence2ScoreSequence(Seq31,HydroScore)
#' @export
CharSequence2ScoreSequence <- function(charseq, dictionary){
  string <- strsplit(charseq, "")[[1]]
  if (!is.data.frame(dictionary)) {
    if (!is.list(dictionary)) {
      stop("[Error Input] Bad dictionary format")
    } else {
      dictionary <- old2newDico(dictionary)
    }
  }
  unname(unlist(dictionary[string,1]))
}

# internal Utilitary function used to convert old format of dictionary to new format
# old format : named list of list of 1 or 2 element
# new format : dataframe with row names qual to letters, first column equal to score,
# and optionnal second column equal to probability of scores
# example : localScore:::old2newDico(Hydroscore)
old2newDico <- function(oldDico)
{
  return(as.data.frame(do.call(rbind, oldDico)))
}

#' @title Convert several character sequences into score sequences
#' @description Convert several character sequence into score sequences. For only one sequence see CharSequence2ScoreSequence() function.
#' @param sequences a list of character sequences given as string
#' @param dictionary a data.frame with rownames containing letters, first column containing associated scores, optional second column containing associated probabilities
#' @return a list of score sequences
#' @seealso \code{\link{CharSequence2ScoreSequence}}
#' @examples
#' data(Seq31)
#' Seq31
#' data(Seq219)
#' Seq219
#' data(HydroScore)
#' MySequences=list("A1"=Seq31,"A2"=Seq219)
#' CharSequences2ScoreSequences(MySequences,HydroScore)
#' @export
CharSequences2ScoreSequences <- function(sequences, dictionary){
  result <- list()
  if (is.null(names(sequences)))
    names(sequences) <- 1:length(sequences)
  for (name  in names(sequences))
    result[[name]] <- CharSequence2ScoreSequence(sequences[[name]], dictionary)
  return(result)
}
#reads file from filepath as Fasta
File2CharSequences <- function(filepath) {
  sequenceList <- list()
  add <- FALSE
  title <- ""
  con <- file(filepath, "r")
  while (TRUE) {
    line <- readLines(con, n = 1)
    if (length(line) == 0) {
      break
    }
    if (add) {
      sequenceList[[title]] <- line
      add <- FALSE
    }
    if (substr(line, 1, 1) == ">") {
      add <- TRUE
      title <- substr(line, 2, nchar(line))
      title <- strsplit(title, ' ')[[1]][[1]]
      #cut comment if any:
      
    }
  }
  close(con)
  return(sequenceList)
}

#' @title Empirical distribution from sequences
#' @description Builds empirical distribution from a list of numerical sequences
#' @details By determining the extreme scores in the sequences, this function creates a vector of probabilities
#' including values that do not occur at all. In this it differs from table(). For example, two sequences containing
#' values from 1:2 and 5:6 will produce a vector of size 6. 
#' @param sequences list of numerical sequences
#' @return empirical distribution from minimum score to maximum score as a vector of floating numbers.
#' @examples
#' seq1 = sample(7:8, size = 10, replace = TRUE)
#' seq2 = sample(2:3, size = 15, replace = TRUE)
#' l = list(seq1, seq2)
#' scoreSequences2probabilityVector(l)
#' @export
scoreSequences2probabilityVector <- function(sequences) {
  #1 concat sequences
  master_seq <- unlist(sequences)
  #2 find max and min
  max <- max(master_seq)
  min <- min(master_seq)
  result <- rep(0, length = max - min + 1)
  names(result) <- min:max
  #3 count occurence of each
  prob <- prop.table(table(master_seq))
  result[names(prob)] <- prob
  return(result)
}

#' @title Automatic analysis
#' @description Calculates local score and p-value for sequence(s) with integer scores.
#' @details This method picks the adequate p-value method for your input.\cr 
#' If no sequences are passed to this function, it will let you pick a FASTA file.\cr 
#' If this is the case, and if you haven't provided any score system 
#' (as you can do by passing a named list with the appropriate scores for each character),
#' the second file dialog which will pop up is for choosing a file containing the score 
#' (and if you provide an extra column for the probabilities, they will be used, too - see
#' section File Formats in the vignette for details).\cr 
#' The function then either uses empirical distribution based on your input - or if you provided
#' a distribution, then yours - to calculate the p-value based on the length of each of the sequences
#' given as input. \cr 
#' You can influence the choice of the method by providing the modelFunc argument. In this case, the
#' function uses exclusively simulation methods (monteCarlo, karlinMonteCarlo).  \cr 
#' By setting the method_limit you can further decide to which extent computation-intensive methods (daudin, exact_mc)
#' should be used to calculate the p-value.
#' Remark that the warnings of the localScoreC() function have be deleted when called by automatic_analysis() function
#' @param sequences sequences to be analyzed (named list)
#' @param scores vector of minimum and maximum score range
#' @param transition_matrix if the sequences are markov chains, this is their transition matrix
#' @param distribution vector of probabilities in ascending score order (iid sequences). Note that names of the vector must be the associated scores.
#' @param modelFunc function to create similar sequences. In this case, Monte Carlo is used to calculate p-value
#' @param method_limit limit length from which on computation-intensive exact calculation methods for p-value are replaced by approximative methods
#' @param score_extremes a vector with two elements: minimal score value, maximal score value
#' @param simulated_sequence_length if a modelFunc is provided and the sequence happens to be longer than method_limit, the method karlinMonteCarlo
#' is used. This method requires the length of the sequences that will be created by the modelFunc for estimation of Gumble parameters.
#' @param ... parameters for modelFunc
#' @param model the underlying model of the sequence (either "iid" for identically independently distributed variable or "markov" for Markov chains)
#' @return A list object containing
#' \item{Local score}{local score...}
#' \item{p-value}{p-value ...}
#' \item{Method}{the method used for the calculus of the p-value}
#' @examples
#' # Minimal example
#' l = list()
#' seq1 = sample(-2:1, size = 3000, replace = TRUE)
#' seq2 = sample(-3:1, size = 150, replace = TRUE)
#' l[["hello"]] = seq1
#' l[["world"]] = seq2
#' automatic_analysis(l, "iid")
#' # Example with a given distribution 
#' automatic_analysis(l,"iid",scores=-3:1,distribution=c(0.3,0.3,0.1,0.1,0.2))
#' #Equivalent call
#' automatic_analysis(l,"iid",score_extremes=c(-3,1),distribution=c(0.3,0.3,0.1,0.1,0.2))
#' # forcing the exact method for the longest sequence
#' aa1=automatic_analysis(l,"iid")
#' aa1$hello$`method applied`
#' aa1$hello$`p-value`
#' aa2=automatic_analysis(l,"iid",method_limit=3000)
#' aa2$hello$`method applied`
#' aa2$hello$`p-value`
#' # Markovian example 
#' MyTransMat <-
#' matrix(c(0.3,0.1,0.1,0.1,0.4, 0.3,0.2,0.2,0.2,0.1, 0.3,0.4,0.1,0.1,0.1, 0.3,0.3,0.3,0.0,0.1, 
#'         0.1,0.3,0.2,0.3,0.1), ncol = 5, byrow=TRUE)
#' MySeq.CM=transmatrix2sequence(matrix = MyTransMat,length=150, score =-2:2)
#' MySeq.CM2=transmatrix2sequence(matrix = MyTransMat,length=110, score =-2:2)
#' automatic_analysis(sequences = list("x1" = MySeq.CM, "x2" = MySeq.CM2), model = "markov")
#' @export
automatic_analysis <- function(sequences, model, scores, transition_matrix, distribution, method_limit = 2000, score_extremes, modelFunc, simulated_sequence_length = 1000, ...){
  #1 Check if the model of the sequences can be deduced
  if ((missing(model) && missing(transition_matrix) && missing(distribution)) || (missing(model) && !missing(transition_matrix) && !missing(distribution)))
    stop("unclear which model is used for sequence creation. Please provide either the model to be used, a transition matrix or a distribution vector.")
  #2 if model is missing, deduce it from perhaps present transition matrix
  if (missing(model)) {
    if (missing(transition_matrix))
      model <- "iid"
    else
      model <- "markov"
  }
  #3 sequences provided? If not, ask user to pick file
  if (missing(sequences)) {
    sequence_filepath <- file.choose()
    sequences <- loadCharSequencesFromFile(sequence_filepath)
  }
  #4 Sequences numeric? If yes, no scores needed, else pick file and use it
  if (!is.numeric(sequences[[1]]) && missing(scores)) {
    score_filepath <- file.choose()
    scores <- loadScoreFromFile(score_filepath)
    sequences <- CharSequences2ScoreSequences(sequences, scores)
  } 
  else if (!is.numeric(sequences[[1]]) && !missing(scores)) {
    #use scores provided in argument
    sequences <- CharSequences2ScoreSequences(sequences, scores)
  }
  
  #4.1 declare result list
  results <- list()
  #5 if extremes of scores are missing, extract them from sequences
  if (missing(score_extremes)) {
    max <- sequences[[1]][[1]]
    min <- sequences[[1]][[1]]
    for (sequence in sequences) {
      if (max(sequence) > max)
        max <- max(sequence)
      if (min(sequence) < min)
        min <- min(sequence)
    }
    score_extremes <- c(min, max)
  }
  #6 Setup Progress Bar
  if (interactive())
  progressBar <- txtProgressBar(min = 0, max = length(sequences), initial = 0, char = "=",
                               width = NA, "progress in sequence analysis", "progressbar", style = 3, file = "")
  progressCounter = 0;
  #7 model specific behaviour
  if (model == "markov") {
    #7.1 check if transition matrix available: If not, build from sequences
    if (missing(transition_matrix)) {
      tmp <- sequences2transmatrix(sequences)
      transition_matrix <- tmp$transition_matrix
      score_values <- tmp$score_values
      }
    #7.2 check if modelFunc provided
    if (missing(modelFunc)) {
      #use exact Method if sequences are not too long
      for (name in names(sequences)) {
        #calculate local score
        localscore <- localScoreC(sequences[[name]],suppressWarnings = TRUE)
        sequence_length <- length(sequences[[name]])
        if (sequence_length <= method_limit) {
          method <- "Exact Method"
          p_value <- exact_mc(localscore$localScore[1], transition_matrix, sequence_length, score_values)
#          p_value <- exact_mc(transition_matrix, localscore$localScore[1], sequence_length, sequence_min = score_extremes[1], sequence_max = score_extremes[2])
          results[[name]] <- list("p-value" = p_value, "method applied" = method, "localScore" = localscore)
        } else {
          warning(paste(c("[Ignoring Sequence] sequence length ", name, " is of length ", sequence_length), collapse = ""))
          results[[name]] <- list("localScore" = localscore)
        }
        progressCounter <- progressCounter + 1
        if (interactive())
          setTxtProgressBar(progressBar, progressCounter)
      }
    } else {
      #Function supplied, using simulation methods for Markov Chain p value determination
      for (name in names(sequences)) {
        #calculate local score
        localscore <- localScoreC(sequences[[name]],suppressWarnings = TRUE)
        sequence_length <- length(sequences[[name]])
        if (sequence_length <= method_limit) {
          method <- "Monte Carlo"
          p_value <- monteCarlo(localscore$localScore[1], modelFunc, ...)
          results[[name]] <- list("p-value" = p_value, "method applied" = method, "localScore" = localscore)
        } else {
          if (mean(sequence[[name]]) >= 0) {
            warning(paste0(c("[Ignoring Sequence] sequence mean >= 0 and sequence length = ", sequence_length)))
            results[[name]] <- list("localScore" = localscore)
          }
          else{
            method <- "Asymptotic Method by Monte Carlo"
            res <- karlinMonteCarlo(local_score = localscore$localScore[1], FUN = modelFunc, sequence_length = sequence_length, simulated_sequence_length = simulated_sequence_length , ...)
            results[[name]] = list("p-value" = res$p_value, "method applied" = method, "localScore" = localscore, "K*" = res$`K*`, "lambda" = res$lambda)
          }
        }
        progressCounter <- progressCounter + 1
        if (interactive())
        setTxtProgressBar(progressBar, progressCounter)
      }
    }
  } 
  else if (model == "iid") {
    #7.1 check if probability vector provided or in score (loading from file)
    if (missing(distribution)) {
      if (!missing(scores) && dim(scores)[2] > 1 ) {
        #distribution provided with score: adjust extremes if file has greater extreme values
        distribution <- scoreDictionnary2probabilityVector(scores)
        score_extremes <- c(min(as.numeric(names(distribution))), max(as.numeric(names(distribution))))
      } else {
        #learn from sequences
        distribution <- scoreSequences2probabilityVector(sequences)
      }
    } else if (!missing(scores)) {
      names(distribution) <- scores
    } else if (!missing(score_extremes)) {
      names(distribution) <- score_extremes[1]:score_extremes[2]
    } else {
      stop("[Bad Input] no score values associated to distribution. Please provide one either by parameter 'scores' or 'score_extreme'.")
    }
  
    # Moyenne des scores sur l'ensemble de la liste
    MeanDistribution <- sum(as.numeric(names(distribution))*distribution)
    
    if (missing(modelFunc)) {
      for (name in names(sequences)) {
        #calculate local score
        localscore <- localScoreC(sequences[[name]], suppressWarnings = TRUE)
        sequence_length <- length(sequences[[name]]) 
        if (sequence_length <= method_limit) {
          method <- "Exact Method Daudin et al"
          p_value <- daudin(localscore$localScore[1], sequence_length, distribution, score_extremes[1], score_extremes[2])
          results[[name]] <- list("p-value" = p_value, "method applied" = method, "localScore" = localscore)
        }
        else if (sequence_length <= method_limit*10) {
          if (MeanDistribution < 0) {
            method <- "Asymptotic Method Karlin et al"
            p_value <- karlin(localscore$localScore[1], sequence_length, distribution, score_extremes[1], score_extremes[2])
            results[[name]] <- list("p-value" = p_value, "method applied" = method, "localScore" = localscore)
          } else {
            warning(paste0(c("[Ignoring Sequence] sequence mean >= 0 and sequence length = ", sequence_length)))
            results[[name]] <- list("localScore" = localscore)
          }
        } else {
          if (MeanDistribution < 0) {
            method <- "Improved Asymptotic Method Mercier et al"
            p_value <- mcc(localscore$localScore[1], sequence_length, distribution, score_extremes[1], score_extremes[2])
            results[[name]] <- list("p-value" = p_value, "method applied" = method, "localScore" = localscore)
          } else {
            warning(paste0(c("[Ignoring Sequence] sequence mean >= 0 and sequence length = ", sequence_length)))
            results[[name]] <- list("localScore" = localscore)
          }
        }
        progressCounter <- progressCounter + 1
        if (interactive())
        setTxtProgressBar(progressBar, progressCounter)      
      }
    } else {
      #Function supplied, using simulation methods for iid p value determination
      for (name in names(sequences)) {
        #calculate local score
        localscore <- localScoreC(sequences[[name]],suppressWarnings = TRUE)
        sequence_length <- length(sequences[[name]])
        if (sequence_length <= method_limit) {
          method <- "Monte Carlo"
          p_value <- monteCarlo(localscore$localScore[1], FUN = modelFunc, ...)
          results[[name]] <- list("p-value" = p_value, "method applied" = method, "localScore" = localscore)
        } else {
          if (MeanDistribution >= 0) {
            warning(paste(c("[Ignoring Sequence] sequence mean >= 0 and sequence length = ", sequence_length), collapse = ""))
            results[[name]] <- list("localScore" = localscore)
          } else {
            method <- "Asymptotic Method by Monte Carlo"
            res <- karlinMonteCarlo(local_score = localscore$localScore[1], FUN = modelFunc, sequence_length = sequence_length, ...)
            results[[name]] <- list("p-value" = res$p_value, "method applied" = method, "localScore" = localscore, "K*" = res$`K*`, "lambda" = res$lambda)
          }
        }
        progressCounter = progressCounter + 1
        if (interactive())
        setTxtProgressBar(progressBar, progressCounter)
      }
    }
  } 
  else {
    stop("unknown model. Please mention either iid or markov.")
  }
  if (interactive())
  close(progressBar)
  return(results)
}



#' @title Check for missing scores values in the score distribution
#' @description Get extremes scores, then create a complete list and set a probability equal to zeros for not present scores
#' @param listScore list of vector containing each (score value, probability)
#' @param score_extremes (optional) vector of min and max value of scores (used for truncation).
#' @return vector containing all score values between extremes and the probability equal to 0 for missing score
#' @examples
#' Mylist <- list("x1"=c(-2,0.1),"x2"=c(0,0.7),"x3"=c(1,0.2))
#' scoreDictionnary2probabilityVector(list = Mylist, score_extremes = c(-2,1))
#' @noRd
scoreDictionnary2probabilityVector <- function(listScore, score_extremes = NULL){
  score <- do.call(rbind, listScore)
  probs <- score[,2]
  names(probs) <- score[,1]

 #check for missing values and set zeros if not present
 #get extremes, then create list and check difference
 missingScores <- setdiff(min(score[,1]):max(score[,1]), score[,1])
 missingProbs <- rep(0,length(missingScores))
 names(missingProbs) <- missingScores
 probs <- c(probs, missingProbs)
 #order from lowest to highest score
 probs <- probs[order(as.numeric(names(probs)))]
 #si la fonction est appele avec des valeurs "limitantes" on coupe ce qui depasse ces valeurs
 if (!is.null(score_extremes)) {
   probs <- subset(probs, as.numeric(names(probs)) >= score_extremes[1] & as.numeric(names(probs)) <= score_extremes[2])
 }
 return(probs)
}

#' @title Convert a real scores vector into an integer scores vector
#' @description Convert real scores into integer scores
#' @details Convert real scores into integer scores by multiplying real
#' scores by a coefficient (default 10) and then assigning probability to corresponding
#' extended (from the minimum to the maximum) integer scores
#' @param RealScore vector of real scores
#' @param ProbRealScore vector of probability
#' @param coef coefficient
#' @return list containing ExtendedIntegerScore and ProbExtendedIntegerScore
#' @examples
#' score <- c(-1,-0.5,0,0.5,1)
#' prob.score <- c(0.2,0,0.4,0.1,0.3)
#' (res1 <- RealScores2IntegerScores(score, prob.score, coef=10))
#' prob.score.err <- c(0.1,0,0.4,0.1,0.3)
#' (res2 <- RealScores2IntegerScores(score, prob.score.err, coef=10))
#' # When coef=1, the function can handle integer scores
#' ex.integer.score <- c(-3,-1,0,1, 5)
#' (res3 <- RealScores2IntegerScores(ex.integer.score, prob.score, coef=1))
#' @export
RealScores2IntegerScores <- function(RealScore, ProbRealScore, coef = 10)
{
  if (sum(ProbRealScore) != 1) warning("Sum of probabilities is different from 1")
  
  IntegerScore <- RealScore * coef
  
  MinIntegerScore <- min(IntegerScore)
  MaxIntegerScore <- max(IntegerScore)
  
  ExtendedIntegerScore <- as.integer(MinIntegerScore:MaxIntegerScore)
  
  ProbExtendedIntegerScore <- rep(0, times = length(ExtendedIntegerScore))
  
  ProbExtendedIntegerScore[ExtendedIntegerScore %in% IntegerScore] <- ProbRealScore
  names(ProbExtendedIntegerScore) <- ExtendedIntegerScore
  
  list(ExtendedIntegerScore = ExtendedIntegerScore,
       ProbExtendedIntegerScore = ProbExtendedIntegerScore)
}

#' @title Tool function used in calculus of probability of excursion (Markov case). Return the vector z
#'
#' @param a score strictly positive
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#'
#' @return Return the vector z
#' @examples
#' theta = c("a","b","c","d")
#' lambda = matrix(c(0.1,0.2,0.4,0.3,
#'                   0.3,0.1,0.5,0.1,
#'                   0.2,0.6,0.1,0.1,
#'                   0.4,0.1,0.1,0.4),
#'                 ncol=4, byrow = TRUE, dimnames = list(theta,theta))
#' all(apply(lambda,1,sum)==1) #TRUE for markov transition matrix
#' stationary_dist = stationary_distribution(lambda))
#' score_function = c(a=-3,b=-1,c=1,d=2)
#' sum(score_function*stationary_dist) #Score expectation (should be <0)
#' proba_excursion_markov_calcul_z(3, theta, lambda, score_function)
#' @noRd
proba_excursion_markov_calcul_z <- function(a, theta, lambda, score_function) {
  # RM théoriquement plus utilisée (le calcul de z est effectué directement dans proba_theoretical_first_excursion_alpha)
  list.theta <- as.list(theta)
  list.a <- as.list(c(-1:a))
  combination <- as.vector(outer(list.theta,list.a, paste, sep = ":"))
  
  P <- create_matrix_P(a, theta, lambda, score_function, combination)
  
  Q_C <- P[,-(length(combination) - length(theta) + 1):-(length(combination))]
  Q_C <- Q_C[,-1:-length(theta)]
  Q_B <- P[,1:length(theta)]                                            
  Q_A <- P[,(length(combination) - length(theta) + 1):length(combination)]
  
  return(z = (solve(diag(ncol(Q_C)) - Q_C)) %*% (Q_A %*% rep(1, ncol(Q_A))))
}

#' @title Used to generate markov's sequences (can output sequence of cumulative scores or simple sequences of scores)
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of size, theta x theta
#' @param prob0 stationary distribution of lambda 
#' @param n length of generated Markov's sequence
#' @return vector containing the generated Markov's score sequence
#' @examples
#' hist_S = generate_markov_hist_score(theta, lambda, score_function, prob0, n)
#' @noRd
generate_markov_hist_score <- function(theta, lambda, score_function, prob0, n){
  card <- length(theta)
  previous_index <- sample(card, size = 1, replace = TRUE, prob = prob0)
  hist_score <- rep(NA, length = (n + 1))
  hist_score[1] <- 0
  hist_score[2] <- score_function[previous_index]
  
  for (x in 3:n ) {
    previous_index <- sample(1:card, size = 1, replace = TRUE, prob = lambda[previous_index,])
    hist_score[x] <- hist_score[x - 1] + score_function[previous_index]
  }
  
  return(hist_score)
}

#' @title Used generate a markov score sequence and extract its first excursion
#' @param a score
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param prob0 stationary distribution of lambda 
#' @param n length of generated Markov's chain
#' @return list containing three elements : hist_S      : cumulative score sequence
#'                                          hist_S_star : cumulative score sequence filled after the first excurtion with -1 if the score is not reached andor else with the max reached score 
#' @examples
#' hist_s_star = generation_empirical_processes(a, theta, lambda, score_function, prob0, n)$hist_S_star
#' @noRd
generation_empirical_processes <- function(a, theta, lambda, score_function, prob0 = NULL, n) {
  if (is.null(prob0)){
    prob0 = stationary_distribution(lambda)
  }
  
  hist_S = generate_markov_hist_score(theta, lambda, score_function, prob0, n)
  
  hist_S_star = rep(NA, length=(n))
  hist_S_star[1] = 0
  
  for (i in 2:(length(hist_S))) {
    #print(hist_S[i])
    if (0 <= hist_S[(i-1)] & hist_S[(i-1)] < a) {
      hist_S_star[(i-1)] = hist_S[(i-1)]
    } else if ( hist_S[(i-1)] >= a ) {
      hist_S_star[(i-1):n] = rep(a, length = ( (n+1) - (i-1)) )
      return(list(hist_S = hist_S, hist_S_star = hist_S_star))
    } else if ( hist_S[(i-1)] < 0  ) {
      hist_S_star[(i-1):n] = rep(-1, length = ( (n+1) - (i-1)) )
      return(list(hist_S = hist_S, hist_S_star = hist_S_star))
    }
  }
  return(list(hist_S = hist_S, hist_S_star = hist_S_star))
}

#' @title return the empirical probability of reaching a score a on the first excursion of a markov's score sequence
#' @param a score
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param prob0 stationary distribution of lambda
#' @param n length of Markov's chain
#' @param N number of repetition of the process
#' @return reel number between 0 and 1 representing the empirical probability of reaching a score of a
#'  on the first excursion 
#' @examples
#' probability = localScore::proba_empirical_first_excursion(5, c("K","L","M"), matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), c(-2,-1,2), c(0.4444444, 0.2962963, 0.2592593), n = 1000, N = 10000)
#' @noRd
proba_empirical_first_excursion_markov <- function(a, theta, lambda, score_function, prob0, n, N) {
  s = 0
  for (x in 1:N) {
    hist_s_star = generation_empirical_processes(a, theta, lambda, score_function, prob0, n)$hist_S_star
    if (hist_s_star[n] == a) {
      s = s + 1
    }
  }
  return(s/N)
}

#' @title return the empirical probability of reaching a score a on the ith (with i >= 2) excursion of a markov's score sequence
#' @param a score
#' @param theta vector containing the alphabet used
#' @param lambda transition probability matrix of size, theta x theta
#' @param score_function vector containing the scores of each letters of the alphabet (must be in the same order as theta)
#' @param prob0 stationary distribution of lambda
#' @param n length of Markov's chain
#' @param N number of repetition of the process
#' @param i wanted excursion
#' @return reel number between 0 and 1 representing the empirical probability of reaching a score of a on the ith excursion 
#' @examples
#' probability = localScore::proba_empirical_ith_excursion(c("K","L","M"), 5, matrix(c(0.5, 0.3, 0.2, 0.4, 0.2, 0.4, 0.4, 0.4, 0.2), ncol = 3, byrow = TRUE), c(-2,-1,2), c(0.4444444, 0.2962963, 0.2592593), i = 2, n = 1000, N = 10000)
#' @noRd
proba_empirical_ith_excursion_markov <- function(a, theta, lambda, score_function, prob0, n, N, i) {
  c = 0
  for ( it in 1:(N+1) ) {
    partial_sum = generation_empirical_processes(a, theta, lambda, score_function, prob0, n)
    partial_sum = partial_sum$hist_S
    index = 0
    exc   = 1
    val_prev_exc = 0
    
    while ( ( exc < i ) & index < ( n - 1 ) ) {
      if ( partial_sum[index+1] >= val_prev_exc ) {
        index = index + 1
      } else {
        exc = exc + 1
        val_prev_exc = partial_sum[index+1]
      }
    }
    
    while ( partial_sum[index+1] >= val_prev_exc & index < ( n - 1 ) & partial_sum[index+1] < ( a + val_prev_exc ) ) {
      index = index + 1
    }
    
    if ( partial_sum[index+1] >= ( a + val_prev_exc ) & exc == i ) {
      c = c + 1
    }
    
  }
  return (c/N)
}

##########################################################

context("Score local detection and Record times")
#library(localScore)

test_that("localScoreC and recordTimes results checks", {
  ##This check 2 things :
  #  1. output of localScoreC is expected (all field)
  #  2. output of recordTimes is expected AND is equal to the output of localScoreC()$RecordTime[-1]
  #Helper function to build object return by localScoreC
  built_expected_results  <-  function(localscore, suboptimalSeg, RecordTime) {
    names(localscore) <- c("value", "begin", "end")
    suboptimalSeg <- as.data.frame(suboptimalSeg) # un truc rapide pour ne pas récrire tous les tests (qui utilisent matrix)
    colnames(suboptimalSeg) <- c("value", "begin", "end")
    resLS <- list(localScore = localscore, suboptimalSegmentScores = suboptimalSeg, RecordTime = RecordTime)
    resRecordFn <- RecordTime[-1]
    return(list(resLS = resLS, resRecordFn = resRecordFn))
  }
  # Helper function to call localScoreC() AND recordTimes()
  built_call <- function(score_sequence) {
    resLS <- localScoreC(score_sequence, suppressWarnings = TRUE)
    resRecordFn <- recordTimes(score_sequence)
    return(list(resLS = resLS, resRecordFn = resRecordFn))
  }
 
  # Test with all null values as input
  expect_equal(built_call(c(0,0,0,0,0)), built_expected_results(localscore = c(0,0,0), 
                                                                suboptimalSeg =  matrix(c(0,0,0), ncol = 3, byrow = TRUE),
                                                                RecordTime = 0))
  
  # Test with all null values as input
  expect_equal(built_call(c(0,0,0,0,0)), built_expected_results(localscore = c(0,0,0), 
                                                                suboptimalSeg =  matrix(c(0,0,0), ncol = 3, byrow = TRUE),
                                                                RecordTime = 0))
  # Test with all negative values as input
  expect_equal(built_call(c(-1,-1,-1,-1,-1)), built_expected_results(localscore = c(0,0,0), 
                                                                     suboptimalSeg =  matrix(c(0,0,0), ncol=3, byrow=TRUE),
                                                                     RecordTime = 0:5))
  # Test with all positive values as input
  expect_equal(built_call(c(1,2,3,4,5)), built_expected_results(localscore = c(15,1,5), 
                                                                suboptimalSeg =  matrix(c(15,1,5), ncol=3, byrow=TRUE),
                                                                RecordTime = 0))
  # Test local score in the middle and sequence beginning with a mountain
  expect_equal(built_call(c(1,-2,1,3,-5)), built_expected_results(localscore = c(4,3,4), 
                                                                  suboptimalSeg =  matrix(c(1,1,1,
                                                                                            4,3,4), ncol=3, byrow=TRUE),
                                                                  RecordTime = c(0,2,5)))
  # Test local score in the middle and sequence ending with a mountain
  expect_equal(built_call(c(-2,1,1,-5,1)), built_expected_results(localscore = c(2,2,3), 
                                                                  suboptimalSeg =  matrix(c(2,2,3,
                                                                                            1,5,5), ncol=3, byrow=TRUE),
                                                                  RecordTime = c(0,1,4)))
  #built_call(c(3,-2,1,1,-5)) (problème pour suboptimal : soit une seule montagne de 1 à 4 soit 2 montagnes [1-1] [3-4])
  # Pour le dire autrement (?) : la fonction localScoreC ne renvoie pas le "même" résultat sur cette séquence, et sa séquence prise dans l'autre sens.
  
  expect_equal(built_call(c(-1,3,-2,1,1,-5)), built_expected_results(localscore = c(3,2,2), 
                                                                     suboptimalSeg =  matrix(c(3,2,2), ncol=3, byrow=TRUE),
                                                                     RecordTime = c(0, 1, 6)))
  # lindley finishing in 0 (not below 0)
  expect_equal(built_call(c(-1,3,-2,1,1,-3)), built_expected_results(localscore = c(3,2,2), 
                                                                     suboptimalSeg =  matrix(c(3,2,2), ncol=3, byrow=TRUE),
                                                                     RecordTime = c(0, 1)))
  
  expect_equal(built_call(c(1,-2,1,1,-5)), built_expected_results(localscore = c(2,3,4), 
                                                                  suboptimalSeg =  matrix(c(1,1,1,
                                                                                            2,3,4), ncol=3, byrow=TRUE),
                                                                  RecordTime = c(0,2,5)))
  # Test two mountains in the middle
  expect_equal(built_call(c(-2,1,1,-5,1,-4)), built_expected_results(localscore = c(2,2,3), 
                                                                     suboptimalSeg =  matrix(c(2,2,3,
                                                                                               1,5,5), ncol=3, byrow=TRUE),
                                                                     RecordTime = c(0, 1, 4, 6)))
})

test_that("localScoreC_double and recordTimes results checks", {
  ##This check 2 things :
  #  1. output of localScoreC_double is expected (all field)
  #  2. output of recordTimes is expected AND is equal to the output of localScoreC_double()$RecordTime[-1]
  #Helper function to build object return by localScoreC_double

  built_expected_results_double  <-  function(localscore, suboptimalSeg, RecordTime) {
    names(localscore) <- c("value", "begin", "end")
    colnames(suboptimalSeg) <- c("value", "begin", "end")
    resLS <- list(localScore = localscore, suboptimalSegmentScores = suboptimalSeg, RecordTime = RecordTime)
    resRecordFn <- RecordTime[-1]
    return(list(resLS = resLS, resRecordFn = resRecordFn))
  }
  # Helper function to call localScoreC() AND recordTimes()
  built_call_double <- function(score_sequence) {
    resLS <- localScoreC_double(score_sequence, suppressWarnings = TRUE)
    resRecordFn <- recordTimes(score_sequence)
    return(list(resLS = resLS, resRecordFn = resRecordFn))
  }
  
  # Test with all null values as input
  expect_equal(built_call_double(0.1*c(0,0,0,0,0)), built_expected_results_double(localscore = c(0.0,0.0,0.0), 
                                                                suboptimalSeg =  data.frame(0,0,0),
                                                                RecordTime = 0))
  
  # Test with all negative values as input
  expect_equal(built_call_double(0.1*c(-1,-1,-1,-1,-1)), built_expected_results_double(localscore = c(0.0,0.0,0.0),  
                                                                                suboptimalSeg =  data.frame(0,0,0),
                                                                                RecordTime = 0:5))
  # Test with all positive values as input
  expect_equal(built_call_double(0.1*c(1,2,3,4,5)), built_expected_results_double(localscore = c(1.5,1,5), 
                                                                suboptimalSeg =  data.frame(1.5,1,5),
                                                                RecordTime = 0))
  # Test local score in the middle and sequence beginning with a mountain
  expect_equal(built_call_double(0.1*c(1,-2,1,3,-5)), built_expected_results_double(localscore = c(0.4,3,4), 
                                                                  suboptimalSeg =  data.frame(
                                                                    value = c(0.1,0.4),
                                                                    begin = c(1,3),
                                                                    end = c(1,4)),
                                                                  RecordTime = c(0,2,5)))
  # Test local score in the middle and sequence ending with a mountain
  expect_equal(built_call_double(0.1*c(-2,1,1,-5,1)), built_expected_results_double(localscore = c(0.2,2,3), 
                                                                  suboptimalSeg =  data.frame(
                                                                    value = c(0.2,0.1),
                                                                    begin = c(2,5),
                                                                    end = c(3,5)),
                                                                  RecordTime = c(0,1,4)))
  #built_call_double(0.1*c(3,-2,1,1,-5)) (problème pour suboptimal : soit une seule montagne de 1 à 4 soit 2 montagnes [1-1] [3-4])
  # Pour le dire autrement (?) : la fonction localScoreC ne renvoie pas le "même" résultat sur cette séquence, et sa séquence prise dans l'autre sens.
  
  expect_equal(built_call_double(0.1*c(-1,3,-2,1,1,-5)), built_expected_results_double(localscore = c(0.3,2,2), 
                                                                     suboptimalSeg =  data.frame(
                                                                       value = 0.3,
                                                                       begin = 2,
                                                                       end = 2),
                                                                     RecordTime = c(0, 1, 6)))
  # lindley finishing in 0 (not below 0)
  expect_equal(built_call_double(0.1*c(-1,3,-2,1,1,-3)), built_expected_results_double(localscore = c(0.3,2,2), 
                                                                                suboptimalSeg =  data.frame(
                                                                                  value = 0.3,
                                                                                  begin = 2,
                                                                                  end = 2),
                                                                                RecordTime = c(0, 1)))
  
  expect_equal(built_call_double(0.1*c(1,-2,1,1,-5)), built_expected_results_double(localscore = c(0.2,3,4), 
                                                                             suboptimalSeg =  data.frame(
                                                                               value = c(0.1,0.2),
                                                                               begin = c(1,3),
                                                                               end = c(1,4)),
                                                                             RecordTime = c(0,2,5)))
  # Test two mountains in the middle
  expect_equal(built_call_double(0.1*c(-2,1,1,-5,1,-4)), built_expected_results_double(localscore = c(0.2,2,3), 
                                                                                suboptimalSeg =  data.frame(
                                                                                  value = c(0.2,0.1),
                                                                                  begin = c(2,5),
                                                                                  end = c(3,5)),
                                                                     RecordTime = c(0, 1, 4, 6)))
})


test_that("Errors and warnings checks", {
  # Test warning when no local score found in input sequence
  expect_warning(localScoreC(c(-1,-1,-1,-1,-1), suppressWarnings=FALSE),"No local score found")
  # Test warning when postive mean of input sequence
  expect_warning(localScoreC(c(-1,10,-1,-1,-1), suppressWarnings=FALSE),"The mean of this sequence is greater than 0. The sequence may be trivial")
  # Test warning when no local score found in input sequence
  expect_warning(localScoreC_double(c(-0.1,0.-1,0.-1,0.-1,0.-1), suppressWarnings=FALSE),"No local score found")
  # Test warning when postive mean of input sequence
  expect_warning(localScoreC_double(c(-0.1,1.0,-0.1,-0.1,-0.1), suppressWarnings=FALSE),"The mean of this sequence is greater than 0. The sequence may be trivial")
  })


test_that("Type output", {
  expect_false(is.integer(localScoreC(c(-2,1,1,-5,1,-4))$localScore))  #attention à ce comportement
  expect_true(is.integer(localScoreC(as.integer(c(-2,1,1,-5,1,-4)))$localScore))  #attention à ce comportement
  expect_false(is.integer(localScoreC(0.5 + c(-2,1,1,-5,1,-4))$localScore))
  expect_true(is.numeric(localScoreC(0.5 + c(-2,1,1,-5,1,-4))$localScore))
})

#' @keywords internal
#' Package for sequence analysis by local score
#'
#' @md
#' @description
#' Provides functionalities for:
#'   * calculating the local score
#'   * calculating statistical relevance (p-value) to find a local Score in a sequence of given distribution
#' 
#' Given a sequence of numerical score \eqn{X_1,\dots,X_n}, the local score is defined : \eqn{H_n = \max_{1 \leq i \leq j \leq n} \sum_{l = i}^j X_l}
#' This package find the value local score and the associated sub-sequence, and also sub-optimal local scores and segments. The complexity is linear with \eqn{n}. It can be viewed as a generalization of a sliding window method, considering all windows size. In order to be pertinent, the expectation of the scores \eqn{X_i} should be negative. Most of the methods concerning statistical relevance implemented in this package only applied on integer scores.
#' 
#' @details
#' Please refer to the vignette of this package or the manual for details on how to use this package.
#' 
#' @references 
#' * An Improved Approximation For Assessing The Statistical Significance of molecular Sequence Features, Mercier and al 2003
#' * Exact distribution for the local score of one i.i.d. random sequence, Sabine Mercier and JJ Daudin, 2001
#' * Limit Distributions of Maximal Segmental Score among Markov-Dependent Partial Sums, Karlin and Dembo 1992
#' * Methods for assessing the statistical significance of molecular sequence features by using general scoring schemes, Karlin and al 1990
#' * Detection de courts segments inverses dans les genomes: methodes et applications, David Robelin 2005
#' * A Linear Time Algorithm for Finding All Maximal Scoring Subsequences, Constable and Bates 1985
#' 
"_PACKAGE"
  
## usethis namespace: start
## usethis namespace: end
NULL

---
title: "Local Score Package"
author: "Sébastien Déjean - Sabine Mercier - Sebastian Simon - David Robelin"
date: "`r Sys.Date()`"
output:
  pdf_document:
    toc: yes
  html_document:
    df_print: paged
    toc: yes
vignette: >
  %\VignetteIndexEntry{Local Score Package} 
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown} 
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  warning = FALSE,
  collapse = TRUE,
  comment = "#>"
)

# Source : https://yihui.org/rmarkdown-cookbook/time-chunk
# all_times <- list()  # store the time for each chunk
# knitr::knit_hooks$set(time_it = local({
#   now <- NULL
#   function(before, options) {
#     if (before) {
#       # record the current time before each chunk
#       now <<- Sys.time()
#     } else {
#       # calculate the time difference after a chunk
#       res <- difftime(Sys.time(), now, units = "secs")
#       all_times[[options$label]] <<- res
#       # return a character string to show the time
#       paste("Time for the code chunk", options$label, " to run:", round(res,
#         2), "seconds")
#     }
#   }
# }))
# knitr::opts_chunk$set(time_it = FALSE)

```

```{r setup, echo=FALSE}
library(localScore)
```

# Citation

Please cite : Robelin, D.; Déjean, S.; Mercier, S. localScore: an R package to highlight optimal and suboptimal segments in a sequence with associated p-values computation. Peer Community Journal, Volume 5 (2025), article no. e141. https://doi.org/10.24072/pcjournal.650

# Introduction

Main purpose: given a numerical sequence of numerical scores, find the maximum sum subsequence value among all possible subsequences. This value is called the local Score.

This package provides functionalities for two main tasks:
1- Calculating the local Score of a given score sequence or, of a given component sequence and a given scoring scheme.
2- Calculating the statistical relevance ($p$-value) of a given local Score, associated to a given sequence length and a given distribution for the model.

Also, the package deals with sub-optimal local scores, that is all strictly positive sum subsequences. Since the second version of this package, it can also calculates the $p$-value of a sub-optimal local scores given its position in sequential order.

This second version of the package deals with a model of independent and identically distributed (I.I.D.) and markov dependent sequences. 

If your in a hurry, the section "In brief" presents the main functions and there typical uses. Details and other useful functions are presented in the remaining of this vignette.

# In brief

In all the examples below, the score expectation should strictly negative, so that the local score has a sense.

## Case of I.I.D. integer score sequence

### Generating an I.I.D. score sequence for this example
```{r}
score.values <- -3:2                                       # Possible score values
score.probability <- c(0.4, 0.0, 0.1, 0.0, 0.2, 0.3)       # Associated score probabilities
score.expectation <- sum(score.values * score.probability) # Score expectation
print(score.expectation)
n <- 1000                                            # sequence size  
score.sequence <- sample(score.values, n, replace = TRUE, prob = score.probability)
```

### Graphical representation via the Lindley process

```{r}
score.lindley <- lindley(score.sequence)
oldpar <- par(no.readonly = TRUE)
par(mfrow = c(2,1))
plot(score.sequence, typ = 'l', main = "Sequence score")
plot(score.lindley, typ = 's', main = "Associated Lindley process")
par(oldpar)
```

### Finding the maximal sum subsequence and all strictly positive sum subsequences

```{r}
segments.sequence <- localScoreC(score.sequence)
localScore.sequence <- segments.sequence$localScore # Local score and position of the segment
print(localScore.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores # suboptimal local scores and positions
print(head(subLocalScore.sequence))
```

### Calculating p-value of local score

```{r}
# Exact p-value (computational limitation, see help(daudin))
daudin(localScore.sequence["value"], 
       sequence_length = n,
       score_probabilities = score.probability,
       sequence_min = min(score.values),
       sequence_max = max(score.values))
# Karlin and Dembo approximation (n big)
karlin(localScore.sequence["value"], 
       sequence_length = n,
       score_probabilities = score.probability,
       sequence_min = min(score.values),
       sequence_max = max(score.values))
# Improved Karlin and Dembo approximation (computational limitation, see help(mcc))
mcc(localScore.sequence["value"], 
       sequence_length = n,
       score_probabilities = score.probability,
       sequence_min = min(score.values),
       sequence_max = max(score.values))
```

## Case of markov integer score sequence

### Generating a markov score sequence for this example

```{r}
transitionMatrix <- matrix(c(0.2, 0.3, 0.5,
                             0.3, 0.4, 0.3,
                             0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
score.values <- c(-3, -1, 2)
row.names(transitionMatrix) <- score.values
score.stationary.distribution <- stationary_distribution(transitionMatrix)
score.expectation <- sum(score.values * score.stationary.distribution)
print(score.expectation)
# Generating example markov sequence of score:
n <- 10000
score.sequence <- transmatrix2sequence(matrix = transitionMatrix,
                                       length = n,
                                       score = score.values)
head(score.sequence)
```

### Finding the maximal sum subsequence and all strictly positive sum subsequences

```{r}
segments.sequence <- localScoreC(score.sequence)
localScore.sequence <- segments.sequence$localScore # Local score and position of the segment
print(localScore.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores # suboptimal local scores and positions
print(head(subLocalScore.sequence))
```

### Calculating p-value of local score

```{r}
# Exact p-value (computational limitation, see help(exact_mc))
exact_mc(local_score = localScore.sequence["value"],
         m = transitionMatrix,
         sequence_length = n,
         prob0 = score.stationary.distribution)
```

## Case of various dependent structure sequence via Monte-Carlo approach

A monte-Carlo function is implemented in this package and allows to compute the $p$-value of the local score in case
of diverse model generating the score sequence. We present an example of use here.

### Generating a complex dependent structure score sequence for this example

```{r}
# Some sort of drifting markov sequence generator
sequence.generator <- function(n, P1, P2, score_values) {
  nstate <- dim(P1)[1]
  sequence.sim <- rep(NA, n)
  sequence.sim[1] <- sample(1:nstate, 1, prob = stationary_distribution(P1))
  for (i in 2:n) {
    P <- (n - i) / (n - 1) * P1 + (i - 1) / (n - 1) * P2
    sequence.sim[i] <- sample(1:nstate, 1, prob = P[sequence.sim[i - 1],])
  }
  return(score_values[sequence.sim])
}

P1 <- matrix(c(0.2, 0.3, 0.5,
               0.3, 0.4, 0.3,
               0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
P2 <- matrix(c(0.2, 0.1, 0.7,
               0.6, 0.4, 0.0,
               0.8, 0.2, 0.2), byrow = TRUE, ncol = 3)
score.values <- c(-4, -1, 2)
n <- 500
score.sequence <- sequence.generator(n, P1, P2, score.values)
head(score.sequence)
mean(score.sequence) # Expected to be strictly negative
```

### Graphical representation via the Lindley process

```{r}
score.lindley <- lindley(score.sequence)
x <- 1:n
# lw1 <- loess(score.sequence ~ x)
oldpar <- par(no.readonly = TRUE)
par(mfrow = c(2,1))
plot(score.sequence, typ = 'l', col = "grey", main = "Sequence score")
# lines(x, lw1$fitted[x], col = "red")}}}
plot(score.lindley, typ = 's', main = "Associated Lindley process")
par(oldpar)
```

### Finding the maximal sum subsequence and all strictly positive sum subsequences

```{r}
segments.sequence <- localScoreC(score.sequence)
localScore.sequence <- segments.sequence$localScore # Local score and position of the segment
print(localScore.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores # suboptimal local scores and positions
print(head(subLocalScore.sequence))
```

### Calculating p-value of local score by Monte Carlo simulation

```{r}
p.value <- monteCarlo(local_score = localScore.sequence["value"],
           FUN = function(n, P1, P2, score_values) {
             return(sequence.generator(n, P1, P2, score_values))
             },
           n = n, P1 = P1, P2 = P2, score_values = score.values,  #generative function parameters
           plot = TRUE,
           numSim = 1000 # low number here to compute the vignette in a "short" time
          )
print(p.value)
```

## Case of  $i$th excursion (sequential order)

Below is a markov example, but also available for i.i.d. case.

### Generating a markov score sequence for this example
```{r}
transitionMatrix <- matrix(c(0.2, 0.3, 0.5,
                             0.3, 0.4, 0.3,
                             0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
score.values <- c(-3, -1, 2)
row.names(transitionMatrix) <- score.values
score.stationary.distribution <- stationary_distribution(transitionMatrix)
score.expectation <- sum(score.values * score.stationary.distribution)
print(score.expectation)
# Generating example markov sequence of score:
n <- 10000
score.sequence <- transmatrix2sequence(matrix = transitionMatrix,
                                       length = n,
                                       score = score.values)
```

### Finding the maximal sum subsequence and all strictly positive sum subsequences

```{r}
segments.sequence <- localScoreC(score.sequence)
subLocalScore.sequence <- segments.sequence$suboptimalSegmentScores
```

### Calculating $p$-value of $i$th excursion

```{r}
iVisualExcursion <- 1 #first strictly positive excursion
iSegment <- subLocalScore.sequence[iVisualExcursion,]
print(iSegment)
# determining i in the sense of Karlin and Dembo (1990) (i >= iVisualExcursion)
i <- sum(segments.sequence$RecordTime <= iSegment$begin)
print(i)
proba_theoretical_ith_excursion_markov(iSegment$value, score.values, transitionMatrix, score.values, i = i)
```

# Local Score computation methods

First defined in *Karlin and Altschul (1990)*, it represents the value of the highest scoring segment in a sequence of scores (formula $H_n$ below). It corresponds to the highest cumulated subsequence sum amongst all possible subsequences (independent of length). 
For the score to be relevant, the expectation of the sequence should be negative. Thus, for a sequence of interest, the possible score of sequence components should be positive and negative for the local score have to be meaningful.

$$H_n = max_{1 \leq i \leq j \leq n} \sum_{l=i}^j X_l $$

## A first example: function ``localScoreC()''

Let us assume a score function taking its values in [-2, -1, 0, 1, 2]. A sample score sequence of length 100 could be

```{r}
mySeq <- sample(-2:2, 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
mySeq
scoreSequenceExpectation <- sum(c(-2:2)*c(0.5, 0.3, 0.05, 0.1, 0.05))
scoreSequenceExpectation
localScoreC(mySeq)

```

The result is a maximum score and for which subsequence it has been found: the starting position "[" and the end of it ("]").
It also yields all other subsequences with a score equal or less and their positions in the `$suboptimalSegmentScores` matrix. Note that those subsequences do not have common positions.
"Stopping Times" are local minima in the cumulated sum of the sequence and correspond to the beginning of excursions (potential segments of interest). The "end" of the segment is the position realizing the (sub)-maximum of the segments of the sequence.

Another example with missing score values.

```{r}
mySeq <- sample(c(-3,2,0,1,5), 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
head(mySeq)
localScoreC(mySeq)
```

Remark: in the case where the local score is realized by more that one segment with the same starting position, the shortest one is chosen. In the example below, the local score of the `seq2` sequence is 6 and there are two segments starting at position 5 which realized it: the first finished at position 6, and the second finished at position 8. In this case, the function `localScoreC` retains the shortest segment.
```{r}
seq1 <- c(1,-2,3,1,-1,2)
seq2 <- c(1,-2,3,1,-1,2,-1,1)
localScoreC(seq1)
localScoreC(seq2)
```

## Example with real scores

```{r}
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score_reels, proba.sple = proba_score_reels,
                               length.sple = 10)
localScoreC(seq.essai) 
```

## Example of alphabetical sequence associated to a scoring function

```{r}
# Loading a fasta protein
data(Seq219)
Seq219
# or using your own fasta sequence
#MySeqAA_P49755 <- as.character(read.table(file="P49755.fasta",skip=1)[,1])
#MySeqAA_P49755
# Loading a scoring function
data(HydroScore)
?HydroScore
# or using your own scoring function
# HydroScoreKyte<- loadScoreFromFile("Kyte1982.txt")
# Transforming the amino acid sequence into a score sequence with the score function
# in HydroScore file
SeqScore_P49755 <- CharSequence2ScoreSequence(Seq219, HydroScore)
head(SeqScore_P49755)
length(SeqScore_P49755)
# Computing the local score
localScoreC(SeqScore_P49755)
```

> Note that the scoring function (here `HydroScore`) could be read from a dedicated file using `loadScoreFromFile()```. See Section "File format" for more details.

# $p$-Value computation methods

There are different methods available to establish the statistical significance, also called $p$-value, of the local score depending on the length of the sequence and the score expectation. 
This value describes the probability to encounter a given local score for a given score distribution or higher for a given sequence length. Therefore it allows to determinate if the local score in question is significant or could have been obtained by chance. Since the second version of this package, two probabilistic models for the sequence are available:
- Identically and Independently Distributed Variables model (I.I.D.)
- Markovian model

For an Identically and Independently Distributed Variables model (I.I.D.), the main function of the packages to calculate the $p$-value of the local score are: `daudin()`, `mcc()`, `karlin()`, `monteCarlo()` and `KarlinMonteCarlo()`, each of them correspond to diverse probability methods found in the literature. In the Markovian model case, the main functions are `exact_mc()` and `monteCarlo()`.
The following sections illustrate and detail the context and use of these functions.
In the I.I.D. case, note than we advise to use the exact method `daudin()` if possible, then `mcc()`, then `karlin()`.
In any case, a more computational intensive Monte Carlo approach is always possible (`monteCarlo()` and `KarlinMonteCarlo()`), and even allow to take into account more complex sequence models using specialized random generation functions.

## Simulating computation: functions "monteCarlo()"

The function `monteCarlo()` simulates a number of score sequences similar (same distribution) to the one having yielded the local score in question. Therefore, it requires a function that produces such sequences and the parameters used by this function. See the help page and the following example to use the empirical distribution of a given sequence, but any other function, such as `rbinom()` or custom functions, are valid too.

In the following example we search the probability to obtain a local score of 10 in the sequence we created in the previous section and which serves as a blueprint for the score sequences to produce. The return value is the $p$-value of the local score for the given score sequence. A plot of the distribution of all local scores simulated and the cumulative distribution function are displayed. These plots can be hidden by setting the argument `plot` to `FALSE`. The number of sequences simulated in our example is 1000, a default value, and can be changed by setting the argument "numSim" to an appropriate value.

Note that the cumulative distribution function plot indicates $P( LocalScore\leq\cdot)$ and so the corresponding $p$-value equals 1 minus the cumulative distribution function value.

```{r,  fig.width=7.2, fig.height=4}
monteCarlo(local_score = 10, FUN = function(x) {
  return(sample(x = x, size = length(x), replace = TRUE))
}, x = SeqScore_P49755)
```

The use of this method depends of computing power, number of simulations, implementation of the simulating function and the length of the sequence. For long sequences, you may prefer the next method which combines simulating and approximated methods and called `KarlinMonteCarlo()` (see next section).

```{r}
# Example
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
monteCarlo(5.5,
  FUN = sample_from_model, plot = TRUE, score.sple = score_reels, proba.sple = proba_score_reels,
  length.sple = 100, numSim = 1000
)
```

## A mixed method: functions "karlinMonteCarlo()"

The function `karlinMonteCarlo()` also uses a function supplied by the user to do simulations. However, it does not deduce directly the $p$-value from the cumulative distribution function. However, this function is used to estimate the parameters of the Gumbel distribution of *Karlin and al.* approximation. Thus, it is suited for long sequences. Note: `simulated_sequence_length` is the length of the simulated sequences. This value must correspond to the length of sequences yielded by FUN. The value of `n` is the sequence length for which we want to compute the $p$-value. If `n` is too large function `MonteCarlo()` could be too much time consuming. Using `karlinMonteCarlo()` with a smaller sequence length `simulated_sequence_length` allows to extract the parameters and then to apply them to the sequence value `n`.

```{r,  fig.width=7.2, fig.height=4}
fu <- function(n, size, prob, shift) {
  rbinom(size = size, n = n, prob = prob) + shift
}
# Note: (maybe) too small simulated_sequence_length value, to reduce vignette build time
# Look at the linear regression graph to adjust this parameter
karlinMonteCarlo(12,
  FUN = fu, n = 10000, size = 8, prob = 0.2, shift = -2,
  sequence_length = 1000000, simulated_sequence_length = 5000
) 
```
If not specified otherwise, the function produces three graphes, two of them like the function `monteCarlo()` and the third a representation of $\ln(-\ln(cf))$, with $cf$ being the cumulated function, showing the linear regression in green color providing the parameters for the Gumbel distribution $K^*$ and $\lambda$.

Example for real scores:

```{r,  fig.width=7.2, fig.height=4}
score_reels <- c(-1.5, -0.5, 0, 0.5, 1.5)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
fu <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
# Note: (maybe) too small simulated_sequence_length value, to reduce vignette build time
# Look at the linear regression graph to adjust this parameter
karlinMonteCarlo(85.5,
  FUN = fu, score.sple = score_reels, proba.sple = proba_score_reels,
                 length.sple = 10000, numSim = 1000, sequence_length = 100000, 
  simulated_sequence_length = 5000
)
```

## Exact method for integer scores: function ``daudin()''

The exact method calculates the $p$-value exploiting the fact of the "stopped" Lindley process, stopped at value the local score is a Markov process. Therefore, an exact $p$-value can be retrieved. The complexity of matrix multiplication involved being $>O(n^2)$, the method can be unsuited for sequences of either great length, $n\geq 10^4$ for example could take too much time for computation, or dispersed scores.  Note that the exact method requires integer scores.

```{r}
daudin(
  local_score = 15, sequence_length = 500, score_probabilities =
    c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1), sequence_min = -3, sequence_max = 2
)
```

This function is based on: *S. Mercier and J.J. Daudin 2001: "Exact distribution for the local score of one i.i.d. random sequence"*

## How to use the exact method for real scores

The exact method requires integer score to be used. For real scores, a homothetic transformation can be considered to be able to use the exact method. This transformation is theoretically validated to still provide an exact probability. The only existing drawback of the homothetic transformation is that the computation time is increasing.
We can check in the following example that the $p$-value computation using the exact method with an homothetic transformation corresponds, approximately, to the real local score $p$-value computed with the Monte Carlo method.

```{r}
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score_reels, proba.sple = proba_score_reels,
                               length.sple = 100)
localScoreC(seq.essai) 
C <- 10 # homothetic coefficient
localScoreC(as.integer(C*seq.essai))
```

We can check that the local score of the sequence which has been homothetically transformed is multiplied by the same coefficient. We are going to compute the $p$-value. For this we need to create the integer score vector and its corresponding distribution from the ones dedicated to real scores:

```{r}
RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)
M.s.r <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ExtendedIntegerScore
M.s.prob <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ProbExtendedIntegerScore
M.SL <- localScoreC(as.integer(C * seq.essai))$localScore[1]
M.SL
pval.E <- daudin(
  local_score = M.SL, sequence_length = length(seq.essai), score_probabilities = M.s.prob,
  sequence_min = -10, sequence_max = 10
)
pval.E
```

Let us compare the result of the exact method with homothetical transformation with MonteCarlo method for the initial sequence and real scores

```{r}
SL.real <- localScoreC(seq.essai)$localScore[1]
SL.real
pval.MC <- monteCarlo(
  local_score = SL.real, FUN = sample_from_model,
  score.sple = score_reels, proba.sple = proba_score_reels,
  length.sple = length(seq.essai), plot = TRUE, numSim = 5000
)
pval.MC
```

We can check that the two  probabilities are close: `r pval.E` for the exact method and `r pval.MC` for Monte Carlo's one. The difference comes from the fact that the Monte Carlo method produces an approximation.

## Approximate method of Karlin *et al.*: function ``karlin()''

The method of Karlin uses the local score's distinctive cumulative distribution following a law of Gumbel to approximate the $p$-value. It is suited for large and very large sequences as the approximation is asymptotic with the sequence length and so more accurate for large sequences ; and secondly very large sequence case can be too much time and space consuming for the exact method whereas Karlin *et al.* method does not depend to the sequence length for the computational criteria. The average score must be non positive.

```{r}
score.v <- -2:1
score.p <- c(0.3, 0.2, 0.2, 0.3)
sum(score.v*score.p)
karlin(
  local_score = 14, sequence_length = 100000, sequence_min = -2, sequence_max = 1,
  score_probabilities = c(0.3, 0.2, 0.2, 0.3)
)
karlin(
  local_score = 14, sequence_length = 1000, sequence_min = -2, sequence_max = 1,
  score_probabilities = c(0.3, 0.2, 0.2, 0.3)
)
```
We verify here that the same local score value 14 is more usual for a longer sequence.

```{r}
# With missing score values
karlin(
  local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 1,
  score_probabilities = c(0.3, 0.2, 0.0, 0.2, 0.3)
)
```

This function is based on: Karlin *et al.* 1990: *"Methods for assessing the statistical significance of molecular sequence features by using general scoring schemes"*

The function `Karlin()` is dedicated for integer scores. For real ones the homothetic solution presented for the exact method can also be applied.

## An improved approximate method: function ``mcc()''

The function `mcc()` uses an improved version of the Karlin's method to calculate the $p$-value. It is suited for sequences of length upper or equal to several hundreds.  Let us compare the three methods on the same case.

```{r}
mcc(
  local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 2,
  score_probabilities = c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1)
)

daudin(
  local_score = 14, sequence_length = 1000, score_probabilities =
    c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1), sequence_min = -3, sequence_max = 2
)
karlin(
  local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 2,
  score_probabilities = c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1)
)
```
We can observe than the improved approximation method with `mcc()` gives a $p$-value equal to `r mcc(local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 2,
    score_probabilities = c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1))` which is more accurate than the one of `karlin()` function equal to `r karlin(local_score = 14, sequence_length = 1000, sequence_min = -3, sequence_max = 2,
       score_probabilities = c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1))` compared to the exact method which computation equal to `r daudin(local_score = 14, sequence_length = 1000, score_probabilities = 
         c(0.2, 0.3, 0.3, 0.0, 0.1, 0.1), sequence_min = -3, sequence_max = 2)`.
       
This function is based on the work of  S. Mercier, D. Cellier and D. Charlot 2003 *"An improved approximation for assessing the statistical significance of molecular sequence features''*.

## An automatic method: function ``automatic_analysis()''
This function is meant as a support for the inexperienced user. Since the use of methods for $p$-value requires some understanding on how these methods work, this function automatically selects an adequate methods based on the sequence given.

There are different use-case scenarios for this function. One can just put a sequence and the model (Markov chains or identically and independently distributed).
```{r}
automatic_analysis(sequences = list("x1" = c(1,-2,2,3,-2, 3, -3, -3, -2)), model = "iid")
```
In the upper example, the sequence is short, so the exact method is adapted. Here is another example. As the sequence is much more longer, the asymptotic approximation of Karlin *et al.* can be used. This is possible because the average score is negative. If not the `MonteCarlo` method could have been preferred by the function.

```{r}
score <- c(-2, -1, 0, 1, 2)
proba_score <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sum(score*proba_score)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score, proba.sple = proba_score, length.sple = 5000)
MyAnalysis <- automatic_analysis(
  sequences = list("x1" = seq.essai),
  distribution = proba_score, score_extremes = c(-2, 2), model = "iid"
)$x1
MyAnalysis$"p-value"
MyAnalysis$"method applied"
MyAnalysis$localScore$localScore
```

For real score, achieved the homothetic transformation before. If not, the results could not be correct.


```{r}
score_reels <- c(-1, -0.5, 0, 0.5, 1)
proba_score_reels <- c(0.2, 0.3, 0.1, 0.2, 0.2)
sample_from_model <- function(score.sple, proba.sple, length.sple) {
  sample(score.sple,
    size = length.sple, prob = proba.sple, replace = TRUE
  )
}
seq.essai <- sample_from_model(score.sple = score_reels, proba.sple = proba_score_reels, length.sple = 1000)

# Homothetie
C <- 10
RealScores2IntegerScores(score_reels,proba_score_reels, coef=C)
M.s.r <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ExtendedIntegerScore
M.s.prob <- RealScores2IntegerScores(score_reels, proba_score_reels, coef = C)$ProbExtendedIntegerScore
# The analysis
MyAnalysis <- automatic_analysis(
  sequences = list("x1" = as.integer(C * seq.essai)), model = "iid",
  distribution = M.s.prob, score_extremes = range(M.s.r)
)
MyAnalysis$x1$"p-value"
MyAnalysis$x1$"method applied"

# Without the homothety, the function gives a wrong result
# MyAnalysis2 <- automatic_analysis(sequences = list("x1" = seq.essai), model = "iid")
# MyAnalysis2$x1$"p-value"
# MyAnalysis2$x1$"method applied"


```

Whenever there is no distribution given, these is learned from the sequence(s) given. The sequence(s) must be passed as **named list**.
If there are more than one sequence, all sequences are treated. A progress bar informs the user about the progress made.
If the sequence(s) passed are not score sequences but sequence(s) of letters, a file picker dialog pops up. Here, the user can choose a file containing a mapping from letters to scores. The format is csv (view section "File Formats" for details) and allows also to provide a distribution that is loaded concurrently to the score. 
One can also chose not to provide sequences at all. In this case, the first dialog that pops up is for selection of a FASTA file (view section "File Formats" for details).
Either way, the user has little influence on the method applied to find the $p$-value. One can change the argument `method_limit` to modify the threshold for the use of exact and approximating methods or supply a function for simulation methods which will result in the use of a simulation method. In this case, make sure to provide a suitable `simulated_sequence_length` argument, as for long sequences, the method `monteCarloKarlin` will be used.

## Markovian model of the sequence : function `exact_mc()`

A markovian dependency of the components of the sequence can be taken into account to calculate the $p$-value of the local score. Note that memory usage and time computation can be too large for a high local score value and high score range, as this method computation needs to allocate a square matrix of size localScore^(range(score_values)). This matrix is then exponentiated to `sequence_length`. So, be aware to only use this function for values respecting your hardware configuration. As an alternative, the `monteCarlo()` function can be used.
See an example of use below. Note that the function `stationary_distribution()` calculates the stationary distribution of the markovian sequence. The probability of the first score can be specified. If not, the stationary distribution is used meaning that the markovian is supposed to be at the stationary state.
```{r}
scoreValues <- c(-2, -1, 2)
mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
initialProb <- stationary_distribution(mTransition)
exact_mc(local_score = 50, m = mTransition, sequence_length = 100, 
        score_values = scoreValues, prob0 = initialProb)
```
Note that if you name the transition matrix rows with the score values, you can omit the score_values parameter. Example below.
```{r}
scoreValues <- c(-2, -1, 2)
mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
rownames(mTransition) <- scoreValues
initialProb <- stationary_distribution(mTransition)
exact_mc(local_score = 50, m = mTransition, sequence_length = 100)
```

Last, note the existence of the function `transmatrix2sequence()` which simulate a markovian sequence given a transition matrix and a initial score value in option. This can be useful in combination with `monteCarlo()` function. Example:

```{r}
MyTransMat <-
  matrix(c(
    0.3, 0.1, 0.1, 0.1, 0.4, 0.2, 0.2, 0.1, 0.2, 0.3, 0.3, 0.4, 0.1, 0.1, 0.1, 0.3, 0.3, 0.1, 0.0, 0.3,
    0.1, 0.1, 0.2, 0.3, 0.3
  ), ncol = 5, byrow = TRUE)

MySeq.CM <- transmatrix2sequence(matrix = MyTransMat, length = 150, score = -2:2)
MySeq.CM
AA.CM <- automatic_analysis(sequences = list("x1" = MySeq.CM), model = "markov")
AA.CM
```

With MonteCarlo method the local score approximate $p$-value is also not significant.

```{r}
Ls.CM <- AA.CM$x1$localScore[[1]][1]
monteCarlo(
  local_score = Ls.CM,
  FUN = transmatrix2sequence, matrix = MyTransMat,
  length = 150, score = -2:2,
  plot = FALSE, numSim = 5000
)
```


# Other Functions

The package provides a number of auxiliary functions for loading or creating sequences for different models.

## Lindley Process: to visualize optimal and suboptimal segments

This function takes a score sequence and shows the Lindley process associated, illustrating the operation of the local score algorithm. Plotting the Lindley process provides a view of the potential optimal segments and their comparison between them, and offers a good representation of potential signal along the sequence, better alternative than a sliding window approach as it captures both a punctual high score value and a succession of small positive score values.
```{r,fig.width=7.2}
set.seed(1)
mySeq <- sample(c(-3,2,0,1,5), 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
lindley(mySeq)
oldpar <- par(no.readonly = TRUE)
par(mfrow = c(2,1))
plot(lindley(mySeq), type = "s")
plot(mySeq,typ = 'l')
par(oldpar)
```


## Record times: gives the record times of a sequence

This function takes a score sequence and return a vector with the record times defined as follow: $K_{0} = 0,$ and $K_{i+1} := inf\{k > K_i : S_k - S_{K_i} < 0\}$, for $i \geq 0$.
```{r}
set.seed(1)
mySeq <- sample(c(-3,2,0,1,5), 100, replace = TRUE, prob = c(0.5, 0.3, 0.05, 0.1, 0.05))
mySeq
recordTimes(mySeq)
```

## Score Loading Function

`loadScoreFromFile()` reads a csv-file returning a named list where names correspond to the first file column and values to the second file column. If a third column is available within the file, it will be read and can be accessed by name, too. An example is given in the first case study. The function is reading a header line by default, so be careful that you have one otherway the first score will be missed.

## Empirical distribution: function "scoreSequences2probabilityVector()"
`scoreSequences2probabilityVector()` takes in a list of score sequences and returns the resulting empirical distribution from the minimal to the maximal score value of all sequences. Thus,

```{r}
seq1 <- sample(7:8, size = 10, replace = TRUE)
seq2 <- sample(2:3, size = 15, replace = TRUE)
l <- list(seq1, seq2)
r <- scoreSequences2probabilityVector(l)
r
length(r)
```

returns a vector of length 7, even if there are only 4 distinct unique values present in the list. Very useful for the use in any non-simulating method of $p$-value.


# Case study

## Medium sequence

Sequence extracted from https://www.uniprot.org/uniprot/P49755/

```{r}
data(Seq219)
data(HydroScore)
SeqScore <- CharSequence2ScoreSequence(Seq219, HydroScore)
n <- length(SeqScore)
n
```


### Local score computation and parameter model setings

```{r}
LS <- localScoreC(SeqScore)$localScore[1]
LS
```

### Parameter model setings
```{r}
prob <- scoreSequences2probabilityVector(list(SeqScore))
prob
``` 

### Exact method

```{r}
time.daudin <- system.time(
  res.daudin <- daudin(
    local_score = LS, sequence_length = n,
       score_probabilities = prob,
       sequence_min = min(SeqScore),
    sequence_max = max(SeqScore)
  )
)
res.daudin
```


### Approximated method

The call of the function `karlin()` is similar to the one of `daudin()`.

```{r}
time.karlin <- system.time(
  res.karlin <- karlin(
    local_score = LS, sequence_length = n,
       score_probabilities = prob,
       sequence_min = min(SeqScore),
    sequence_max = max(SeqScore)
  )
)
res.karlin
```
The two $p$-values are different because the sequence length $n$ is equal `r n`. It is not enough large to have a good approximation with the approximated method.

### Improved approximation

The call of the function `mcc()' is still the same.

```{r}
time.mcc <- system.time(
  res.mcc <- mcc(
    local_score = LS, sequence_length = n,
       score_probabilities = prob,
       sequence_min = min(SeqScore),
    sequence_max = max(SeqScore)
  )
)
res.mcc
```
We can verify here that this approximation is more accurate than the one of Karlin *et al.* for sequences of length of several hundred components.


### Monte Carlo

Let us do a very quick empirical computation with only 200 repetitions.

```{r}
time.MonteCarlo1 <- system.time(
  res.MonteCarlo1 <- monteCarlo(
    local_score = LS,
    FUN = function(x) {
      return(sample(
        x = x, size = length(x),
        replace = TRUE
      ))
    },
    x = SeqScore, numSim = 200
  )
)
res.MonteCarlo1
```
The $p$-value estimation is `r res.MonteCarlo1` which is around the exact value `r res.daudin`. Let us increase the number of repetition to be more accurate.

```{r}
time.MonteCarlo2 <- system.time(
  res.MonteCarlo2 <- monteCarlo(
    local_score = LS,
    FUN = function(x) {
      return(sample(
        x = x, size = length(x),
        replace = TRUE
      ))
    },
    x = SeqScore, numSim = 5000
  )
)
res.MonteCarlo2
```

### Result and time computation comparison

$P$-value
```{r}
res.pval <- c(
  Daudin = res.daudin, Karlin = res.karlin, MCC = res.mcc,
  MonteCarlo1 = res.MonteCarlo1, MonteCarlo1 = res.MonteCarlo2
)
names(res.pval) <- c("Exact", "Approximation", "Improved appx", "MonteCarlo1", "MonteCarlo2")
res.pval
```
Computation time

```{r}
rbind(time.daudin, time.karlin, time.mcc,time.MonteCarlo1, time.MonteCarlo2)
```

Time computation for Monte-Carlo method depends to the number of repetition. For medium  sequences exact method is around 30 time more time consuming than the mcc method. Karlin's method is the fastest one but can be not accurate if the sequence are too short (here $n=219$ a couple of hundred is not enough, a thousand may be preferred to be closest to convergence).

## Short sequence

```{r}
data(Seq31)
SeqScore.Short <- CharSequence2ScoreSequence(Seq31, HydroScore)
n.short <- length(SeqScore.Short)
n.short
```

Sequence of length `r n.short`.
For short sequences, it is easier and usual to obtain a empirical positive expectation for the score. So the functions based on approximated methods can't be used and an error message is given.

```{r}
SeqScore.S <- SeqScore.Short
LS.S <- localScoreC(SeqScore.S)$localScore[1]
prob.S <- scoreSequences2probabilityVector(list(SeqScore.S))

LS.S
prob.S
```

```{r}
time.daudin <- system.time(
  res.daudin. <- daudin(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.S,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  )
)

time.karlin <- system.time(
  res.karlin <- try(karlin(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.S,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  ))
)

time.mcc <- system.time(
  res.mcc <- try(mcc(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.S,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  ))
)

# karlinMonteCarlo is meaningless for short sequence as its object is to avoid
# simulating very long sequences.
# time.karlinMonteCarlo <- system.time(
# res.karlinMonteCarlo <-
#     karlinMonteCarlo(
#       local_score = LS.S, plot = FALSE,
#                    sequence_length = n.short,
#                    simulated_sequence_length = 1000,
#                    FUN = sample, x = min(SeqScore.S):max(SeqScore.S),
#                    size = 1000, prob = prob.S, replace = TRUE,
#       numSim = 50000
#     )
# )

time.MonteCarlo <- system.time(
  res.MonteCarlo <- monteCarlo(
    local_score = LS.S, plot = FALSE,
    FUN = function(x) {
      return(sample(
        x = x, size = length(x),
        replace = TRUE
      ))
    },
    x = SeqScore.S, numSim = 5000
  )
)
```

### Results

```{r}
res.pval <- c(Daudin = res.daudin, MonteCarlo = res.MonteCarlo)
names(res.pval) <- c("Daudin", "MonteCarlo")
res.pval
rbind(time.daudin, time.MonteCarlo)
```
Here an example using another probability vector with non positive average score. In this example, the local score is very huge and realized by the whole sequence, the $p$-value is very low as confirmed by the exact method.

```{r}
set.seed(1)
prob.bis <- dnorm(-5:5, mean = -0.5, sd = 1)
prob.bis <- prob.bis / sum(prob.bis)
names(prob.bis) <- -5:5
# Score Expectation
sum((-5:5)*prob.bis)

time.mcc <- system.time(
  res.mcc <- mcc(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.bis,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  )
)

time.daudin <- system.time(
  res.daudin <- daudin(
    local_score = LS.S, sequence_length = n.short,
       score_probabilities = prob.bis,
       sequence_min = min(SeqScore.S),
    sequence_max = max(SeqScore.S)
  )
)

simu <- function(n, p) {
  return(sample(x = -5:5, size = n, replace = TRUE, prob = p))
}
time.MonteCarlo <- system.time(
res.MonteCarlo <-
    monteCarlo(
      local_score = LS.S, plot = FALSE,
      FUN = simu, n.short, prob.bis, numSim = 5000
    )
)
```
```{r}
res.pval <- c(MCC=res.mcc,Daudin = res.daudin, MonteCarlo = res.MonteCarlo)
names(res.pval) <- c("MCC", "Daudin", "MonteCarlo")
res.pval
rbind(time.mcc,time.daudin, time.MonteCarlo)
```

For short sequences, exact method is fast, more precise and must be prefered.

## Large sequence

```{r}
data(Seq1093)
SeqScore.Long <- CharSequence2ScoreSequence(Seq1093, HydroScore)
n.Long <- length(SeqScore.Long)
n.Long
SeqScore.Long <- CharSequence2ScoreSequence(Seq1093, HydroScore)
LS.L <- localScoreC(SeqScore.Long)$localScore[1]
LS.L
prob.L <- scoreSequences2probabilityVector(list(SeqScore.Long))
prob.L
sum(prob.L*as.numeric(names(prob.L))) 
```

Sequence of length `r n.Long` with a local score equal to `r LS.L`. The average score is non positive so approximated methods can be used.  

```{r echo=FALSE}
time.daudin.L <- system.time(
  res.daudin.L <- daudin(
    local_score = LS.L, sequence_length = n.Long,
       score_probabilities = prob.L,
       sequence_min = min(SeqScore.Long),
    sequence_max = max(SeqScore.Long)
  )
)

time.karlin.L <- system.time(
  res.karlin.L <- karlin(
    local_score = LS.L, sequence_length = n.Long,
       score_probabilities = prob.L,
       sequence_min = min(SeqScore.Long),
    sequence_max = max(SeqScore.Long)
  )
)

time.mcc.L <- system.time(
  res.mcc.L <- mcc(
    local_score = LS.L, sequence_length = n.Long,
       score_probabilities = prob.L,
       sequence_min = min(SeqScore.Long),
    sequence_max = max(SeqScore.Long)
  )
)

time.MonteCarlo.L <- system.time(
  res.MonteCarlo.L <-
    monteCarlo(
      local_score = LS.L,
                   FUN = sample, x = min(SeqScore.Long):max(SeqScore.Long),
                   size = n.Long, prob = prob.L, replace = TRUE, plot = FALSE,
      numSim = 5000
    )
)

# karlinMonteCarlo is meaningless for short sequence as its object is to avoid
# simulating very long sequences.
# time.karlinMonteCarlo.L <- system.time(
#   res.karlinMonteCarlo.L <-
#     karlinMonteCarlo(
#       local_score = LS.L,
#       sequence_length = n.Long,
#       simulated_sequence_length = 1000,
#       FUN = sample, x = min(SeqScore.Long):max(SeqScore.Long),
#       size = 1000, prob = prob.L, replace = TRUE,
#       numSim = 10000, plot = FALSE
#     )
# )
```

### Results 

```{r}
res.pval.L <- c(res.daudin.L, res.mcc.L, res.karlin.L, NA,res.MonteCarlo.L)
names(res.pval.L) <- c("Daudin", "MCC", "Karlin", "KarlinMonteCarlo", "MonteCarlo")
res.pval.L
```

```{r}
rbind(
  time.daudin.L, time.karlin.L, time.mcc.L, NA,
  time.MonteCarlo.L
)
```

Even for large sequences of several thousands, the exact method is still fast enough but it could become too much time consuming for a sequence data set with numerous sequences. The approximated methods must be preferred.

## Several sequences

The function `automatic_analysis()` can analysis a named list of sequences. It choose the adequate method for each sequence. Here in the following example, the exact method is used for the short sequence, whereas an 
asymptotic method is used for the long one.

```{r}
MySeqsList <- list(Seq31, Seq219, Seq1093)
names(MySeqsList) <- c("Q09FU3.fasta", "P49755.fasta", "Q60519.fasta")
MySeqsScore <- lapply(MySeqsList, FUN = CharSequence2ScoreSequence, HydroScore)
AA <- automatic_analysis(MySeqsScore, model = "iid")
AA$Q09FU3.fasta
AA$Q09FU3.fasta$`method applied`
AA$Q60519.fasta$`method applied`
```

We can observe differences between the $p$-value of the short sequence obtained in the case study for the only short sequence, and the one obtained with the automatic analysis. Note that the distribution vector of the scores used are different which induces a different $p$-value. 

```{r}
cbind(prob, prob.S, prob.L, "3 sequences" = scoreSequences2probabilityVector(MySeqsScore))
```

Using the probability vector of the three sequences to compute the $p$-value of the local score of the short sequence with the function `daudin()`, we recover an identical $p$-value than we have obtained with the `automatic analysis()`.

```{r}
daudin.bis <- daudin(local_score = LS.S, sequence_length = n.short, score_probabilities = scoreSequences2probabilityVector(MySeqsScore), sequence_max = 5, sequence_min = -5)
daudin.bis
AA$Q09FU3.fasta$`p-value`
# automatic_analysis(sequences=list('MySeq.Short'=MySeq.Short), model='iid', distribution=proba.S)

```

## A larger example with a SCOP data base

```{r}
data(HydroScore)
data(SeqListSCOPe)
MySeqScoreList <- lapply(SeqListSCOPe, FUN = CharSequence2ScoreSequence, HydroScore)
head(MySeqScoreList)
AA <- automatic_analysis(sequences = MySeqScoreList, model = "iid")
AA[[1]]
# the p-value of the first 10 sequences 
sapply(AA, function(x) {
  x$`p-value`
})[1:10]
# the 20th smallest p-values
sort(sapply(AA, function(x) {
  x$`p-value`
}))[1:20]
which(sapply(AA, function(x) {
  x$`p-value`
}) < 0.05)
table(sapply(AA, function(x) {
  x$`method`
}))
# The maximum sequence length equals 404 so it here normal that the exact method is used for all the 606 sequences of the data base 
scoreSequences2probabilityVector(MySeqScoreList)
```

# File Formats
This package allows input in file form. For the package to work, please respect the following conventions.

## Sequence Files
The package accepts files in FASTA format: 
Every sequence is preceeded by a title (marked by a ">") and a line break. One sequence takes one line, followed by a line break and a line only containing a tab.

```
>HUMAN_NM_018998_2
TGAGTAGGGCTGGCAGAGCTGGGGCCTCATGGCTGTGTAGTAGCAGGCCCCCGCCCCGCGACCTGGCCAGGCGATCACTACAGCCGCCCCTGCCGAACAG
  
>Mouse_NM_013908_3
CCCCATGAGGACCCAGAACCCTCAATGGAGAAGAGTCAGGATTTGCTGTGCTGCCAGAGTGAACTGGCCTGGTAATTACCCTGCAGCCTTTCTGGAACAG
  
>HUMAN_NM_018998_3
GTGAGCACGGGCGGCGGGTTGACCCTGCCCCCGCCCCACGCCGACAGCCTGTCCAGCCCCGGCCTCCCCACAG
  
>Mouse_NM_013908_4
GTAAGTGTGGGCATTGGGTTGGGCTACCTGTCCCATTGTGCCCTGCCAGCAGTCTGCCCAGCTGTGGCCTTCCCCCCAG
  
>HUMAN_NM_018998_5
GGTGCTCACAGCCCCAGAGACACCACTGAGGTAGGAAGCTGCCCTGGAGTGATGTCCTTGGGGCATTGGACAGGGACCCTCACCGTAGCCCTCCCTGCAG
```

## Score Files
A score file is a csv file that contains a header line and each line contains a letter and its score. Optionally one can also provide a probability for each score.
Example:

```
Letters,Scores,Probabilities  
L,-2,0.04  
M,-1,0.04  
N,0,0.04  
```

## Transition Matrix Files
A csv file only containing the values of the matrix.
Example:

```
0.2,0.3,0.5  
0.3,0.4,0.3  
0.2,0.4,0.4  
```

<!-- --- -->
<!-- *Total time to build this vignette (in s): * -->
<!-- ```{r} -->
<!-- print(sum(unlist(all_times))) -->
<!-- ``` -->
<!-- *details* -->
<!-- ```{r} -->
<!-- print(all_times) -->
<!-- ``` -->

---
title: "Exact distribution of excursions height"
author: "Sébastien Déjean - Charly Marti - Sabine Mercier - Sebastian Simon - David Robelin"
date: "`r Sys.Date()`"
output:
  pdf_document:
    toc: yes
  html_document:
    df_print: paged
    toc: yes
vignette: >
  %\VignetteIndexEntry{Exact distribution of excursions height} 
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown} 
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  warning = FALSE,
  collapse = TRUE,
  comment = "#>"
)

# Source : https://yihui.org/rmarkdown-cookbook/time-chunk
# all_times <- list()  # store the time for each chunk
# knitr::knit_hooks$set(time_it = local({
#   now <- NULL
#   function(before, options) {
#     if (before) {
#       # record the current time before each chunk
#       now <<- Sys.time()
#     } else {
#       # calculate the time difference after a chunk
#       res <- difftime(Sys.time(), now, units = "secs")
#       all_times[[options$label]] <<- res
#       # return a character string to show the time
#       paste("Time for the code chunk", options$label, " to run:", round(res,
#         2), "seconds")
#     }
#   }
# }))
# knitr::opts_chunk$set(time_it = FALSE)

```

```{r setup, echo=FALSE}
library(localScore)
```

# Introduction

The goal of this part of the package is to calculate the theoretical probability that the $i$-th excursion reach the threshold score $a$, for a Markov chain with a known transition matrix and a given score function. 
The main function for this objective is `proba_theoretical_ith_excursion_markov()`.

This section of the localScore package is the direct implementation of the pre publication: "Exact distribution of excursion heights of the Lindley
process in a Markovian model" written by Carlos Cortés Rojas, Simona Grusea and Sabine Mercier.

## Computation method 
The main goal of this function is to calculate the probability that the first excursion  of the Lindley sequence associated to the studied sequence is greater or equal to $a$, conditionally to $\alpha$ a potential beginning of this sequence. 
It also computes the transition probability matrix of the beginnings of excursions: `matrix_M`. 

The product of this matrix, elevated to the power $i-1$, with the first vector of the probabilities conditionally to $\alpha$, gives the probabilities conditionally to $\alpha$ for the $i$-th excursion.
To return the global probability, the function multiplies the conditional vector with the distribution of the first letter of the sequence.

## Definition of a mathematical excursion 

The number of an excursion is given by the mathematical definition of Karlin  and Altschul (1990). Its corresponds to the number of record times of the Lindley sequence associated to the score sequence $(X_k)_{1\leq k\leq n}$ and recursively defined as follows:
$$T_0:=0\qquad \mbox{and }\qquad T_{(k+1)}:=\inf\{i\geq T_k+1, \sum_{j=T_k+1}^i X_j<0\}\ .$$
Please  note that the sum $\sum_{j=T_k+1}^i X_j$ must be non positive. If it is equal to zero, it will mathematically be still the same Lindley excursion.
$$(A_k)_k\quad \mbox{with}\quad T_{(i-1)}+1\leq k <T_i$$ 
is called the $i$-th positive excursion
of the sequence in the term of Karlin and Altschul. This mathematical definition includes the index with a Linldey score equal to 0, whereas visually we "see" the excursion beginning at the following index.

The mathematical number of an excursion must be distinguished from the one of the corresponding visible excursion because of the possible appearance of flat excursions. See below for an example.

```{r}
seq <- c(-1,2,-3,-4,-1,3,-1,-2,3,-4,2)
lindley(seq)
plot(1:length(seq), lindley(seq), type = 'l')
localScoreC(seq)
```
We can "see" three positive excursions, with top at index $\{2; 6; 9\}$; plus one last not achieved at index $\{11\}$. Their exists also five flats excursions: at index $\{1 ; 3 ; 4 ; 5 ; 10\}$. Note also that the Lindley excursion 6-10 reach 0 at index 8 without going in non positive values, so it is still the same excursion. 

In summary, the excursions list is: $\{1 ; 2-3 ; 4 ; 5 ; 6-10 ; 11\}$.
The following function gives the record time of the mathematical excursion that must be used to compute the statistical significance. Note that we choose to omit the conventional first record time which is always 0.
```{r}
recordTimes(seq)
```

So the sub optimal excursion values given by `$suboptimalSegmentScores` 2 3 3 2 are achieved by the excursion number 2, 5 (twice) and 6. At this point, we wish to lighten a difference between the notion of "excursion" defined mathematically by Karlin and Dembo (1992), and the results given in `$suboptimalSegmentScores`. The segment [6-10] is an excursion in mathematical sense as the cumulative process starting at position 7 never hit a negative value before the 10th position. Nevetheless this process hit a null value indicating that this single excursion contain at least two positive score segments (here : at position 6 and 9). We chose to indicate in `$suboptimalSegmentScores` all positive consecutive sub-sequences, as it can be more meaningful when applied on real data.

To calculate the $p$-value of a given excursion, we have to know which number of excursion it is in sequential order.
Notice that it will not bring any problem of making a mistake in the number of the excursion for a number exceeding 10, and a small difference for a lower number, as the sequence composed by the begin-component of each excursion, which is also a Markov chain, can reach the stationary distribution. For the first excursions, it is far more easy to recover the record time of the corresponding mathematical excursion in the list.

# Toy examples
As the function calculates a theoretical probability, we don't need a sequence of scores but the transition matrix of the markov chain: $\Lambda$. It also requires a score function with: integer scores, a negative expectation and at least one possible positive score.
We also need $\theta$, an alphabet (can contains numbers) with unique values ; and $i$: the rank of the excursion on which we calculate the probabilities.
The optional parameters are `epsilon` and `prob0`. `epsilon` is a threshold for the computation of matrix $M$ and `prob0` is the probability distribution of the first letter of the sequence.
Obviously, `theta`, the score function and `prob0` should have the same length $p$, and the dimension of `lambda` should be $(p,p)$.

In this example, we want to know the probability that the second excursion reach a score greater or equal than 5 with the matrix $\Lambda$ and a score function of $(-1,1)$, given that the first score of the sequence have 50 percent chances to be -1.
```{r}
proba_theoretical_ith_excursion_markov(a = 5,
                                       theta = c("A", "B"),
                                       lambda = matrix(c(0.5, 0.5, 0.8, 0.2),
                                                       ncol = 2, byrow = TRUE),
                                       score_function = c(-1,1),
                                       i = 2,
                                       epsilon = 1e-16,
                                       prob0 = c(0.5, 0.5))
```
In this following example, we see that this is not a problem to have missing score values in the score function, and that theta can contain numbers.
Initial value of `epsilon` is 1e-16, and if we don't precise the value of `prob0`, the function compute the stationary distribution of $\Lambda$, and use it for the distribution of the first letter of the sequence.
```{r}
proba_theoretical_ith_excursion_markov(a = 5,
                                       theta = c("K", 12, 1.54),
                                       lambda = matrix(c(0.9, 0, 0.1, 0.9, 0.1, 0, 0, 0.8, 0.2),
                                                       ncol = 3, byrow = TRUE),
                                       score_function = c(-3,-1,2),
                                       i = 4)
```
In the following example, the function takes longer as we increased the number of scores (complexity of $O(length(\theta)^3))$, but here with 20 scores, the computational time is acceptable.
```{r}
transition_matrix <- matrix(runif(400, min = 0, max = 1), nrow = 20)
transition_matrix <- t(apply(transition_matrix, 1, function(x) x/sum(x)))
theta <- letters[1:20]
score_f <- c(-3,-1,2,-3,-1,2,-3,-1,2,-1,
              -3,-1,2,-3,-1,2,-3,-1,2,-1)
sum(stationary_distribution(transition_matrix)*score_f) #score expectation (stationary)
system.time(pv1 <- proba_theoretical_ith_excursion_markov(a=5, theta, transition_matrix, score_f, i = 4))
pv1$proba_q_i_geq_a
```

# A study case

Let us consider the protein `Seq1093` of 1093 amino acid proposed in the package `localScore` which corresponds to the Q60519.fasta in UniProt Data base. 

## Optimal excursion, the local score 

Using the hydrophobic score scale called `HydroScore`, we compute the local score, corresponding to the height of the highest excursion.

```{r}
library(localScore)
data("Seq1093")
Seq1093
nchar(Seq1093)
data(HydroScore)
LongSeqScore <- CharSequence2ScoreSequence(Seq1093, HydroScore)
table(LongSeqScore)
localScoreC(LongSeqScore)

head(sort(localScoreC(LongSeqScore)$suboptimalSegmentScores[,1], decreasing = TRUE))
plot(1:1093,lindley(LongSeqScore), type = 'l')
plot(1:400,lindley(LongSeqScore)[1:400], type = 'l')
```

We get: local score = 65.
Let us compute its $p$-value in a I.I.D. model.

```{r}
LS <- localScoreC(LongSeqScore)$localScore["value"]
prob1 <- scoreSequences2probabilityVector(list(LongSeqScore))
prob1
daudin(local_score = LS, sequence_length = length(LongSeqScore),
       score_probabilities = prob1,
       sequence_min = min(LongSeqScore),
       sequence_max = max(LongSeqScore))
```
In an independent model the $p$-value equals $7.2\%$, that is non significant using the nominal level $5\%$. 

Let us compute it in a markovian model. We need the transition matrix.

```{r}
tmpMarkovParameters <- sequences2transmatrix(LongSeqScore)
lambda <- tmpMarkovParameters$transition_matrix
apply(lambda, 1, sum) #to check stochasticity
prob0 <- stationary_distribution(lambda)
print(prob0)
score_values <- tmpMarkovParameters$score_value
print(score_values)
```
Notice that the score value 1 is not present in the sequence.


```{r}
exact_mc(LS, lambda, sequence_length = length(LongSeqScore), score_values = score_values)
``` 
The $p$-value in a Markovian model, equal to $7.7\%$ is very similar in this case with the one of the independent model.


# Sub optimal excursions

Let us consider a study on the first and the last sub optimal excursions.

```{r}
subOptSegment <- localScoreC(LongSeqScore)$suboptimalSegmentScores
o <- order(subOptSegment[,1], decreasing = TRUE)
subOptSegment <- subOptSegment[o,] # reordering segments by decreasing score values
print(subOptSegment)
```
The sub optimal scores are 40, 36 and 33 realized by the "visual" excursions number 1, 15 and 11.
Such number of excursion, 11 and 15 are large enough to avoid considering the mathematical number. The first excursion is mathematically the first one (see below).

```{r}
lindley(LongSeqScore)
recordTimes(LongSeqScore)
LongSeqScore[1] > 0
```

Let us consider a study on the first and the last sub optimal excursions in the sequential order of the sequence.

First excursion height equal to 40; last one, the 77th one, height equal to 6. 
```{r}
subOptSegment["1",] #First excursion
subOptSegment[as.character(dim(subOptSegment)[1]),] #Last excursion
```

Let us compute their $p$-values.


```{r}
theta <- letters[1:length(score_values)] # arbitrary
score_function <- score_values           # defined earlier 
a <- 40 
i <- 1
if (FALSE) { # To avoid long computation on CRAN servers... (Comment if you
                                                          # want to try)
  system.time(pv2 <- proba_theoretical_ith_excursion_markov(a, theta, lambda, 
                                          score_function,i)$proba_q_i_geq_a)
  } else {
  pv2 <- 0.1741784
  }
#> utilisateur système  écoulé
#> 38.061       0.033   38.137
pv2

a <- 6 
i <- 77
system.time(pv3<-proba_theoretical_ith_excursion_markov(a, theta, lambda, 
                                       score_function, i)$proba_q_i_geq_a)
pv3
```

First excursion : height equal to 40 with $p$-value=$0.45\%$;
Last excursion, the 77th one, height equal to 6 and $p$-value=$17\%$.

The time computation of the first $p$-value is larger because of the larger value of $a$.

We can consider that from the 20th mountain we reached the stationary distribution for the beginning of excursion. We shall therefore take a lower value for $i$ for the last excursion to evaluate the difference. Let us try $i=20$ instead of 77. 

```{r}
a <- 6 
i <- 20
system.time(pv4 <- proba_theoretical_ith_excursion_markov(a, theta, 
                                       lambda, 
                                       score_function,i)$proba_q_i_geq_a)
pv4

i <-  10
system.time(pv5 <- proba_theoretical_ith_excursion_markov(a, theta, 
                                       lambda, 
                                       score_function,i)$proba_q_i_geq_a)
pv5
```
We obtain the same value as expected even for $i=10$.

## With a reverse lecture of the protein

As the lecture of a protein could be done in both direction, we also consider the reverse sequence.

```{r}
LongSeqScore.inv <- rev(LongSeqScore)
localScoreC(LongSeqScore.inv)
head(sort(localScoreC(LongSeqScore.inv)$suboptimalSegmentScores[,1], decreasing = TRUE))
plot(1:1093,lindley(LongSeqScore.inv), type = 'l')
```

That leads to: first excursion equal to 6; last excursion, the 96th one, equal to 38. 

The corresponding $p$-values are:

```{r}
markov_parameters <- sequences2transmatrix(LongSeqScore.inv)
lambda.inv <- markov_parameters$transition_matrix
if (FALSE) {
  system.time(pv5 <- proba_theoretical_ith_excursion_markov(a = 38, theta, 
                                                            lambda.inv, 
                                                            score_function, i = 96
  )$proba_q_i_geq_a)
} else {
  pv5 <- 0.005460216
}
#> utilisateur système écoulé
#> 35.478      0.046   35.573
pv5
proba_theoretical_ith_excursion_markov(a = 6, theta, lambda.inv,
                                       score_function,i = 1
                                       )$proba_q_i_geq_a
```
Last excursion, the 96th one, equal to 38, with $p$-value equal to $0.55\%$.
First excursion equal to 6, with $p$-value equal to $17\%$.

The last excursion is still significant. The first one is still non significant.

Remark : Even with a Bonferroni correction to take into account the multiple test (here two studies excursions), the sequence possesses a significant segment. Whereas considering the highest value over all the excursions of the whole sequence, the local score value, the sequence is not significant.

## What about the excursion realising the local score

Let us consider the excursion 30 as an excursion among the others. For the first way to read the protein:
```{r}
a <- 65 
i <- 30
if (FALSE) { # To avoid long computation on CRAN servers... (Comment if you
                                                          # want to try)
  system.time(pv6 <- proba_theoretical_ith_excursion_markov(a, theta, 
                                                            lambda, score_function, 
                                                            i)$proba_q_i_geq_a)
} else {
  pv6 <- 0.0004221151
}
#> utilisateur système écoulé
#> 104.861     0.101   105.062
pv6 
```

There is a less than 4 in 10,000 chance of having a first mountain over 65.

Or with the second way to read the protein:
```{r}
subOptSegment.inv <- localScoreC(LongSeqScore.inv)$suboptimalSegmentScores
which.max(subOptSegment.inv[,1])
print(subOptSegment.inv[16,])

a <- 65 
i <- 16
if (FALSE) {# To avoid long computation on CRAN servers... (Comment if you
                                                          # want to try)
  system.time(pv7 <- proba_theoretical_ith_excursion_markov(a, theta, lambda.inv, 
                                                            score_function,i)$proba_q_i_geq_a)
} else {
  pv7 <- 0.0004379929
}
#> utilisateur système écoulé
#> 103.018     0.088   103.229
pv7 
```
As expected we obtain a similar $p$-value.
 
Although the highest mountain of height 65 in this sequence of length 1093 does not have a significant value at the $5\%$ threshold, it does not mean that the sequence has not interesting segments. Observing a mountain exceeding 40 is significant for example, and it is the same for values 36 and 33. All these tests remain significant even if a correction of type Bonferroni is taken.


---

<!-- *Total time to build this vignette (in s): * -->
<!-- ```{r} -->
<!-- print(sum(unlist(all_times))) -->
<!-- ``` -->
<!-- *details* -->
<!-- ```{r} -->
<!-- print(all_times) -->
<!-- ``` -->

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{loadMatrixFromFile}
\alias{loadMatrixFromFile}
\title{Loads matrix from csv-File}
\usage{
loadMatrixFromFile(filepath)
}
\arguments{
\item{filepath}{optional: Location of file on disk. If not provided, a file picker dialog will be opened.}
}
\value{
A Matrix Object
}
\description{
Reads a csv file without header and returns the matrix. For file formats please see section "File Formats" in vignette.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{transmatrix2sequence}
\alias{transmatrix2sequence}
\title{Sampling function for Markov chains}
\usage{
transmatrix2sequence(matrix, length, initialIndex, score)
}
\arguments{
\item{matrix}{transition matrix of Markov process}

\item{length}{length of sequence to be sampled}

\item{initialIndex}{(optional) index of matrix which should be initial value of sequence. If none supplied, a value from the stationary distribution is sampled as initial value.}

\item{score}{(optional) a vector representing the scores (in ascending order) of the matrix index. If supplied, the result will be a vector of these values.}
}
\value{
a Markov chain sampled from the transition matrix
}
\description{
Creates Markov chains based on a transition matrix. Can be used as parameter for the Monte Carlo function.
}
\details{
The transition matrix is considered representing the transition from one score to another such that the score in the first row is the lowest 
and the last row are the transitions from the highest score to another. The matrix must be stochastic (no rows filled up with only '0' values).

It is possible to have the same score for different states of the markov chain. 
If no score supplied, the function returns a markov chain with state in 1:ncol(matrix)
}
\examples{
B <-  matrix (c(0.2, 0.8, 0.4, 0.6), nrow = 2, byrow = TRUE)
transmatrix2sequence(B, length = 10)
transmatrix2sequence(B, length = 10, score = c(-2,1))
transmatrix2sequence(B, length = 10, score = c("A","B"))
MyTransMat <-
   matrix(c(0.3,0.1,0.1,0.1,0.4,
            0.2,0.2,0.1,0.2,0.3,
            0.3,0.4,0.1,0.1,0.1,
            0.3,0.3,0.1,0.0,0.3,
            0.1,0.1,0.2,0.3,0.3),
            ncol = 5, byrow=TRUE)
MySeq.CM <- transmatrix2sequence(matrix = MyTransMat,length=90, score =c(-2,-1,0,2,3))
MySeq.CM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{karlin}
\alias{karlin}
\alias{karlin_parameters}
\title{Karlin [p-value] [iid]}
\usage{
karlin(
  local_score,
  sequence_length,
  score_probabilities,
  sequence_min = NULL,
  sequence_max = NULL,
  score_values = NULL
)

karlin_parameters(
  score_probabilities,
  sequence_min = NULL,
  sequence_max = NULL,
  score_values = NULL
)
}
\arguments{
\item{local_score}{the observed local score}

\item{sequence_length}{length of the sequence}

\item{score_probabilities}{the probabilities for each score from lowest to greatest (Optionnaly with scores as names)}

\item{sequence_min}{minimum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{sequence_max}{maximum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{score_values}{vector of integer score values, associated to score_probabilities  (optional if
\code{sequence_min} and \code{sequence_max} OR names(score_probabilities) are defined)}
}
\value{
A double representing the probability of a local score as high as the one given as argument
}
\description{
\code{karlin} Calculates an approximated p-value of a given local score value and a long sequence length in the identically and independently distributed model for the sequence. See also \code{\link{mcc}} function for another approximated method in the i.i.d. model that improved the one given by \code{\link{karlin}} or \code{\link{daudin}} for exact calculation. \cr
\code{karlin_parameters} is a annex function returning the parameters \eqn{\lambda}, \eqn{K^+} and \eqn{K^*} defined in Karlin and Dembo (1992).
}
\details{
This method works the better the longer the sequence is. Important note : the calculus of the parameter of the distribution uses
the resolution of a polynome which is a function of the score distribution, of order max(score)-min(score). There exists only empirical methods to solve a polynome of order greater that 5
with no warranty of reliable solution.
The found roots are checked internally to the function and an error message is throw in case of inconsistent. In such case, you could try to change your score scheme (in case of discretization)
or use the function \code{\link{karlinMonteCarlo}} .
This function implements the formulae given in Karlin and Dembo (1992), page 115-6. As the score is discrete here (lattice score function),
there is no limit distribution of the local score with the size of the sequence, but an inferior and a superior bound
are given. The output of this function is conservative as it gives the upper bound for the p-value.
Notice the lower bound can easily be found as it is the same call of function with parameter value local_score+1.
}
\examples{
karlin(150, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -5, 5)
p1 <- karlin(local_score = 15, sequence_length = 5000, 
       score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
       sequence_min = -3, sequence_max = 2)
p2 <- karlin(local_score = 15, sequence_length = 5000, 
       score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
       score_values = -3:2)
p1 == p2 # TRUE

prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
score_values <- which(prob != 0) - 6 # keep only non null probability scores
prob0 <- prob[prob != 0]             # and associated probability
p <- karlin(150, 10000, prob, sequence_min = -5, sequence_max =  5)
p0 <- karlin(150, 10000, prob0, score_values = score_values) 
names(prob0) <- score_values
p1 <- karlin(150, 10000, prob0)
p == p0 # TRUE
p == p1 # TRUE
}
\seealso{
\code{\link{mcc}}, \code{\link{daudin}}, \code{\link{karlinMonteCarlo}}, \code{\link{monteCarlo}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{mcc}
\alias{mcc}
\title{MCC [p-value] [iid]}
\usage{
mcc(
  local_score,
  sequence_length,
  score_probabilities,
  sequence_min = NULL,
  sequence_max = NULL,
  score_values = NULL
)
}
\arguments{
\item{local_score}{the observed local score}

\item{sequence_length}{length of the sequence}

\item{score_probabilities}{the probabilities for each score from lowest to greatest (Optionnaly with scores as names)}

\item{sequence_min}{minimum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{sequence_max}{maximum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{score_values}{vector of integer score values, associated to score_probabilities  (optional if
\code{sequence_min} and \code{sequence_max} OR names(score_probabilities) are defined)}
}
\value{
A double representing the probability of a local score as high as the one given as argument
}
\description{
Calculates an approximated p-value for a given local score value and a medium to long sequence length in the identically and independently distributed model
}
\details{
This methods is actually an improved method of Karlin and produces more precise results. It should be privileged whenever possible. \cr
As with karlin, the method works the better the longer the sequence. Important note : the calculus of the parameter of the distribution uses
the resolution of a polynome which is a function of the score distribution, of order max(score)-min(score). There exists only empirical methods to solve a polynome of order greater that 5
with no warranty of reliable solution.
The found roots are checked internally to the function and an error message is throw in case of inconsistency. In such case, you could try to change your score scheme (in case of discretization)
or use the function \code{\link{karlinMonteCarlo}} .
}
\examples{
mcc(40, 100, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -6, 4)
mcc(40, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -6, 4)
mcc(150, 10000, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -5, 5)
p1 <- mcc(local_score = 15, sequence_length = 5000, 
       score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
       sequence_min = -3, sequence_max = 2)
p2 <- mcc(local_score = 15, sequence_length = 5000, 
       score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
       score_values = -3:2)
p1 == p2 # TRUE

prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
score_values <- which(prob != 0) - 6 # keep only non null probability scores
prob0 <- prob[prob != 0]             # and associated probability
p <- mcc(150, 10000, prob, sequence_min = -5, sequence_max =  5)
p0 <- mcc(150, 10000, prob0, score_values = score_values)
names(prob0) <- score_values
p1 <- mcc(150, 10000, prob0)
p == p0 # TRUE
p == p1 # TRUE
}
\seealso{
\code{\link{karlin}}, \code{\link{daudin}}, \code{\link{karlinMonteCarlo}}, \code{\link{monteCarlo}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{CharSequences2ScoreSequences}
\alias{CharSequences2ScoreSequences}
\title{Convert several character sequences into score sequences}
\usage{
CharSequences2ScoreSequences(sequences, dictionary)
}
\arguments{
\item{sequences}{a list of character sequences given as string}

\item{dictionary}{a data.frame with rownames containing letters, first column containing associated scores, optional second column containing associated probabilities}
}
\value{
a list of score sequences
}
\description{
Convert several character sequence into score sequences. For only one sequence see CharSequence2ScoreSequence() function.
}
\examples{
data(Seq31)
Seq31
data(Seq219)
Seq219
data(HydroScore)
MySequences=list("A1"=Seq31,"A2"=Seq219)
CharSequences2ScoreSequences(MySequences,HydroScore)
}
\seealso{
\code{\link{CharSequence2ScoreSequence}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{CharSequence2ScoreSequence}
\alias{CharSequence2ScoreSequence}
\title{Convert a character sequence into a score sequence}
\usage{
CharSequence2ScoreSequence(charseq, dictionary)
}
\arguments{
\item{charseq}{a character sequence, given as a string}

\item{dictionary}{a data.frame with rownames containing letters, first column containing associated scores, optional second column containing associated probabilities}
}
\value{
a vector of a score sequence
}
\description{
Convert a character sequence into a score sequence. See CharSequences2ScoreSequences() function for several sequences
}
\examples{
data(Seq31)
Seq31
data(HydroScore)
CharSequence2ScoreSequence(Seq31,HydroScore)
}
\seealso{
\code{\link{CharSequences2ScoreSequences}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{recordTimes}
\alias{recordTimes}
\title{Calculate the record times of a sequence}
\usage{
recordTimes(sequence)
}
\arguments{
\item{sequence}{numeric sequence of a Lindley process, eg service time per
customer}
}
\value{
a vector with the record times. If no record time are found, return
  empty vector \code{integer(0)}
}
\description{
For a given sequence of real numbers, this function returns the
  index of the values where Lindley/CUSUM process are <0
}
\details{
Note that the first record times which is always 0 is not included
in the returned vector
}
\examples{
####This example should return this vector: c(1,3,4,5,10)
seq1 <- c(-1,2,-2.00001,-4,-1,3,-1,-2,3,-4,2)
recordTimes(seq1)
####This example should return integer(0) because there is no record times in this sequence
seq2 <- c(4,1,0,-1,-2,5,-5,0,4)
recordTimes(seq2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{sequences2transmatrix}
\alias{sequences2transmatrix}
\title{Transition matrix from sequence(s)}
\usage{
sequences2transmatrix(sequences)
}
\arguments{
\item{sequences}{Sequences to be analyzed, can be a list of vectors, or a vector}
}
\value{
A list object containing
\item{transition_matrix}{Transition Matrix with row names and column names are the associated score/state}
\item{score_value}{a vector containing the score/state, ordered in the same way as the matrix columns and rows}
}
\description{
Calculates the transition matrix by counting occurrences of tuples in given vector list
}
\details{
In output, score_value is coerced as integer if possible. Else, it is a character vector containing the states
of the Markov chain.
}
\examples{
myseq <- sample(LETTERS[1:2], size=20,replace=TRUE)
sequences2transmatrix(myseq)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localScore-package.R
\docType{package}
\name{localScore-package}
\alias{localScore}
\alias{localScore-package}
\title{localScore: Package for Sequence Analysis by Local Score}
\description{
Provides functionalities for:
\itemize{
\item calculating the local score
\item calculating statistical relevance (p-value) to find a local Score in a sequence of given distribution
}

Given a sequence of numerical score \eqn{X_1,\dots,X_n}, the local score is defined : \eqn{H_n = \max_{1 \leq i \leq j \leq n} \sum_{l = i}^j X_l}
This package find the value local score and the associated sub-sequence, and also sub-optimal local scores and segments. The complexity is linear with \eqn{n}. It can be viewed as a generalization of a sliding window method, considering all windows size. In order to be pertinent, the expectation of the scores \eqn{X_i} should be negative. Most of the methods concerning statistical relevance implemented in this package only applied on integer scores.
}
\details{
Please refer to the vignette of this package or the manual for details on how to use this package.
}
\references{
\itemize{
\item An Improved Approximation For Assessing The Statistical Significance of molecular Sequence Features, Mercier and al 2003
\item Exact distribution for the local score of one i.i.d. random sequence, Sabine Mercier and JJ Daudin, 2001
\item Limit Distributions of Maximal Segmental Score among Markov-Dependent Partial Sums, Karlin and Dembo 1992
\item Methods for assessing the statistical significance of molecular sequence features by using general scoring schemes, Karlin and al 1990
\item Detection de courts segments inverses dans les genomes: methodes et applications, David Robelin 2005
\item A Linear Time Algorithm for Finding All Maximal Scoring Subsequences, Constable and Bates 1985
}
}
\author{
\strong{Maintainer}: David Robelin \email{david.robelin@inrae.fr}

Authors:
\itemize{
  \item Sebastian Simon
  \item Chris Verschelden
  \item Charly Marty
  \item Sabine Mercier \email{sabine.mercier@univ-tlse2.fr}
  \item Sebastien Dejean \email{sebastien.dejean@math.univ-toulouse.fr}
}

Other contributors:
\itemize{
  \item The authors of Eigen the library for the included version of Eigen [copyright holder]
}

}
\keyword{Package}
\keyword{analysis}
\keyword{by}
\keyword{for}
\keyword{internal}
\keyword{local}
\keyword{score}
\keyword{sequence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{stationary_distribution}
\alias{stationary_distribution}
\title{Stationary distribution [Markov chains]}
\usage{
stationary_distribution(m)
}
\arguments{
\item{m}{Transition Matrix [matrix object]}
}
\value{
A vector with the probabilities
}
\description{
Calculates stationary distribution of markov transition matrix by use of eigenvectors of length 1
}
\examples{
B = t(matrix (c(0.2, 0.8, 0.4, 0.6), nrow = 2))
stationary_distribution(B)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/excursion.R
\name{proba_theoretical_ith_excursion_markov}
\alias{proba_theoretical_ith_excursion_markov}
\title{theoretical probability of reaching the threshold score a on the i-th excursion (sequential order) of a markov's score sequence}
\usage{
proba_theoretical_ith_excursion_markov(
  a,
  theta,
  lambda,
  score_function,
  i,
  prob0 = NULL,
  epsilon = 1e-16
)
}
\arguments{
\item{a}{threshold score}

\item{theta}{vector containing the alphabet used}

\item{lambda}{transition probability matrix of the markov chain, square, same size and order as theta}

\item{score_function}{vector containing the scores of each letters of the alphabet (must be in the same order as theta)}

\item{i}{number of wanted excursion (sequential order)}

\item{prob0}{probability distribution of the first letter of the sequence (stationary distribution of lambda if the parameter is NULL),
used only for computation of proba_q_i_geq_a}

\item{epsilon}{threshold for the computation of matrix M: to calculate the probabilities, this function need to compute the matrix M (transition probability matrix of the beginning of excursions, as in the paper)
with a recurrence, this recurrence stop at a certain number of iterations or when the maximal absolute value of the difference between an iteration and the next one is < epsilon.}
}
\value{
a list containing:\tabular{ll}{
    \code{proba_q_i_geq_a} \tab reel number between 0 and 1 representing the theoretical probability that the i-th excursion is greater or equal a \cr
    \tab \cr
    \code{P_alpha} \tab vector containing the probabilities (of reaching the threshold score on the i-th excursion) depending on the first letter of the sequence (alpha) \cr
}
}
\description{
This function implements the results of the paper: "Exact distribution of excursion heights of the Lindley process in a Markovian model", this is an
exact method. 
Scores of score function have to be integers, the expectancy of the score have to be negative, the score function have to contains at least
one positive integer, i have to be >0
}
\details{
Be careful: the computational time is exponential in function of the maximum score of the score function.
The computational time also increase with the rise of the threshold score a and with the lowering of the minimum of the score function and have a cubic complexity in function of the size of theta.
Lowering epsilon can also decrease the execution time of the function but it can have an impact on the accuracy of the probabilities.
}
\examples{
## In this example: the probability to reach a score of a in the third excursion is: 0.04237269
## The conditional probabilities to reach a score of a in the third excursion if the first
## letter of the sequence is K, L or M are respectively: 0.04239004, 0.04247805 and 0.04222251
proba_theoretical_ith_excursion_markov(a = 5,
               theta = c("K","L","M"),
               lambda = matrix(c(0.5, 0.3, 0.2,
                                 0.4, 0.2, 0.4,
                                 0.4, 0.4, 0.2), 
                               ncol = 3, byrow = TRUE),
               score_function = c(-2,-1,2),
               i = 3,
               prob0 = c(0.4444444, 0.2962963, 0.2592593))
### This example implements the numerical application of the paper,
###    the global probability is 0.2095639
proba_theoretical_ith_excursion_markov(a = 6,
            theta = c("a","b","c","d","e"),
            lambda = matrix(c(0.1, 0.7, 0.05, 0.05, 0.1,
                              0.3, 0.3, 0.2, 0.15, 0.05,
                              0.1, 0.4, 0.15, 0.2, 0.15,
                              0.5, 0.05, 0.25, 0.1, 0.1,
                              0.25, 0.05, 0.5, 0.15, 0.05),
                              ncol = 5,nrow=5, byrow = TRUE),
            score_function = c(-3,-2,-1,6,7),
            i = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Seq1093}
\alias{Seq1093}
\title{Long protein sequence}
\format{
A character string with 1093 characters corresponding to Q60519.fasta in UniProt Data base.
}
\source{
\url{https://www.uniprot.org/}
}
\usage{
data(Seq1093)
}
\description{
A long protein sequence.
}
\examples{
data(Seq1093)
Seq1093
nchar(Seq1093)
data(HydroScore)
seqScore <- CharSequence2ScoreSequence(Seq1093,HydroScore)
seqScore[1:50]
localScoreC(seqScore)$localScore
LS <- localScoreC(seqScore)$localScore[1]
prob1 <- scoreSequences2probabilityVector(list(seqScore))
daudin(local_score = LS, sequence_length = nchar(Seq1093),
       score_probabilities = prob1,
       sequence_min = min(seqScore),
       sequence_max = max(seqScore))
karlin(local_score = LS, sequence_length = nchar(Seq1093),
       score_probabilities = prob1,
       sequence_min = min(seqScore),
       sequence_max = max(seqScore))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{aeso.data}
\alias{aeso.data}
\alias{SJSyndrome.data}
\alias{MySeqList}
\alias{dico}
\alias{LongSeq}
\alias{MidSeq}
\alias{ShortSeq}
\title{Deprecated}
\format{
Refers to the "See also" links below to obtain a description of each dataset.
}
\usage{
data(Aeso) ; data(SJSyndrome) ;
       data(SeqListSCOPe) ; data(HydroScore) ;
       data(Seq1093) ; data(Seq219) ; data(Seq31)
}
\description{
Some datasets are deprecated and will be removed in next version of the package.
Please use instead:
\itemize{
  \item data(aeso.data) -> data(Aeso)
  \item data(SJSyndrome.data) -> data(SJSyndrome)
  \item data(MySeqList) -> data(SeqListSCOPe)
  \item data(dico) -> data(HydroScore)
  \item data(LongSeq) -> data(Seq1093)
  \item data(MidSeq) -> data(Seq219)
  \item data(ShortSeq) -> data(Seq31)
}
}
\seealso{
\code{\link{Aeso}} \code{\link{SJSyndrome}} \code{\link{SeqListSCOPe}} 
         \code{\link{HydroScore}} \code{\link{Seq1093}} \code{\link{Seq219}} \code{\link{Seq31}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{RealScores2IntegerScores}
\alias{RealScores2IntegerScores}
\title{Convert a real scores vector into an integer scores vector}
\usage{
RealScores2IntegerScores(RealScore, ProbRealScore, coef = 10)
}
\arguments{
\item{RealScore}{vector of real scores}

\item{ProbRealScore}{vector of probability}

\item{coef}{coefficient}
}
\value{
list containing ExtendedIntegerScore and ProbExtendedIntegerScore
}
\description{
Convert real scores into integer scores
}
\details{
Convert real scores into integer scores by multiplying real
scores by a coefficient (default 10) and then assigning probability to corresponding
extended (from the minimum to the maximum) integer scores
}
\examples{
score <- c(-1,-0.5,0,0.5,1)
prob.score <- c(0.2,0,0.4,0.1,0.3)
(res1 <- RealScores2IntegerScores(score, prob.score, coef=10))
prob.score.err <- c(0.1,0,0.4,0.1,0.3)
(res2 <- RealScores2IntegerScores(score, prob.score.err, coef=10))
# When coef=1, the function can handle integer scores
ex.integer.score <- c(-3,-1,0,1, 5)
(res3 <- RealScores2IntegerScores(ex.integer.score, prob.score, coef=1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{loadScoreFromFile}
\alias{loadScoreFromFile}
\title{Load score from file}
\usage{
loadScoreFromFile(filepath, header = TRUE, ...)
}
\arguments{
\item{filepath}{optional: location of file on disk. If not provided, a file picker dialogue box will be opened.}

\item{header}{does the file contain a header line (defaut to \code{TRUE})}

\item{...}{optional: use arguments from read.csv}
}
\value{
A data.frame. Rownames correspond to the first column, usually Letters. Associated numerical scores are in the second column. If probabilities are provided, 
they will be loaded too and presumed to be in the third column
}
\description{
Reads a csv file containing an alphabet, the associated scores, and eventually the associated probabilities.
}
\details{
The file should contains a header and 2 or 3 columns : first column the letters, second column the associated scores, optional third column associated probabilities. Letters should be unique and probabilities should sum to 1.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Aeso}
\alias{Aeso}
\title{Congenital oesophageal atresia data}
\format{
A matrix of 2191 lines and 2 columns. Each line is a day on the first column, and associated to a case (0/1) on the second column.
}
\source{
Dolk H. Secular pattern of congenital oesophageal atresia--George Knox, 1959. J Epidemiol Community Health. 1997;51(2):114-115. <doi:10.1136/jech.51.2.114>
}
\usage{
data(Aeso)
}
\description{
The data consists of individual dates of birth over n=35 cases of the birth defects oesophageal and tracheo-oesophagean fistula
observed in a hospital in Birmingham, U.K., over 2191 days from 1950 through 1955, with Day one set as 1 January 1950
}
\examples{
data(Aeso)
head(Aeso)
p <- sum(Aeso[,2]) / dim(Aeso)[1]
print(p)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/excursion.R
\name{proba_theoretical_first_excursion_iid}
\alias{proba_theoretical_first_excursion_iid}
\title{Probability \eqn{P(Q(1)\geq a)} that the height of the first excursion is greater or equal to \code{a} given a i.i.d. model on the letters sequence}
\usage{
proba_theoretical_first_excursion_iid(
  a,
  theta,
  theta_distribution,
  score_function
)
}
\arguments{
\item{a}{score strictly positive}

\item{theta}{vector containing the alphabet used}

\item{theta_distribution}{distribution vector of theta}

\item{score_function}{vector containing the scores of each letters of the alphabet (must be in the same order as theta)}
}
\value{
theoretical probability of reaching a score of a on the first excursion supposing an  i.i.d model on the letters sequence
}
\description{
Mathematical definition of an excursion of the Lindley process is based on the record times of the partial
sum sequence associated to the score sequence (see Karlin and Altschul 1990, Karlin and Dembo 1992) and
define the successive times where the partial sums are strictly decreasing. There must be distinguished
from the visual excursions of the Lindley sequence. The number \code{i} is the number of excursion in sequential order. Detailed definitions are given in the vignette.
}
\details{
Beware that a sequence beginning with a negative score gives a "flat" excursion, with score 0 are considered.
}
\examples{
proba_theoretical_first_excursion_iid(3, c("a","b","c","d"), 
                                      c(a=0.1,b=0.2,c=0.4,d=0.3), c(a=-3,b=-1,c=1,d=2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/excursion.R
\name{MarkovParameters}
\alias{MarkovParameters}
\title{Functions parameters global description}
\arguments{
\item{theta}{alphabet used (vector of character)}

\item{lambda}{transition probability matrix of size, theta x theta}

\item{score_function}{vector containing the scores of each letters of the alphabet (must be in the same order as theta)}

\item{a}{score strictly positive}

\item{prob0}{Distribution of the first element of the Markov chain. Default to stationary distribution of lambda}
}
\description{
Functions parameters global description
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R, R/rMethods.R
\name{localScoreC}
\alias{localScoreC}
\alias{localScoreC_double}
\alias{localScoreC_int}
\title{Local score}
\usage{
localScoreC(v, suppressWarnings = FALSE)

localScoreC_double(v, suppressWarnings = FALSE)

localScoreC_int(v, suppressWarnings = FALSE)
}
\arguments{
\item{v}{a sequence of numerical values as vector (integer or double).}

\item{suppressWarnings}{(optional) if warnings should not be displayed}
}
\value{
A list containing:
\item{localScore}{the local score value and the begin and end index of the segment realizing this optimal score;}
\item{suboptimalSegmentScores}{An array containing sub-optimal local scores, that is all the local maxima of the Lindley rocess (non negative excursion) and their begin and end index;}
\item{RecordTime}{The record times of the Lindley process as defined in Karlin and Dembo (1990).}
}
\description{
Calculates the local score for a sequence of scores, the sub-optimal segments, and the associated record times. The local score is the maximal sum of values contained in a segment among all possible segments of the sequence. In other word, it generalizes a sliding window approach, considering of all possible windows size.
}
\details{
The \code{localScoreC} function is implemented in a templated C function. Be aware that the type of the output (\code{integer} or \code{double}) depends on the type of the input. The function \code{localScoreC_double} \code{localScoreC_int} explicitly use the corresponding type (with an eventual conversion in case of integer). Warning: in R, \code{typeof(c(1,3,4,10)) == "double"}. You can set a type of a vector with \code{mode()} or \code{as.integer()} functions for example. \cr
\code{localScoreC_int} is just a call to \code{as.integer()} before calling \code{localScoreC}. \code{localScore_double} is just a call to \code{localScoreC}, and as such is deprecated.
}
\examples{
localScoreC(c(1.2,-2.1,3.5,1.7,-1.1,2.3))
# one segment realizing the local score value
seq.OneSegment <- c(1,-2,3,1,-1,2)
localScoreC(seq.OneSegment) 
seq.TwoSegments <- c(1,-2,3,1,2,-2,-2,-1,1,-2,3,1,2,-1,-2,-2,-1,1)
# two segments realizing the local score value
localScoreC(seq.TwoSegments) 
# only the first realization
localScoreC(seq.TwoSegments)$localScore 
# all the realization of the local together with the suboptimal ones
localScoreC(seq.TwoSegments)$suboptimalSegmentScores 
# for small sequences, you can also use lindley() function to check if 
# several segments achieve the local Score
lindley(seq.TwoSegments) 
plot(1:length(seq.TwoSegments),lindley(seq.TwoSegments),type='b')
seq.TwoSegments.InSameExcursion <- c(1,-2,3,2,-1,0,1,-2,-2,-4,1)
localScoreC(seq.TwoSegments.InSameExcursion)
# lindley() shows two realizations in the same excursion (no 0 value between the two LS values)
lindley(seq.TwoSegments.InSameExcursion) 
# same beginning index but two possible ending indexes
# only one excursion realizes the local score even in there is two possible lengths of segment
localScoreC(seq.TwoSegments.InSameExcursion)$suboptimalSegmentScores 
plot(1:length(seq.TwoSegments.InSameExcursion),lindley(seq.TwoSegments.InSameExcursion),type='b')
# Technical note about type correspondance
seq.OneSegment <- c(1,-2,3,1,-1,2)
seq.OneSegmentI <- as.integer(seq.OneSegment)
typeof(seq.OneSegment)  # "double" (beware)
typeof(seq.OneSegmentI) # "integer"
LS1 <- localScoreC(seq.OneSegment, suppressWarnings = TRUE)
LS1I <- localScoreC(seq.OneSegmentI, suppressWarnings = TRUE)
typeof(LS1$localScore)  # "double"
typeof(LS1I$localScore) # "integer"
typeof(LS1$suboptimalSegmentScores$value)  # "double"
typeof(LS1I$suboptimalSegmentScores$value) # "integer"
# Force to use integer values (trunk values if necessary)
seq2 <- seq.OneSegment + 0.5
localScoreC(seq2, suppressWarnings = TRUE)
localScoreC_int(seq.OneSegment, suppressWarnings = TRUE)
}
\seealso{
\code{\link{lindley}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{maxPartialSumd}
\alias{maxPartialSumd}
\title{Maximum of the partial sum [probability] [iid]}
\usage{
maxPartialSumd(
  k,
  score_probabilities,
  sequence_min = NULL,
  sequence_max = NULL,
  score_values = NULL
)
}
\arguments{
\item{k}{value at which calculates the probability}

\item{score_probabilities}{the probabilities for each score from lowest to greatest (Optionnaly with scores as names)}

\item{sequence_min}{minimum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{sequence_max}{maximum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{score_values}{vector of integer score values, associated to score_probabilities  (optional if
\code{sequence_min} and \code{sequence_max} OR names(score_probabilities) are defined)}
}
\value{
A double representing the probability of the maximum of the partial sum process equal to k
}
\description{
Calculates the distribution of the maximum of the partial sum process for a given value in the identically and independently distributed model
}
\details{
Implement the formula (4) of the article Mercier, S., Cellier, D., & Charlot, D. (2003). An improved approximation for assessing the statistical significance of molecular sequence features. Journal of Applied Probability, 40(2), 427-441. doi:10.1239/jap/1053003554 \cr
Important note : the calculus of the parameter of the distribution uses
the resolution of a polynome which is a function of the score distribution, of order max(score)-min(score). There exists only empirical methods to solve a polynome of order greater that 5
with no warranty of reliable solution.
The found roots are checked internally to the function and an error message is throw in case of inconsistency.
}
\examples{
maxPartialSumd(10, c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02), -6, 4)
prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
score_values <- which(prob != 0) - 7 # keep only non null probability scores
prob0 <- prob[prob != 0]             # and associated probability
p <- maxPartialSumd(10, prob, sequence_min = -6, sequence_max =  4)
p0 <- maxPartialSumd(10,prob0, score_values = score_values)
p == p0 # TRUE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{lindley}
\alias{lindley}
\title{Lindley process}
\usage{
lindley(sequence)
}
\arguments{
\item{sequence}{numeric sequence of a Lindley process, eg service time per customer}
}
\value{
a vector with the Lindley process steps
}
\description{
Creates a sequence of a Lindley process, also called CUSUM process, on a given sequence.  For a sequence (X_k)k, the Lindley process is defined as follows: W_0:=0 and W_(k+1)=max(0,W_k+X_(k+1)). It defines positive excursions above 0.
}
\examples{
MySeq <- c(1,2,3,-4,1,-3,-1,2,3,-4,1)
lindley(MySeq)
plot(1:length(MySeq),lindley(MySeq),type='b')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{scoreSequences2probabilityVector}
\alias{scoreSequences2probabilityVector}
\title{Empirical distribution from sequences}
\usage{
scoreSequences2probabilityVector(sequences)
}
\arguments{
\item{sequences}{list of numerical sequences}
}
\value{
empirical distribution from minimum score to maximum score as a vector of floating numbers.
}
\description{
Builds empirical distribution from a list of numerical sequences
}
\details{
By determining the extreme scores in the sequences, this function creates a vector of probabilities
including values that do not occur at all. In this it differs from table(). For example, two sequences containing
values from 1:2 and 5:6 will produce a vector of size 6.
}
\examples{
seq1 = sample(7:8, size = 10, replace = TRUE)
seq2 = sample(2:3, size = 15, replace = TRUE)
l = list(seq1, seq2)
scoreSequences2probabilityVector(l)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{HydroScore}
\alias{HydroScore}
\title{Dictionary}
\format{
A score function for the 20 amino acid
}
\source{
Kyte & Doolittle (1982) J. Mol. Biol. 157, 105-132
}
\usage{
data(HydroScore)
}
\description{
Provides integer scores related to an hydrophobicity level of each amino acid. This score function is inspired by the Kyte and Doolittle (1982) scale.
}
\examples{
data(HydroScore)
HydroScore
data(Seq219)
Seq219
seqScore <- CharSequence2ScoreSequence(Seq219,HydroScore)
seqScore[1:30]
localScoreC(seqScore)$localScore
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SeqListSCOPe}
\alias{SeqListSCOPe}
\title{Several sequences}
\format{
A list of 285 character strings with their entry codes as names
}
\source{
Structural Classification Of Proteins database (SCOP). More precisely this data contain the 285 protein sequences of the data called "CF_scop2dom_20140205aa" with length from 31 to 404.
}
\usage{
data(SeqListSCOPe)
}
\description{
A vector of character strings
}
\examples{
data(SeqListSCOPe)
head(SeqListSCOPe)
SeqListSCOPe[1]
nchar(SeqListSCOPe[1])
summary(sapply(SeqListSCOPe, nchar))
data(HydroScore)
MySeqScoreList <- lapply(SeqListSCOPe, FUN=CharSequence2ScoreSequence, HydroScore)
head(MySeqScoreList)
AA <- automatic_analysis(sequences=MySeqScoreList, model='iid')
AA[[1]]
# the p-value of the first 10 sequences 
sapply(AA, function(x){x$`p-value`})[1:10]
# the 20th smallest p-values
sort(sapply(AA, function(x){x$`p-value`}))[1:20]
which(sapply(AA, function(x){x$`p-value`})<0.05)
table(sapply(AA, function(x){x$`method`}))
# The maximum sequence length equals 404 so it here normal that the exact method is used for
# all the 606 sequences of the data base 
# Score distribution learned on the data set
scoreSequences2probabilityVector(MySeqScoreList)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{automatic_analysis}
\alias{automatic_analysis}
\title{Automatic analysis}
\usage{
automatic_analysis(
  sequences,
  model,
  scores,
  transition_matrix,
  distribution,
  method_limit = 2000,
  score_extremes,
  modelFunc,
  simulated_sequence_length = 1000,
  ...
)
}
\arguments{
\item{sequences}{sequences to be analyzed (named list)}

\item{model}{the underlying model of the sequence (either "iid" for identically independently distributed variable or "markov" for Markov chains)}

\item{scores}{vector of minimum and maximum score range}

\item{transition_matrix}{if the sequences are markov chains, this is their transition matrix}

\item{distribution}{vector of probabilities in ascending score order (iid sequences). Note that names of the vector must be the associated scores.}

\item{method_limit}{limit length from which on computation-intensive exact calculation methods for p-value are replaced by approximative methods}

\item{score_extremes}{a vector with two elements: minimal score value, maximal score value}

\item{modelFunc}{function to create similar sequences. In this case, Monte Carlo is used to calculate p-value}

\item{simulated_sequence_length}{if a modelFunc is provided and the sequence happens to be longer than method_limit, the method karlinMonteCarlo
is used. This method requires the length of the sequences that will be created by the modelFunc for estimation of Gumble parameters.}

\item{...}{parameters for modelFunc}
}
\value{
A list object containing
\item{Local score}{local score...}
\item{p-value}{p-value ...}
\item{Method}{the method used for the calculus of the p-value}
}
\description{
Calculates local score and p-value for sequence(s) with integer scores.
}
\details{
This method picks the adequate p-value method for your input.\cr 
If no sequences are passed to this function, it will let you pick a FASTA file.\cr 
If this is the case, and if you haven't provided any score system 
(as you can do by passing a named list with the appropriate scores for each character),
the second file dialog which will pop up is for choosing a file containing the score 
(and if you provide an extra column for the probabilities, they will be used, too - see
section File Formats in the vignette for details).\cr 
The function then either uses empirical distribution based on your input - or if you provided
a distribution, then yours - to calculate the p-value based on the length of each of the sequences
given as input. \cr 
You can influence the choice of the method by providing the modelFunc argument. In this case, the
function uses exclusively simulation methods (monteCarlo, karlinMonteCarlo).  \cr 
By setting the method_limit you can further decide to which extent computation-intensive methods (daudin, exact_mc)
should be used to calculate the p-value.
Remark that the warnings of the localScoreC() function have be deleted when called by automatic_analysis() function
}
\examples{
# Minimal example
l = list()
seq1 = sample(-2:1, size = 3000, replace = TRUE)
seq2 = sample(-3:1, size = 150, replace = TRUE)
l[["hello"]] = seq1
l[["world"]] = seq2
automatic_analysis(l, "iid")
# Example with a given distribution 
automatic_analysis(l,"iid",scores=-3:1,distribution=c(0.3,0.3,0.1,0.1,0.2))
#Equivalent call
automatic_analysis(l,"iid",score_extremes=c(-3,1),distribution=c(0.3,0.3,0.1,0.1,0.2))
# forcing the exact method for the longest sequence
aa1=automatic_analysis(l,"iid")
aa1$hello$`method applied`
aa1$hello$`p-value`
aa2=automatic_analysis(l,"iid",method_limit=3000)
aa2$hello$`method applied`
aa2$hello$`p-value`
# Markovian example 
MyTransMat <-
matrix(c(0.3,0.1,0.1,0.1,0.4, 0.3,0.2,0.2,0.2,0.1, 0.3,0.4,0.1,0.1,0.1, 0.3,0.3,0.3,0.0,0.1, 
        0.1,0.3,0.2,0.3,0.1), ncol = 5, byrow=TRUE)
MySeq.CM=transmatrix2sequence(matrix = MyTransMat,length=150, score =-2:2)
MySeq.CM2=transmatrix2sequence(matrix = MyTransMat,length=110, score =-2:2)
automatic_analysis(sequences = list("x1" = MySeq.CM, "x2" = MySeq.CM2), model = "markov")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Seq31}
\alias{Seq31}
\title{Short protein sequence}
\format{
A character string with 31 characters "MLTITSYFGFLLAALTITSVLFIGLNKIRLI"
}
\source{
\url{https://www.uniprot.org/}
}
\usage{
data(Seq31)
}
\description{
A short protein sequence of 31 amino acids corresponding to Q09FU3.fasta query in UniProt Data base.
}
\examples{
data(Seq31)
Seq31
nchar(Seq31)
data(HydroScore) 
SeqScore <- CharSequence2ScoreSequence(Seq31,HydroScore)
SeqScore
localScoreC(SeqScore)$localScore
LS <- localScoreC(SeqScore)$localScore[1]
prob1 <- scoreSequences2probabilityVector(list(SeqScore))
daudin(local_score = LS, sequence_length = nchar(Seq31),
               score_probabilities = prob1,
               sequence_min = min(SeqScore),
               sequence_max = max(SeqScore))
score <- -5:5
prob2 <- c(0.15,0.15,0.1,0.1,0.0,0.05,0.15,0.05,0.2,0.0,0.05)
sum(prob2*score)
karlin(local_score = LS, sequence_length = nchar(Seq31),
       score_probabilities = prob2,
       sequence_min = min(SeqScore),
       sequence_max = max(SeqScore))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Seq219}
\alias{Seq219}
\title{Protein sequence}
\format{
A character string with 219 characters corresponding to P49755.fasta query in UniProt Data base.
}
\source{
\url{https://www.uniprot.org/}
}
\usage{
data(Seq219)
}
\description{
A protein sequence.
}
\examples{
data(Seq219)
Seq219
nchar(Seq219)
data(HydroScore)
seqScore=CharSequence2ScoreSequence(Seq219,HydroScore)
seqScore[1:30]
localScoreC(seqScore)$localScore
prob1 <- scoreSequences2probabilityVector(list(seqScore))
daudin(local_score = 52, sequence_length = nchar(Seq219),
               score_probabilities = prob1,
               sequence_min = min(seqScore),
               sequence_max = max(seqScore))
score <- -5:5
prob2 <- c(0.15,0.15,0.1,0.1,0.0,0.05,0.15,0.05,0.2,0.0,0.05)
daudin(local_score = 52, sequence_length = nchar(Seq219),
       score_probabilities = prob2,
       sequence_min = min(seqScore),
       sequence_max = max(seqScore))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{exact_mc}
\alias{exact_mc}
\title{Exact method for p-value [Markov chains]}
\usage{
exact_mc(local_score, m, sequence_length, score_values = NULL, prob0 = NULL)
}
\arguments{
\item{local_score}{Integer local score for which the p-value should be calculated}

\item{m}{Transition matrix [matrix object]. Optionality, rownames can be corresponding score values. m should be a transition matrix of an ergodic Markov chain.}

\item{sequence_length}{Length of the sequence}

\item{score_values}{A integer vector of sequence score values (optional). If not set, the rownames of m are used if they are numeric and set.}

\item{prob0}{Vector of probability distribution of the first score of the sequence (optional). If not set, the stationnary distribution of m is used.}
}
\value{
A double representing the probability of a local score as high as the one given as argument
}
\description{
Calculates the exact p-value for short numerical Markov chains. Memory usage and time computation can be too large for a high local score value and high score range (see details).
}
\details{
This method computation needs to allocate a square matrix of size local_score^(range(score_values)). This matrix is then exponentiated to sequence_length.
}
\examples{
mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
scoreValues <- -1:1
initialProb <- stationary_distribution(mTransition)
exact_mc(local_score = 12, m = mTransition, sequence_length = 100, 
        score_values = scoreValues, prob0 = initialProb)
exact_mc(local_score = 150, m = mTransition, sequence_length = 1000, 
         score_values = scoreValues, prob0 = initialProb)
rownames(mTransition) <- scoreValues
exact_mc(local_score = 12, m = mTransition, sequence_length = 100, prob0 = initialProb)
# Minimal specification
exact_mc(local_score = 12, m = mTransition, sequence_length = 100)
# Score valeus with "holes"
scoreValues <- c(-2, -1, 2)
mTransition <- matrix(c(0.2, 0.3, 0.5, 0.3, 0.4, 0.3, 0.2, 0.4, 0.4), byrow = TRUE, ncol = 3)
initialProb <- stationary_distribution(mTransition)
exact_mc(local_score = 50, m = mTransition, sequence_length = 100, 
        score_values = scoreValues, prob0 = initialProb)
}
\seealso{
\code{\link{monteCarlo}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{daudin}
\alias{daudin}
\title{Daudin [p-value] [iid]}
\usage{
daudin(
  local_score,
  sequence_length,
  score_probabilities,
  sequence_min = NULL,
  sequence_max = NULL,
  score_values = NULL
)
}
\arguments{
\item{local_score}{the observed local score}

\item{sequence_length}{length of the sequence}

\item{score_probabilities}{the probabilities for each score from lowest to greatest (Optionnaly with scores as names)}

\item{sequence_min}{minimum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{sequence_max}{maximum score (optional if \code{score_values} OR names(score_probabilities) is defined)}

\item{score_values}{vector of integer score values, associated to score_probabilities  (optional if
\code{sequence_min} and \code{sequence_max} OR names(score_probabilities) are defined)}
}
\value{
A double representing the probability of a local score as high as the one given as argument.
}
\description{
Calculates the exact p-value in the identically and independently distributed of a given local score, a sequence length that 'must not be too large' and for a given score distribution
}
\details{
Either \code{sequence_min} and \code{sequence_max} are specified as input, OR all possible score values in 
\code{score_values} vector ; one of this choice is required. 

Small in this context depends heavily on your machine. 
On a 3,7GHZ machine this means for daudin(1000, 5000, c(0.2, 0.2, 0.2, 0.1, 0.2, 0.1), -2, 3)
an execution time of ~2 seconds. This is due to the calculation method using matrix exponentiation
which takes times. The size of the matrix of the exponentiation is equal to a+1 with a the
local score value. The matrix must be put at the power n, with n the sequence length.
Moreover, it is known that the local score value is expected to be in mean of order log(n).
}
\examples{
p1 <- daudin(local_score = 4, sequence_length = 50, 
       score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
       sequence_min = -3, sequence_max = 2)
p2 <- daudin(local_score = 4, sequence_length = 50, 
       score_probabilities = c(0.2, 0.3, 0.1, 0.2, 0.1, 0.1), 
       score_values = as.integer(-3:2))
p1 == p2 # TRUE

prob <- c(0.08, 0.32, 0.08, 0.00, 0.08, 0.00, 0.00, 0.08, 0.02, 0.32, 0.02)
score_values <- which(prob != 0) - 6 # keep only non null probability scores
prob0 <- prob[prob != 0]             # and associated probability
p <- daudin(150, 10000, prob, sequence_min = -5, sequence_max =  5)
p0 <- daudin(150, 10000, prob0, score_values = score_values)
names(prob0) <- score_values
p1 <- daudin(150, 10000, prob0)
p == p0 # TRUE
p == p1 # TRUE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{karlinMonteCarlo}
\alias{karlinMonteCarlo}
\alias{karlinMonteCarlo_double}
\title{Monte Carlo - Karlin [p-value]}
\usage{
karlinMonteCarlo(
  local_score,
  sequence_length,
  simulated_sequence_length,
  FUN,
  ...,
  numSim = 1000,
  plot = TRUE,
  keepSimu = FALSE
)

karlinMonteCarlo_double(
  local_score,
  sequence_length,
  simulated_sequence_length,
  FUN,
  ...,
  numSim = 1000,
  plot = TRUE,
  keepSimu = FALSE
)
}
\arguments{
\item{local_score}{local score observed in a sequence.}

\item{sequence_length}{length of the sequence}

\item{simulated_sequence_length}{length of simulated sequences produced by}

\item{FUN}{function to simulate similar sequences with.}

\item{...}{parameters for FUN}

\item{numSim}{number of sequences to create for estimation
FUN}

\item{plot}{boolean value if to display plots for cumulated function, density
and linearization of cumulative density function}

\item{keepSimu}{Boolean, default to FALSE. If TRUE, the simulated local
scores are returned as the localScores element of the output list.}
}
\value{
If \code{keepSimu} is FALSE, returns a list containing:\tabular{ll}{
  \code{p_value} \tab Probability to obtain a local score with a value
  greater or equal to the parameter \code{local_score} \cr \tab \cr \code{K*}
  \tab Parameter \eqn{K^*} defined in Karlin and Dembo (1990) \cr \tab
  \cr \code{lambda} \tab Parameter \eqn{\lambda} defined in Karlin and
  Dembo (1990) \cr }
If \code{keepSimu} is TRUE, returns a list containing:\tabular{ll}{
    \code{p_value} \tab Probability to obtain a local score with a value greater or equal to the parameter \code{local_score} \cr
    \tab \cr
    \code{K*} \tab Parameter \eqn{K^*} defined in Karlin and Dembo (1990) \cr
    \tab \cr
    \code{lambda} \tab Parameter \eqn{\lambda} defined in Karlin and Dembo (1990) \cr
    \code{localScores} \tab Vector of size \code{numSim} containing the simulated local scores for sequence size of \code{simulated_sequence_length}
}
}
\description{
Estimates p-value of the local score based on a Monte Carlo
  estimation of Gumble parameters from simulations of smaller sequences with
  same distribution. Appropriate for great sequences with length > 10^3, for
  i.i.d and markovian sequence models.
}
\details{
The length of the simulated sequences is an argument specific to the
  function provided for simulation. Thus, it has to be provided also in the
  parameter \code{simulated_sequence_length} in the arguments of the "Monte
  Carlo - Karlin" function. It is a crucial detail as it influences precision
  and computation time of the result. Note that to get an appropriate
  estimation, the given average score must be non-positive. Be careful that
  the parameters names of the function \code{FUN} should differ from those of
  \code{karlinMonteCarlo} function. \cr Methods - Parameters \eqn{K^\star}
  and \eqn{\lambda} of Karlin and Dembo (1990) are estimated by a linear
  regression on the log(-log(cumulative distribution function of the local
  scores)) on shorter sequences (size \code{simulated_sequence_length}). The
  formula used are : \eqn{\hat{\lambda} = -\hat{b}} and \eqn{\hat{K^\star} =
  exp(\hat{a})/simulated\_sequence\_length} where \eqn{\hat{a}} is the
  intercept of the regression and \eqn{\hat{b}} is the slope of the
  regression. Then p-value is given by \eqn{p = exp(-K^\star * exp(-\lambda*x
  ))} where \eqn{x = local\_score - \log(sequence\_length)/\lambda}. \cr The
  density plot produced by \code{plot == TRUE} depends on the type of the
  simulated local scores: if they are integer, a barplot of relative
  frequency is used, else \code{plot(density(...))} is used. \cr This
  function calls \code{\link{localScoreC}} which type of the output depends
  on the type of the input. To be efficient, be aware to use a simulating
  function \code{FUN} that return a vector of adequate type ("integer" or
  "numeric"). Warning: in R, \code{typeof(c(1,3,4,10)) == "double"}. You can
  set a type of a vector with \code{mode()} or \code{as.integer()} functions
  for example. \cr \code{karlinMonteCarlo_double()} is deprecated. At this
  point, it is just a call to \code{karlinMonteCarlo()} function.
}
\examples{
\donttest{
mySeq <- sample(-7:6, replace = TRUE, size = 100000)
#MonteCarlo taking random sample from the input sequence itself
karlinMonteCarlo(local_score = 160, sequence_length = 100000,
               simulated_sequence_length = 1000,
               FUN = function(x, sim_length) {
                        return(sample(x = x,
                               size = sim_length,
                               replace = TRUE))
                     },
               x = mySeq,
               sim_length = 1000,
               numSim = 1000)
}
\donttest{
#Markovian example (longer computation)
MyTransMat_reels <-  matrix(c(0.3, 0.1, 0.1, 0.1, 0.4,
                              0.2, 0.2, 0.1, 0.2, 0.3,
                              0.3, 0.4, 0.1, 0.1, 0.1,
                              0.3, 0.3, 0.1, 0.2, 0.1,
                              0.2, 0.1, 0.2, 0.4, 0.1),
                              ncol = 5, byrow=TRUE)
karlinMonteCarlo(local_score = 18.5, sequence_length = 200000,
                 simulated_sequence_length = 1500,
                 FUN = transmatrix2sequence,
                 matrix = MyTransMat_reels,
                 score =c(-1.5,-0.5,0,0.5,1), length = 1500,
                 plot=TRUE, numSim = 1500)
}
}
\seealso{
\code{\link{monteCarlo}} \code{\link{localScoreC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMethods.R
\name{monteCarlo}
\alias{monteCarlo}
\alias{monteCarlo_double}
\title{Monte Carlo method [p-value]}
\usage{
monteCarlo(local_score, FUN, ..., plot = TRUE, numSim = 1000, keepSimu = FALSE)

monteCarlo_double(
  local_score,
  FUN,
  ...,
  plot = TRUE,
  numSim = 1000,
  keepSimu = FALSE
)
}
\arguments{
\item{local_score}{local score observed in a segment.}

\item{FUN}{function to simulate similar sequences with.}

\item{...}{parameters for FUN}

\item{plot}{boolean value if to display plots for cumulated function and density}

\item{numSim}{number of sequences to generate during simulation}

\item{keepSimu}{Boolean, default to FALSE. If TRUE, the simulated local scores are returned as the localScores element of the output list.}
}
\value{
If \code{keepSimu} is FALSE, returns a numeric value corresponding to the probability to obtain a local score with value greater or equal to the parameter \code{local_score}. \cr
If \code{keepSimu} is TRUE, returns a list containing:\tabular{ll}{
    \code{p_value} \tab Floating value corresponding to the probability to obtain a local score with a value greater or equal to the parameter \code{local_score} \cr
    \code{localScores} \tab Vector of size \code{numSim} containing the simulated local scores
}
}
\description{
Calculates an empirical p-value based on Monte Carlo simulations.
Perfect for small sequences (both markov chains and identically and independently distributed) with length ~ 10^3.
}
\details{
Be careful that the parameters names of the function FUN should differ from those of monteCarlo function.\cr
The density plot produced by \code{plot == TRUE} depends on the type of the simulated local scores: 
if they are integer, a barplot of relative frequency is used, else \code{plot(density(...))} is used. \cr
This function calls \code{\link{localScoreC}} which type of the output depends on the type of the input. 
To be efficient, be aware to use a simulating function \code{FUN} that return a vector of adequate type ("integer" or "numeric"). Warning: in R, \code{typeof(c(1,3,4,10)) == "double"}. You can set a type of a vector with \code{mode()} or \code{as.integer()} functions for example. \cr 
\code{monteCarlo_double()} is deprecated. At this point, it is just a call to \code{monteCarlo()} function.
}
\section{Functions}{
\itemize{
\item \code{monteCarlo_double()}: Monte-Carlo function for double [deprecated]

}}
\examples{
\donttest{
monteCarlo(120, FUN = rbinom, n = 100, size = 5, prob=0.2)
}
\donttest{
mySeq <- sample(-7:3, replace = TRUE, size = 1000)
monteCarlo(local_score = 18, FUN = function(x) {return(sample(x = x, 
           size = length(x), replace = TRUE))}, x = mySeq)
}
\donttest{
#Examples of non integer score function
mySeq2 <- sample(-7:6 - 0.5, replace = TRUE, size = 1000)
monteCarlo(local_score = 50.5, FUN = function(x) {return(sample(x = x, 
           size = length(x), replace = TRUE))}, x = mySeq2)
}
\donttest{
#Examinating simulated local scores
mySeq2 <- sample(-7:6, replace = TRUE, size = 1000)
simu <- monteCarlo(local_score = 50.5, FUN = function(x) {return(sample(x = x, 
           size = length(x), replace = TRUE))}, x = mySeq2, keepSimu = TRUE)
hist(simu$localScores)
}
\donttest{
# Markovian example
MyTransMat <-
    matrix(c(0.3,0.1,0.1,0.1,0.4, 0.2,0.2,0.1,0.2,0.3, 0.3,0.4,0.1,0.1,0.1, 0.3,0.3,0.1,0.0,0.3,
             0.1,0.1,0.2,0.3,0.3), ncol = 5, byrow=TRUE)
monteCarlo(local_score = 50,
          FUN = transmatrix2sequence, matrix = MyTransMat,
          length=150, score = c(-2,-1,0,2,3), plot=FALSE, numSim = 5000)
}
}
\seealso{
\code{\link{karlinMonteCarlo}} \code{\link{localScoreC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SJSyndrome}
\alias{SJSyndrome}
\title{Stevens-Johnson syndrome data}
\format{
A data.frame of 824 lines, each describing a syndrome appearance described by 15 covariates:
Case ID,
Initial FDA Received Date,
days since last fda,
Event Date,
Latest FDA Received Date,
Suspect Product Names,
Suspect Product Active Ingredients,
Reason for Use,
Reactions,
Serious,
Outcomes,
Sex,
Patient Age,
Sender,
Concomitant Product Names
The third column correspond to the number of days between two adverse events.
}
\source{
FDA open data.
}
\usage{
data(SJSyndrome)
}
\description{
The Stevens-Johnson syndrome is an acute and serious dermatological disease due to a drug allergy. 
The syndrome appearance is life-threatening emergency. They are very rare, around 2 cases per million people per year.
}
\examples{
data(SJSyndrome)
summary(SJSyndrome)
}
\keyword{datasets}
